#ifndef TXCURRENTALGORITHM_H
#define TXCURRENTALGORITHM_H

#include "AnalysisAlgorithm.h"

namespace SctAnalysis {
    /**
       Algorithm for TxCurrent Test.
    */
class TxCurrentAlgorithm : public AnalysisAlgorithm {
public:
  // categorise the data from any particular TxCurrent value:
  typedef enum {undecided,white,black,blackAndWhite,coloured} Category;
public:
  static std::string categoryToString(const Category & category) {
    return
      category==undecided     ? "undecided" :
      category==white         ? "white" :
      category==black         ? "black" :
      category==blackAndWhite ? "blackAndWhite" :
      category==coloured      ? "coloured" :
      "couldNotFigureCategoryOut";
  };
public:
    virtual bool canAnalyze() const;
    virtual void analyze();
    virtual void loadData();
    virtual shared_ptr<SctData::TestResult> createTestResult() const;
    virtual boost::shared_ptr<AnalysisAlgorithm> clone(const TestData& testData, const string& moduleName) const throw();
    /** 
	constructor 
     */
    TxCurrentAlgorithm(const TestData& testData, const string& moduleName, const AnalysisAlgorithm& alg) throw() : AnalysisAlgorithm(testData, moduleName, alg) {}
    /** 
	destructor 
     */
    virtual ~TxCurrentAlgorithm() throw() {}
    
private:
    TxCurrentAlgorithm() {}
    static bool inMap;
};
}

#endif //TXCURRENTALGORITHM_H
