#ifndef SCT_ARCHIVEFILE_IOMANAGERARCHIVEFILE_H
#define SCT_ARCHIVEFILE_IOMANAGERARCHIVEFILE_H

#include "Sct/Archive/IOManagerArchive.h"
#include "Sct/IoExceptions.h"
#include "Sct/LogicErrors.h"
#include "SctData/ResultHeader.h"
#include <mysql.h>
#include "CalibrationController/IS/TestData.h"
//#include "sqlplus.hh"

class Sct::Serializable;
//class MysqlQuery;

namespace SctArchiving {
  /** exception thrown when queries go wrong */
class MysqlException : public Sct::IoException{
public:
  MysqlException(const string& msg, const string& file, int line);
  MysqlException(const string& msg, const string& query, const string& file, int line);
  ~MysqlException() throw();
private:
  MysqlException();
  std::string query;
};

class ZlibStringCompressor;
  /**
     This IOManager reads from and writes to a database.
     It is expected to be used in archival, and so also makes use of streamer 
     versioning by inheriting from IOManagerStreamerVersioned.
     Uses a <A HREF="http://www.mysql.com">mysql</a> back-end.
   */
class IOManagerDB : public Sct::Archive::IOManagerArchive {
public:
  /** 
      retrieve access to singleton 
  */
  static IOManagerDB& instance();
  /** 
      write using default streamers 
  */
  virtual void write(const Sct::Serializable& ob, const Sct::IOParams* params = 0) const;
  /** 
      read using appropriate streamers 
  */
  virtual shared_ptr<Sct::Serializable> read(const string& name, const Sct::IOParams* params = 0) const;
  virtual ~IOManagerDB();
  /** 
      get some status information 
   */
  virtual std::string status() const;
  /** 
      Add a run - gets location identifier from getLocationIdentifier
      
  */
  virtual void addRun(unsigned long runNumber);
  /**
     Returns true if run already exists
   */
  virtual bool runExists(unsigned long runNumber);
  /** 
      Add a test @param alsoAddScans if this is true, it will loop over the scans in the test
      adding them too
   */
  virtual void addTest(const TestData& testdata, bool alsoAddScans);
  /**
     check if scan exists
   */
  virtual bool testExists(const TestData& testdata);
  /**
      Add a scan. Identifies correct Test from start time of TestData.
  */  
  virtual void addScan(const TestData& testdata, unsigned scanNumber);
  /**
     check if scan exists
  */
  virtual bool scanExists(const TestData& testdata, unsigned scanNumber);

  /** get string identifying data set e.g. Barrel3_Cold_Oxford */
  virtual std::string getLocationName();
  /** */
  virtual void setLocationName(const std::string& locationName);
  /** conversion just removes 'T' from isotime */
  static std::string convertToMysqlDateTimeFormat(const std::string& isotime);

 private:
  /* get the LocationID for this location */
  //int getLocationID();

  /** throw MysqlException if there is an error in the mysql object */
  void checkForErrors(const string& msg, const string& file, int line);

  /** throw MysqlException if there is an error in the mysql object - listing query */
  void checkForErrors(const string& msg, const string& query, const string& file, int line);

  IOManagerDB();  ///< constructor.
  //MysqlQuery getQuery() const; ///< get a null query.
  static IOManagerDB* s_man; ///< pointer to singleton
  /**
     keep a string compressor handy to zip and unzip BLOB data.
  */
  boost::shared_ptr<ZlibStringCompressor> m_compressor;
  //AJB ONLY FOR MYSQL++ boost::shared_ptr<Connection> m_connection; ///< pointer to the database connection

  MYSQL* m_mysql; ///< the main MYSQL object
  
  /**
     get the header from a TestResult or ScanResult
  */
  static const SctData::ResultHeader& getHeader( const Sct::Serializable& ob);
  /**
     change query to prepare to insert ob.
  */
  //void prepareInsertion(const Sct::Serializable& ob, MysqlQuery& q) const;
  /**
     make the "`WHERE'" part of the query matching the name by building a 
     IOName from it.
  */
  //static std::string getQueryMatching(const string& name);
  /**
     find the name of the table which the ob should to into.
   */
  static std::string getTable(const Sct::Serializable& ob);

  std::string m_locationName;
};
}
#endif //SCT_ARCHIVEFILE_IOMANAGERARCHIVEFILE_H
