#include "../IOManagerDB.h"
#include "IOManager_Connect_Test.h"
namespace SctArchiving{

void checkCreation(){
  IOManagerDB& man = IOManagerDB::instance();
  man.setLocationName("MyLoc");
}  

void checkDateTimeParsing(){
  std::string input="20050301T121347";
  std::string output=IOManagerDB::instance().convertToMysqlDateTimeFormat(input);
  BOOST_CHECK_EQUAL(output, "20050301121347");
}


void checkThatRunExists(){
  IOManagerDB& man = IOManagerDB::instance();
  unsigned run=1;
  if (!man.runExists(run)) {
    std::cout << "adding run " << run << std::endl;
    man.addRun(run);
  }
  BOOST_CHECK(man.runExists(run));
}

void checkDosentWorkIfLocationIsWrong(){
  IOManagerDB& man = IOManagerDB::instance();
  std::string originalLocation=man.getLocationName();
  man.setLocationName("this_is_a_very_unlikely_location_name");
  BOOST_CHECK_THROW(man.addRun(1), MysqlException);
  man.setLocationName(originalLocation);
}

void checkCanAddTest(){
  TestData testdata;
  testdata.testName="WeirdTest";
  testdata.startTime="20050301T121347";
  testdata.endTime="20050301T121356";
  testdata.runNumber=1;

  IOManagerDB& man = IOManagerDB::instance();
  if (!man.testExists(testdata)){
    BOOST_CHECK(man.runExists(testdata.runNumber));
    man.addTest(testdata, false);
  }
  BOOST_CHECK(man.testExists(testdata));
}

void checkCanAddTestAndScans(){
  TestData testdata;
  testdata.testName="WeirdTest";
  testdata.startTime="20050301T120700";
  testdata.endTime="20050301T120800";
  testdata.runNumber=1;
  testdata.startScanNumber=101;
  testdata.nScans=2;

  IOManagerDB& man = IOManagerDB::instance();
  if (!man.testExists(testdata)){
    BOOST_CHECK(man.runExists(testdata.runNumber));
    man.addTest(testdata, true);
  }
  BOOST_CHECK(man.testExists(testdata));
  BOOST_CHECK(man.scanExists(testdata,101));
  BOOST_CHECK(man.scanExists(testdata,102));
}

IOManager_Connect_Test::IOManager_Connect_Test(){
    add (BOOST_TEST_CASE(&checkCreation));
    add (BOOST_TEST_CASE(&checkDateTimeParsing));
    add (BOOST_TEST_CASE(&checkThatRunExists));
    add (BOOST_TEST_CASE(&checkDosentWorkIfLocationIsWrong));
    add (BOOST_TEST_CASE(&checkCanAddTest));
    add (BOOST_TEST_CASE(&checkCanAddTestAndScans));
}
}
