#ifdef linux
#undef linux
#endif

#include "SctApiCall.h"
#include "RunController.h"
#include "SctApiAccessException.h"
#include "CalibrationControllerImpl.h"
#include "ipc/CalibrationController.h"
#include "Sct/SctNames.h"
#include "Sct/StringStreamer.h"
#include "Sct/Exception.h"
#include "Sct/LogicErrors.h"
#include <rc/RunParams.h>
#include <rc/Controller.h>
#include <ipc/partition.h>
#include <is/isinfo.h>
#include <is/isinfotmpl.h> // for ISInfoInt
#include <pmg/pmg_initSync.h>
#include <iostream>
#include <fstream>
//inserted
#include "cmdl/cmdargs.h"
#include "dal/implementation.h"
#include "config/Configuration.h"
#include "dal/util.h"

using namespace std;
using namespace Sct;


inline std::string str_lkuvysjhgfdksjhd(const char * word)
{
   return word ? word : "";
} 

int main(int argc, char** argv) {
   using namespace SctCalibrationController;

    cout << "Starting Calibration controller" << endl;
    setExceptionHandlers(argv[0]);
       
    IPCCore::init(argc,argv);
    SctNames::Mrs() << "CCRC" << MRS_TEXT("CCRC::main done IPCCore::init") << MRS_DIAGNOSTIC << ENDM;    

    //inserted
    //CmdArgStr    partition  ('p',"partition","partition",  "Name of the IPC partition (default $TDAQ_PARTITION)");
    CmdArgStr    name       ('n',"name",     "controller", "Name of the controller"/*,CmdArg::isREQ*/); // at the moment we cheat and ignore this one!
    CmdArgStr    data       ('d',"data",     "db-name",    "Configuration DB name");  //must have this argument unless you specify N on the command line!
    //CmdArgStr    rules      ('R',"rules",    "rules",      "CLIPS rulesbase (default Controller:TransitionTable)");
    //CmdArgStr    server     ('s',"server",   "is-server",  "Name of the IS server (default RunCtrl)");
    //CmdArgFloat  interval   ('I',"interval", "interval",   "Time interval to publish controller state");
    //CmdArgInt    verbosity  ('v',"verbosity","level",      "Output level (default 0 )");
    CmdArgBool   oks        ('O',"oks"                     ,"Load Configuration from OKS");
    //CmdArgBool   rdb        ('Q',"rdb"                     ,"Load Configuration from RDB");
    CmdArgBool   none       ('N',"none"                    ,"Do not load Configuration");
    // should have one of 'O' or 'N'

    //inserted
    CmdLine  cmd(*argv,&name,&data,&oks,&none,NULL);
    
    //inserted
    cmd.description("The SCT Calibration Controller.");

    //inserted
    CmdArgvIter  argvIter(--argc,++argv);
    if (cmd.parse(argvIter)) 
      {
        std::cerr << "ERROR in " << __FILE__ << " at line " << __LINE__ << " when parsing command line args." << std::endl ;
        return EXIT_FAILURE;
      }

    //inserted
    std::auto_ptr<ConfigurationImpl> configImpl
      (
       none ? 0 : dal::create_config_implementation
       (   
        oks ? dal::OksDB_Implementation :
        //rdb ? dal::RdbDB_Implementation :
        dal::DefaultDB_Implementation
        )
       );
    
    //inserted
    std::auto_ptr<Configuration> config(none ? 0 : new Configuration(str_lkuvysjhgfdksjhd(data),configImpl.get()));
    if (config.get()) 
      {
        config->register_convert_function(dal::SubstituteVariables::from_environment);
      } 
    

    RunControl& user = RunControl::instance();
    cout << "Instance created" << endl;
        
    //rc.rc_start("CalibrationController", "SCT");
    RC::Controller controller(SctNames::getPartition(), //partition
                              user,  // user-routines impl
                              "CalibrationController", // name
                              "", // rules
                              config.get(), // erm??
                              "RunCtrl" // "is server" (maybe should be something else, but this is the rc_empty_controller default
                              );
      
    pmg_initSync();
    controller.run();
    cout << "All over" << endl;

    return 0;
}

namespace SctCalibrationController {

RunControl::RunControl() : cc(CalibrationControllerImpl::initialize(*this)) {
}
    
RunControl& RunControl::instance() {
    static RunControl rc;
    return rc;
}

int RunControl::getRunNumber() {
  if (m_isInfoDictionary.get()) {
    
    RunParams runParams; 
    runParams.run_number = defaultRunNumber;
    SctNames::Mrs() << "RC_DEBUG" << MRS_TEXT(Sct::StringStreamer<< "Before I make any attempt to see the run number my runParams.run_number holds defaultRunNumber="<< runParams.run_number)<< MRS_INFORMATION << ENDM;
    const ISInfo::Status isStatus = m_isInfoDictionary->getValue("RunParams.RunParams",runParams);
    if (isStatus!=ISInfo::Success){
      SctNames::Mrs() << "RC_DEBUG" << MRS_TEXT("The attempt to get the run params was unsuccessful.  Could not determine run number.  Using default one (probably wrong!)")<< MRS_ERROR << ENDM;
      return defaultRunNumber;
    }
    SctNames::Mrs() << "RC_DEBUG" << MRS_TEXT(Sct::StringStreamer<<"After I make any attempt to see the run number my runParams.run_number holds "<< runParams.run_number)<< MRS_INFORMATION << ENDM;
    return runParams.run_number;
  } else {
    SctNames::Mrs() << "RC_getRunNumber" << MRS_TEXT("The CalibrationController's RunController failed to find the Information Service so a default run-number is being returned in place of the correct one!") << MRS_INFORMATION << ENDM;
    return defaultRunNumber;
  };
};
  
bool RunControl::loadAction () {
    api = SctNames::getPartition().lookup<Sct_SctApi::SctApiIPC>(Sct_SctApi::SctApiIPC::instanceName);
    if (CORBA::is_nil(api)) {
	SctNames::Mrs() << "RC_INIT" << MRS_TEXT("RunController failed to find SctApi") << MRS_ERROR << ENDM;
	return false;
    }
    cc.setApi(api);

    SctNames::Mrs() << "RC_loadAction" << MRS_TEXT("Trying to get isInfoDictionary ...")<< MRS_INFORMATION << ENDM;
    m_isInfoDictionary = boost::shared_ptr<ISInfoDictionary>(new ISInfoDictionary(SctNames::getPartition()));
    if (!m_isInfoDictionary.get()) {
      SctNames::Mrs() << "RC_INIT" << MRS_TEXT(Sct::StringStreamer<< "RunController failed to find Information Server in partition "<< SctNames::getPartition().name()<<".  This means numbers like the run number will be set to default (probably incorrect) values!") << MRS_ERROR << ENDM;
    } else {
      SctNames::Mrs() << "RC_INIT" << MRS_TEXT(Sct::StringStreamer<< "RunController succeeded in finding Information Server in partition "<< SctNames::getPartition().name()<< ".  This means numbers like the run number should be correct.") << MRS_INFORMATION << ENDM;
    };
    
    try { 
      if (CORBA::is_nil(api)) { throw InvalidArgumentError("Null pointer to SctApi", __FILE__, __LINE__); } 
      try {
	const int rn=getRunNumber();        
	SctNames::Mrs() << "RC_INIT" << MRS_TEXT(Sct::StringStreamer<< "RunController found runNumber="<<rn) 
			<< MRS_INFORMATION << ENDM;
	(api)->initialiseAll(rn); 
      } catch (const CORBA::Exception & exe) { 
	throw raiseSctApiException(exe, "SctApi failed to initialiseAll", __FILE__, __LINE__);
      } 
    } catch(SctApiAccessException& e) {
      e.sendToMrs(MRS_ERROR);
      return false;
    }
    SctNames::Mrs() << "RC_INIT" << MRS_TEXT("CC RunController initialised SctApi") << MRS_DIAGNOSTIC << ENDM;
    
    return true;
}

bool RunControl::configureAction() {
    cout << "CC configureAction" << endl;
    try {
      // send PHYSICS to all
        APICALL(api, configureAllModules(), "SctApi failed to configureAllModules")
    } catch(SctApiAccessException& e) {
        e.sendToMrs(MRS_ERROR);
        return false;
    }
    SctNames::Mrs() << "RC_CONFIG" << MRS_TEXT("CC RunController configured SctApi") << MRS_DIAGNOSTIC << ENDM;
    return true;
}

  /// AJB was startAction
bool RunControl::prepareAction() {
    cout << "CC prepare action" << endl;
    if (CORBA::is_nil(api)) {
	cout << "Can't enter PREPARE as we don't have the api" << endl;
	return false;
    }

    RunParams runParams;
    ISInfoDictionary& is = SctNames::getISDictionary();
    ISInfo::Status isStatus;

    isStatus = is.findValue("RunParams.RunParams", runParams);

    if (isStatus!=ISInfo::Success){
      SctNames::Mrs() << "RC_PREPARE" << MRS_TEXT("CalibrationController failed to find RunParameters") << MRS_ERROR << ENDM;
      return false;
    }

    try {
        APICALL(api, setRunNumber(runParams.run_number), "Failed to set run number")
        APICALL(api, setScanNumber(0), "Failed to set scan number")
        cout << "New Run, setting run number and scan number: " << runParams.run_number << "  " << 0 << endl;
        cc.takeControl(runParams.run_number);
    } catch (SctApiAccessException& e) {
        e.sendToMrs(MRS_ERROR);
        return false;
    }
    return true;
}

  /// was exitRunning
bool RunControl::stopEFAction() {
    cout << "CC exit stopFE action" << endl;
    cc.giveupControl();
    return true;
}

bool RunControl::unloadAction () {
    cout << "CC unloadaction" << endl;

    try {
        APICALL(api, shutdownAll(), "Could not shutdown all")
    } catch (SctApiAccessException& e) {
        e.sendToMrs(MRS_ERROR);
        return false;
    }
    cc.reset();

    return true;
}

bool RunControl::resetAction () {
    cout << "CC resetAction" << endl;
    cc.reset();
    return true;
}

void RunControl::sctApiError(const string& reason) {
  SctNames::Mrs() << "CC_API_ERROR" << MRS_PARAM<const char*>("reason",reason.c_str()) 
		  << MRS_TEXT("Calibration Controller found error with SctApi") << MRS_WARNING << ENDM;
    this->raiseError(reason);
    cc.giveupControl();
}

}
