#ifndef SCTCALIBRATIONCONTROLLER_MARKSPACERATIOTEST_H
#define SCTCALIBRATIONCONTROLLER_MARKSPACERATIOTEST_H

#include "../ipc/TestRequest.h"
#include "../ipc/CalibrationController.h"
#include "../ipc/ScanLibrary.h"
#include "../ipc/CalibrationController.hh"

#include "../SctApiAccessException.h"
#include "../SctApiCall.h"
#include "../CalibrationControllerImpl.h"
#include "Sct/LogicErrors.h"
#include "Sct/StdExceptionWrapper.h"
#include "SctApi/extraScans.h"

#include "sctConfIPC/configipc.h"

#include <CommonWithDsp/ABCD/ABCDscans.h>
#include <iostream>
#include <vector>

using namespace Sct;
using namespace std;
using namespace SctCalibrationController::Ipc;
using namespace SctCalibrationController;

namespace SctCalibrationController {
namespace Scripts {
    
class MarkSpaceRatioTest : public TestRequestImpl {
private:
   static const bool registeredInTestLibrary;

public:
   static Sct_CalibrationController::TestRequest_ptr instance() {
	static MarkSpaceRatioTest rt;
	return rt._this();
    }
   
   virtual void startTestRealImpl(Sct_SctApi::SctApiIPC_ptr the_api, string& name, unsigned short& testVariable, unsigned long& nScans, vector<double>& testPoints) {	
     this->api = Sct_SctApi::SctApiIPC::_duplicate(the_api);
     // No masked channels
     name = "MarkSpaceRatioTest";
     testVariable = ST_TX_MARKSPACE;
     
     for (unsigned i=0; i<31; i+=2){
       register_values.push_back(i);
     }

     nScans = 0;

     // do twice - before and after bit-flip.
     for (unsigned i=0; i<2; ++i){
       for (vector<int>::const_iterator value=register_values.begin(); 
	    value!= register_values.end(); ++value){
	 ++nScans;
	 testPoints.push_back(*value);
       }
     }
     Sct_SctApi::BankList banks;
     banks.length(1);
     banks[0]=Sct_SctApi::SCAN_CONFIG;
     APICALL(api, modifyABCDVar(ST_FEEDTHROUGH, 0), "MarkSpaceRatioTest:startTest error setting feedthrough on API");
     APICALL(api, setABCDModules(banks),"MarkSpaceRatioTest:startTest error setting feedthrough on ROD"); 
     APICALL(api, sendABCDModules(Sct_SctApi::SCAN_CONFIG), "MarkSpaceRatioTest:startTest error setting feedthrough to MODULES");
   }
   
   virtual Sct_CalibrationController::ScanRequest_ptr getNextScanRealImpl(unsigned long index) {
     Sct_CalibrationController::ScanLibrary_var sl = CalibrationControllerImpl::instance()->getScanLibrary();
     const bool issueSoftReset = false;
     Sct_CalibrationController::ScanRequest_ptr request = sl->rawScan(nTriggers, issueSoftReset);

     request->setWidthCorba(1024);
     request->setConfigureModulesCorba(false);

     Sct_SctApi::Scan_var scan = request->getScanCorba();
     Sct_SctApi::Trigger_var t;

     APICALL(scan, setOption(Sct_SctApi::Scan::enableDataMode, 0), "MarkSpaceRatioTest : failed to disable dataMode")
     APIRETCALL(scan, t, getTrigger1(), "Failed to get Trigger1")       
     APICALL(t, singleL1A(), "Failed to set trigger type")

     float msr = register_values[index%register_values.size()];
     APICALL(api, modifyABCDVar(ST_TX_MARKSPACE, msr), "MarkSpaceRatioTest coulndt set MSR");

     APICALL(scan, configure(ST_RX_DELAY, 0, 24, 1), "Failed to set Scan information")

     // half-way through send a bit-flip
     if (index==register_values.size()) sendOddNumberOfBits();

     return request;
   }
    
    virtual bool canFeedbackRealImpl() {
	return true;
    }
    
    virtual void endTestRealImpl() {
      APICALL(api, modifyABCDVar(ST_FEEDTHROUGH, 1), "MarkSpaceRatioTest:endTest error unsetting feedthrough on API");
      APICALL(api, setABCDModules(Sct_SctApi::SCAN_CONFIG),"MarkSpaceRatioTest:endTest error unsetting feedthrough on ROD"); 
      APICALL(api, sendABCDModules(Sct_SctApi::SCAN_CONFIG), "MarkSpaceRatioTest:endTest error unsetting feedthrough to MODULES");

      api = Sct_SctApi::SctApiIPC::_nil();
      register_values.clear();
    }

    virtual void sendOddNumberOfBits(){
      using std::list;
      try{
	boost::shared_ptr<SctConfiguration::Configuration> config(new SctConfiguration::ConfigIPC);
	
	Sct_SctApi::Trigger_var trig;    
	APIRETCALL(api, trig, createTrigger(), "MarkSpaceRatioTest: Couldn't create trigger")
	APICALL(trig, soft(), "Failed to set trigger type")

	//send to all modules
	list<unsigned> partitions=config->listPartitions();
	for (list<unsigned>::const_iterator p=partitions.begin(); p!=partitions.end(); ++p){
	  list<unsigned> crates=config->listCratesInPartition(*p);
	  for (list<unsigned>::const_iterator c=crates.begin(); c!=crates.end(); ++c){
	    list<unsigned> rods=config->listRodsInCrate(*p, *c);
	    for (list<unsigned>::const_iterator r=rods.begin(); r!=rods.end(); ++r){
	      std::cout << "MarkSpaceRatio: sendTrigger ("<<*p<<", "<<*c<<", "<<*r<<", soft)"<<std::endl;
	      APICALL(api, sendTrigger(*p, *c, *r, trig), "MarkSpaceRatioTest Could not send trigger")
	    }
	  }
        }
       }catch (std::exception& e){
	 throw Sct::StdExceptionWrapper(e, __FILE__, __LINE__);
       }
    }

private:
    vector<int> register_values;
    MarkSpaceRatioTest() {}
    Sct_SctApi::SctApiIPC_var api;    
    static const int nTriggers = 1;
};
}
}
#endif //SCTCALIBRATIONCONTROLLER_MARKSPACERATIOTEST_H
