#ifndef STROBEDELAYTEST_H
#define STROBEDELAYTEST_H

#include "../ipc/TestRequest.h"
#include "../ipc/CalibrationController.h"
#include "../ipc/ScanLibrary.h"
#include "../ipc/CalibrationController.hh"
#include "../CalibrationControllerImpl.h"

#include "../SctApiAccessException.h"
#include "Sct/LogicErrors.h"
#include "../SctApiCall.h"
#include "Sct/BugReport.h"

#include <CommonWithDsp/ABCD/ABCDscans.h>
#include <iostream>
#include <vector>

using namespace Sct;
using namespace std;
using namespace SctCalibrationController::Ipc;
using namespace SctCalibrationController;

namespace SctCalibrationController {
namespace Scripts {

  /** Does a single StrobeDelay scan with feedback */
    
class StrobeDelayTest : public TestRequestImpl {
private:
   static const bool registeredInTestLibrary;

public:	
    static Sct_CalibrationController::TestRequest_ptr instance() {
	static StrobeDelayTest rc;
	return rc._this();
    }
    
    virtual void startTestRealImpl(Sct_SctApi::SctApiIPC_ptr api, 
			   string& name, 
			   unsigned short& testVariable, 
			   unsigned long& nScans, 
			   vector<double>& testPoints) {	
      Sct::BugReport("BUG01", "StrobeDelayTest.cpp","startTest 1");
       this->api = Sct_SctApi::SctApiIPC::_duplicate(api);

        // Compression mode 2
        APICALL(api, modifyABCDVar(ST_COMPRESSION, 2), "StrobeDelayTest::startTest")

        // Edge detect on
        APICALL(api, modifyABCDVar(ST_EDGE_DETECT, 1), "StrobeDelayTest::startTest")

        // No masked channels
        APICALL(api, modifyABCDVar(ST_SEND_MASK, 0), "StrobeDelayTest::startTest")

        // Set threshold to 2fC (based on previous configuration)
        APICALL(api, modifyABCDVar(ST_QTHR, 2), "StrobeDelayTest::startTest")

        // 4.00 fC input charge
        APICALL(api, modifyABCDVar(ST_VCAL, 40), "StrobeDelayTest::startTest")

	name = "StrobeDelayTest";
	testVariable = ST_ELAPSED;
	nScans = 1;

	testPoints.push_back(0);
      Sct::BugReport("BUG01", "StrobeDelayTest.cpp","startTest 2");
    }
    
    virtual Sct_CalibrationController::ScanRequest_ptr getNextScanRealImpl(unsigned long index) {
     Sct::BugReport("BUG01", "StrobeDelayTest.cpp","getNextScan 1/3");
	Sct_CalibrationController::ScanLibrary_var sl = CalibrationControllerImpl::instance()->getScanLibrary();
     Sct::BugReport("BUG01", "StrobeDelayTest.cpp","getNextScan 2/3");
     Sct_CalibrationController::ScanRequest_ptr ans = sl->strobeDelayScan(100);
     Sct::BugReport("BUG01", "StrobeDelayTest.cpp","getNextScan 3/3");
     return ans;
    }
    
    virtual bool canFeedbackRealImpl() {
     Sct::BugReport("BUG01", "StrobeDelayTest.cpp","canFeedback");
	return true;
    }    
    
    virtual void endTestRealImpl() {
      Sct::BugReport("BUG01", "StrobeDelayTest.cpp","endTest 1");
	api = Sct_SctApi::SctApiIPC::_nil();
       Sct::BugReport("BUG01", "StrobeDelayTest.cpp","endTest 2");
   }
    
private:
    StrobeDelayTest() {}
    Sct_SctApi::SctApiIPC_var api;    
};
}
}
#endif //STROBEDELAYTEST_H
