/*
 * ExitPrompt.java
 *
 */

package GuiComponents.Panels;
import java.util.Vector;
import java.util.Map;
import java.util.HashMap;
import java.util.regex.*;
import javax.swing.Timer;
import GuiComponents.System.*;
import is.*;
import ServiceInformation.*;

/**
 *
 * @author  robinson
 */
public class ServiceStatusGUI extends javax.swing.JPanel implements InfoListener {
    private myTableAdapter serverInfoTable;
    private javax.swing.JTable table;
    private javax.swing.JButton cancelButton;
    private javax.swing.JLabel jLabel1,jLabel2;
    private javax.swing.JScrollPane jScrollPane1;
    
    private final String serverName = "ServiceStatus";
    
    public final static int TIME_INTERVAL = 2000;
    private Timer timer;
    private boolean subscribed=false;

    private Pattern oPattern;

    Map serverMap;

    public static void main(String args[]) {

        javax.swing.JFrame frame = new javax.swing.JFrame("Service Status");
        ServiceStatusGUI gui = new ServiceStatusGUI();
        frame.getContentPane().add(gui);

        frame.addWindowListener(new java.awt.event.WindowAdapter() {
             public void windowClosing(java.awt.event.WindowEvent e) {
//	         gui.close();
                 System.exit(0);
             }
         });

        frame.pack();
        frame.setVisible(true);
    }
    
    /** Creates new form ExitPrompt */
    public ServiceStatusGUI() {

        initComponents();
        oPattern = Pattern.compile(serverName+"\\.(.*)");
        fillTable();
	subscribe();
        startTimer();
    }
    
    public void startTimer() {
       timer = new Timer(TIME_INTERVAL, new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
               try {
	         boolean exists = Sct.IS.SctNames.getISRepository().contains(serverName+".anything");
	       }catch(Exception e) {
	         // server does not exist!
		 if(subscribed) {
		    subscribed=false;
		    updateLabel();
		    return;
		    }
		 }
	       // server does exist!
	       if(!subscribed) {
	         fillTable();
		 subscribe();
		 }
            }
        });
     timer.setRepeats(true);
     timer.start();
     }
    
    public void subscribe() {
            subscribed = GuiComponents.System.ISInterface.getInstance().subscribe(serverName, ".*", true, this).matches(".*successful.*");
	    updateLabel();
        }

    public void unsubscribe() {
            if(!subscribed) return;
            GuiComponents.System.ISInterface.getInstance().unsubscribe(serverName, ".*");
	}
	
	
    abstract class StringRunnable implements Runnable {
	protected String name;
	StringRunnable(String name) {
	    this.name = name;
	}
    }

    public void infoCreated(InfoEvent e) {
//      System.out.println("created "+e.getName());
       javax.swing.SwingUtilities.invokeLater(new StringRunnable(e.getName()) {
          public void run() {
             fillTable();
             }
        });
    }


    public void infoDeleted(InfoEvent e) {
       javax.swing.SwingUtilities.invokeLater(new StringRunnable(e.getName()) {
          public void run() {
            fillTable();
          }
        });
    }

    public void infoUpdated(InfoEvent e) {
//        System.out.println("updated "+e.getName());
        javax.swing.SwingUtilities.invokeLater(new StringRunnable(e.getName()) {
          public void run() {
            Matcher matcher = oPattern.matcher(name);
	    if(!matcher.matches()) return;
	    String service = name.substring(matcher.start(1),matcher.end(1));
	    if(!serverMap.containsKey(service)) {  // must be a new object! (somwehow...)
	       fillTable();
	       return;
	       }
	    int row = ((Integer)serverMap.get(service)).intValue();
	    updateTable(row,name,service);
            }
        });
    }
    

    private void initComponents() {

        java.awt.Font displayScaleFont = new java.awt.Font("Arial",java.awt.Font.PLAIN,10);

        setBackground(new java.awt.Color(255, 255, 255));

        serverInfoTable = new myTableAdapter();
        table = new javax.swing.JTable(serverInfoTable);
        jScrollPane1 = new javax.swing.JScrollPane(table);
        table.setSelectionMode(javax.swing.ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);

        java.awt.GridBagConstraints gridBagConstraints;

//        buttonGroup1 = new javax.swing.ButtonGroup();
        jLabel1 = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        jScrollPane1 = new javax.swing.JScrollPane(table);
        cancelButton = new javax.swing.JButton();

        setLayout(new java.awt.GridBagLayout());


        jLabel1.setText("Service Status");
        jLabel1.setFont( new java.awt.Font("Arial",java.awt.Font.BOLD,16));
        jLabel1.setForeground(new java.awt.Color(51, 0, 153));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 5, 5);
        gridBagConstraints.weighty = 0.1;
        add(jLabel1, gridBagConstraints);

        jLabel2.setText("Last update: Not yet updated.");
        jLabel2.setFont(displayScaleFont);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 0, 5);
        gridBagConstraints.weighty = 0.1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.SOUTHWEST;
        add(jLabel2, gridBagConstraints);

        jScrollPane1.setBackground(new java.awt.Color(255, 255, 255));
        table.setBackground(new java.awt.Color(255, 255, 255));
	table.setFont(displayScaleFont);
        jScrollPane1.setBorder(new javax.swing.border.BevelBorder(javax.swing.border.BevelBorder.LOWERED));
        jScrollPane1.setMaximumSize(new java.awt.Dimension(800, 300));
        jScrollPane1.setMinimumSize(new java.awt.Dimension(300, 100));
        jScrollPane1.setPreferredSize(new java.awt.Dimension(400, 100));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.insets = new java.awt.Insets(0, 5, 5, 5);
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weighty = 0.8;
        add(jScrollPane1, gridBagConstraints);




    }


    private void cancelButtonActionPerformed(java.awt.event.ActionEvent evt) {
        unsubscribe();
    }   


  public void fillTable() {

       serverMap = new HashMap();
       try {
           Vector tableList = new Vector();
	   Vector theLine = new Vector();

           String[] objects = GuiComponents.System.ISInterface.getInstance().getISObjects(serverName);
	   for(int i=0;i<objects.length;i++) {
             Matcher matcher = oPattern.matcher(objects[i]);
	     String service = matcher.matches() ? objects[i].substring(matcher.start(1),matcher.end(1)) : null;
             if(service==null) {
	        System.err.println("SctGUI - failed to match object name "+objects[i]);
		continue;
		}
		
	      Vector thisLine = getDataLine(objects[i],service);
	      if(thisLine==null) continue;
	      
	      tableList.addElement(thisLine);
	      
	      serverMap.put(service,new Integer(i));
	   }
           serverInfoTable.reTable(tableList);
	   updateLabel();

        }catch(Exception ioE){}
   }

private void updateTable(int row, String name, String service) {
        Vector theLine = getDataLine(name,service);
	if(theLine==null) return;
        serverInfoTable.setValueAt(theLine,row);
	updateLabel();
	}

   
private Vector getDataLine(String objectName, String service) {
         try {
	      ServiceInformation.ServiceStatus status = new ServiceInformation.ServiceStatus();
              Sct.IS.SctNames.getISRepository().getValue(objectName, status);
              Vector thisLine = new Vector();
	      thisLine.addElement(service);
	      thisLine.addElement(new Integer(status.done));
	      thisLine.addElement(new Integer(status.queue));
	      thisLine.addElement(new Float(status.averageTimeEach));
	      Float left=new Float(status.timeLeft);
	      Integer hours = new Integer(left.intValue()/3600);
	      Integer mins = new Integer(left.intValue()%3600/60);
	      Integer secs = new Integer(left.intValue()%60);
	      thisLine.addElement(new String(" "+hours+"h "+mins+"m "+secs+"s"));
	      return thisLine;
        }catch(Exception e){System.err.println("Exception reading SctService object "+objectName+" : "+e.toString());}
	return null;
	}

private void updateLabel() {
        try {
	  java.util.Date today = new java.util.Date();
          java.text.DateFormat formatter = java.text.DateFormat.getTimeInstance(java.text.DateFormat.DEFAULT,new java.util.Locale("en","GB"));
	  if(subscribed) {
	     jLabel2.setForeground(java.awt.Color.black);
	     jLabel2.setText("Service status last changed at "+formatter.format(today));
	     }
	  else {
	      jLabel2.setForeground(java.awt.Color.red);
	      jLabel2.setText("IS server is not currently available - updates suspended.");
	      }
          }catch(Exception e){}
	}



public class myTableAdapter extends javax.swing.table.AbstractTableModel {
    Vector		rows;
    public int recordCount;
    String[]            columnNames = {"Service","# Done","# Queued","Av.Time(s)","Time Left"};
    public myTableAdapter() {

         rows = new Vector();
         recordCount=0;
         }


// update the table using externally queried and processed data
    public void reTable(Vector theData) {
         rows = theData;
         recordCount= (theData.size()>=0) ? theData.size() : 0;
         fireTableChanged(null); // Tell the listeners a new table has arrived.
         return ;
    }


    public String getColumnName(int column) {
        if (columnNames[column] != null) {
            return columnNames[column];
        } else {
            return "";
        }
    }

    public Class getColumnClass(int column) {
//            return String;
            return getValueAt(0, column).getClass();

   }

    public boolean isCellEditable(int row, int column) {
            return false;

    }

    public int getColumnCount() {
        return columnNames.length;
    }

    // Data methods

    public int getRowCount() {
        return rows.size();
    }

    public Object getValueAt(int aRow, int aColumn) {
        Vector row = (Vector)rows.elementAt(aRow);
        return row.elementAt(aColumn);
    }
    
    public void setValueAt(Object value, int row) {
        rows.setElementAt(value,row);
        fireTableRowsUpdated(row,row);
	}


    public void setValueAt(Object value, int row, int column) {
        Vector dataRow = (Vector)rows.elementAt(row);
        dataRow.setElementAt(value, column);

    }
 }



}
