#include <stdio.h>

#include <ipc/core.h>
#include "is/isrepository.h"
#include "is/exception.h"

using namespace std;

namespace
{
    extern const char repository_tag_begin[] = "<repository items=\"%u\" callbacks=\"%u\">";
    extern const char repository_tag_end[] = "</repository>";
    extern const char info_tag_begin[] = "<info state=\"%u\" values=\"%u\" callbacks=\"%u\">";
    extern const char info_tag_end[] = "</info>";
    extern const char data_tag_begin[] = "<o c=";
    extern const char data_tag_end[] = "</o>";
    extern const char callback_tag_begin[] = "<callback>";
    extern const char callback_tag_end[] = "</callback>";
    extern const char name_tag_begin[] = "<name>";
    extern const char name_tag_end[] = "</name>";
};

template <const char btag[], const char etag[]>
ostream& put( ostream & out, const string & str )
{
    out << btag << str << etag << endl;
    return out;
}

template < const char btag[], const char etag[], unsigned bn, unsigned en >
istream& get( istream & in, string & str )
{
    char buff[ bn + 1 ];
    in.get( buff, bn, '\n' );
    
    if ( strcmp( btag, buff ) )
    {
    	throw is::BadBackupFile();
    }
    
    char c = 0;
    while( in.good() )
    {
    	in.get( c );
	if ( c != '<' ) str += c;
	else
	{
	    in.putback( c );
	    char end[ en + 1 ];
	    in.get( end, en, '\n' );
	    if ( !strcmp( etag, end ) )
		break;
	    else
		str += end;
	}
    }
    
    while( in.get( c ).good() && ( c == ' ' || c == '\n' ) );
    
    in.putback( c );
    
    if ( !in.good() )
    {
    	throw is::BadBackupFile();
    }
    
    return in;
}

ostream& operator<<( ostream& out, const is::callback_var & cb )
{
    string ior = IPCCore::objectToString( cb, IPCCore::Corbaloc );
    put<callback_tag_begin,callback_tag_end>( out, ior );
    return out;
}

istream& operator>>( istream& in, is::callback_var & cb )
{
    string ior;
    get<callback_tag_begin,callback_tag_end,
    	sizeof(callback_tag_begin),sizeof(callback_tag_end)>( in, ior );

    try
    {
	CORBA::Object_var object = IPCCore::stringToObject( ior );
	cb = is::callback::_narrow( object );
	if ( CORBA::is_nil( cb ) || cb -> _non_existent() )
	{
	    throw is::InvalidCallback();
	}
    }
    catch(...)
    {
	throw is::InvalidCallback();
    }
    return in;
}

ostream& operator<<( ostream& out, const deque<string> & qu )
{
    for ( unsigned int i = 0; i < qu.size(); i++ )
    {
    	out << qu[i].c_str() << endl;
    }
    return out;
}

istream& operator>>( istream& in, deque<string> & qu )
{
    for ( unsigned int i = 0; i < qu.size(); i++ )
    {
	string data;
	get<data_tag_begin,data_tag_end,
	    sizeof(data_tag_begin),sizeof(data_tag_end)>( in , data );
	qu[i] = data_tag_begin + data + data_tag_end;
    }
    return in;
}

ostream& operator<<( ostream& out, const ISRepository::Callback & cb )
{
    put<name_tag_begin, name_tag_end>( out, cb.rexpression_.str() );
    out << cb.callback_;
    return out;
}

istream& operator>>( istream& in, ISRepository::Callback & cb )
{
    string rexp;
    get<name_tag_begin, name_tag_end,
    	sizeof(name_tag_begin),sizeof(name_tag_end)>( in, rexp );
    cb.rexpression_ = rexp;

    in >> cb.callback_;
	
    return in;
}

ostream& operator<<( ostream & out, ISInfoTrue & iso )
{
    char header[256];
    sprintf( header, info_tag_begin, (int)iso.state_, iso.data_.size(), iso.callbacks_.size() );
    out << header << endl;
    
    if ( iso.state_ == ISInfoTrue::Public )
    {
	out << iso.data_;
    }
    
    ISInfoTrue::CallbackMap::iterator it;
    
    for ( it = iso.callbacks_.begin(); it != iso.callbacks_.end( ); ++it )
    {
	out << *it;
    }
    out << info_tag_end << endl;
    return out;
}

istream& operator>>( istream & in, ISInfoTrue & iso )
{
    string header;
    getline( in, header );
    unsigned int history_num, callback_num;
    if ( sscanf( header.c_str(), info_tag_begin, (unsigned int*)&iso.state_, &history_num, &callback_num ) != 3 )
    {
    	throw is::BadBackupFile();
    }

    if ( iso.state_ == ISInfoTrue::Public )
    {
	iso.data_.resize( iso.max_history_size_ < history_num ? iso.max_history_size_ : history_num );
	in >> iso.data_;
    }
    
    for ( unsigned int i = 0; i < callback_num; i++ )
    {
	try
	{
	    is::callback_var cb;
	    in >> cb;
	    iso.callbacks_.push_back( cb );
	}
	catch(...)
	{
	}
    }
    
    string end;
    in >> end;
    if ( end != info_tag_end )
    {
    	throw is::BadBackupFile();
    }
    in.ignore();
    return in;
}

ostream& operator<<( ostream & out, ISRepository & isf )
{	
    if ( isf.verbosity_ > 0 )
    {
	cout << OWLTime() << " :: INFORMATION [::operator<<] saving information:" << endl;
    }
	
    ISRepository::InfoMap::iterator    it;
    
    isf.mutex_.lock();
    
    char header[256];
    sprintf( header, repository_tag_begin, isf.information_.size(), isf.callbacks_.size() );
    out << header << endl;
    
    for ( it = isf.information_.begin( ); it != isf.information_.end( ); ++it )
    {
	put<name_tag_begin, name_tag_end>( out, it->first );
	out << *(it->second);
	
	if ( isf.verbosity_ > 1 )
	{
	    cout << OWLTime() << " :: INFORMATION [::operator<<] object " << it->first << " is saved" << endl;
	}
    }
    isf.mutex_.unlock();
    
    if ( isf.verbosity_ > 0 )
    {
	cout << OWLTime() << " :: " << isf.information_.size() << " information object(s) saved" << endl;
    }
    
    isf.cb_mutex_.read_lock();
    ISRepository::CallbackMap::iterator cit = isf.callbacks_.begin( );
    
    for ( cit = isf.callbacks_.begin( ); cit != isf.callbacks_.end( ); ++cit )
    {
	out << *(*cit);
	if ( isf.verbosity_ > 1 )
	{
	    cout << OWLTime() << " :: INFORMATION [::operator<<] subscription for " 
	    		      << (*cit) -> rexpression_.str() << " is saved" << endl;
	}
    }
    isf.cb_mutex_.unlock();
    
    if ( isf.verbosity_ > 0 )
    {
	cout << OWLTime() << " :: " << isf.callbacks_.size() << " subscription(s) saved" << endl;
    }
    
    out << repository_tag_end << endl;
    return out;
}

istream& operator>>( istream & in, ISRepository & isf )
{    
    if ( isf.verbosity_ > 0 )
    {
	cout << OWLTime() << " :: INFORMATION [::operator>>] restoring information:" << endl;
    }

    string header;
    getline( in, header );
    unsigned int info_num = 0;
    unsigned int callback_num = 0;
    if ( sscanf( header.c_str(), repository_tag_begin, &info_num, &callback_num ) != 2 )
    {
	throw is::BadBackupFile();
    }
    
    isf.mutex_.lock();
    for ( unsigned int i = 0; i < info_num ; i++ )
    {
	string name;
	get<name_tag_begin, name_tag_end,
    	    sizeof(name_tag_begin),sizeof(name_tag_end)>( in, name );
	ISInfoTrue * info = new ISInfoTrue( name.c_str(), isf.max_history_size_ );
	in >> *info;
	isf.information_[string(name)] = info;

	if ( isf.verbosity_ > 0 )
	{
	    cout << OWLTime() << " :: INFORMATION [::operator>>] object " 
	    		      << name << " is restored" << endl;
	}
    }
    isf.mutex_.unlock();
    if ( isf.verbosity_ > 0 )
    {
	cout << OWLTime() << " :: INFORMATION [::operator>>] " 
			  << info_num << " information object(s) restored" << endl;
    }
    
    isf.cb_mutex_.write_lock();
    for ( unsigned int i = 0; i < callback_num; )
    {
	ISRepository::Callback * cb = new ISRepository::Callback;
	try
	{
	    in >> *cb;
	    isf.callbacks_.push_back( cb );
	    if ( isf.verbosity_ > 0 )
	    {
		cout << OWLTime() << " :: INFORMATION [::operator>>] subscription to (" 
				  << cb -> rexpression_.str() << ") is restored" << endl;
	    }
	    i++;
	}
	catch ( is::InvalidCallback & ex )
	{
	    if ( isf.verbosity_ > 0 )
	    {
		cout << OWLTime() << " :: WARNING [::operator>>] Invalid subscription (" 
				  << cb -> rexpression_.str() << ") is not restored" << endl;
	    }
	    delete cb;
	    callback_num--;
	}
    }
    isf.cb_mutex_.unlock();
    if ( isf.verbosity_ > 0 )
    {
	cout << OWLTime() << " :: INFORMATION [::operator>>] " << callback_num
			  << " subscription(s) restored" << endl;
    }
    
    string end;
    in >> end;
    if ( end != repository_tag_end )
    {
    	throw is::BadBackupFile();
    }
    return in;
}

