#include <owl/time.h>
#include "is/isinfotrue.h"
#include "ipc/core.h"

ISInfoTrue::ISInfoTrue( const char * name, const char * data, unsigned long max_history_size )
  : name_( name ),
    state_( Public ),
    frozen_( 0 ),
    max_history_size_( max_history_size )
{
    data_.push_front( data );
}

ISInfoTrue::ISInfoTrue( const char * name, unsigned long max_history_size )
  : name_( name ),
    state_( Private ),
    frozen_( 0 ),
    max_history_size_( max_history_size )
{
    data_.push_front( "" );
}

void ISInfoTrue::validate ( const char * data, int verbosity )
{
    data_.front() = data;
    state_ = Public;
    
    inform( data, is::Created, verbosity );
}

void ISInfoTrue::invalidate ( int verbosity )
{
    state_ = Private;
    
    inform( data_.front().c_str(), is::Deleted, verbosity );    
}

void ISInfoTrue::inform( const char * data, is::Reason reason, int verbosity )
{
    if ( callbacks_.empty() )
    {
	return;
    }
	    
    CallbackMap::iterator it = callbacks_.begin();
	
    while ( it != callbacks_.end() )
    {
	try
	{
	    (*it) -> inform( data, reason );
	    if ( verbosity > 0 )
	    {
		std::cout << OWLTime() << " :: INFORMATION [ISInfoTrue::inform] subscriber for the (" 
		     << name_ << ") has been notified" << std::endl;
	    }
	    it++;
	}
	catch (...)
	{
	    if ( verbosity > 0 )
	    {
		std::cout << OWLTime() << " :: WARNING [ISInfoTrue::inform] subscriber for the (" 
		     << name_ << ") is gone unexpectedly" << std::endl;
	    }
	    it = callbacks_.erase( it );
	}
    }
}

bool ISInfoTrue::update ( const char * data, bool keep_history, int verbosity )
{
/*  if ( type_ != type )
    {
	status -> returnCode = is_E_NotCompatible;
	return false;
    }
*/
    if ( keep_history )
    {
	data_.push_front( data );
	if ( data_.size() > max_history_size_ )
	{
	    data_.pop_back();
	}
    }
    else
    {
	data_.front() = data;
    }
    inform( data, is::Updated, verbosity );
    
    return true;
}

bool ISInfoTrue::unsubscribe ( is::callback_ptr cb )
{    	
    for ( CallbackMap::iterator it = callbacks_.begin(); it != callbacks_.end(); it++ )
    {
	if ( (*it)->_is_equivalent( cb ) )
	{
	    callbacks_.erase( it );
	    return true;
	}
    }
    return false;
}

