

#include "CountedExample.h"
#include <ipc/core.h>
#include <unistd.h>
#include <iostream>

void sl(const unsigned int s) {
    std::cout << "Sleeping for " << s << " seconds" << std::endl;
    sleep(s);
};

void createACountedExampleAndDoStuff() {
  std::cout << "Making a CountedExample ..." << std::endl;
  CountedExample & countedExampleSrv = *(new CountedExample); // not supposed to do "LargeCarrot largeCarrot;" as this can lead to largeCarrot being deleted twice -- once when largeCarrot goes out of scope, and once if/when someone remotely requests it be deleted.".  For this reason, we are equally only supposed to delete it with _destroy();
  std::cout << "... finishing making a CountedExample" << std::endl;
  sl(1);
  {
    // get a reference to the countedExample object
    std::cout << "Getting a reference to the countedExample object" << std::endl;
    Stew::CountedExample_var countedExampleRef = countedExampleSrv._this();
    sl(1);
  };

  {
    // get a different reference to the countedExample object
    std::cout << "Getting a different reference to the countedExample object" << std::endl;
    Stew::CountedExample_var countedExampleRef = countedExampleSrv._this();
    sl(1);

    std::cout << "Decrementing the count on the second reference ... should lead to the death of the servant" << std::endl;
    countedExampleRef->removeRef(); // we implemented this method so that it would end the life of the servant.
    // it is vital we call shutdown for this temporary object otherwise we will leak memory.
    sl(1);

  };
  
};

int main(int argc, char * argv[]) {
  std::cout << "This program demonstrates how to use an object that will live only for a short time (eg a return type) that will not be published in the IPC::Partition index, and where the lifetime of the servant managing this object is to be controlled by the user manually via calls to addCount() and removeCount().  This makes it a little bit like an \"unsafe\" boost::shared_ptr." << std::endl;
  std::cout << "Initialising IPC ..." << std::endl;
  IPCCore::init(argc, argv);

  // create the server for a CountedExample object
  createACountedExampleAndDoStuff();

  std::cout << "Bye" << std::endl;
};
