#include "Addressing_Test.h"
#include "../src/LogicErrors.h"
#include "../src/Addressing.h"
#include <boost/test/unit_test.hpp>

using boost::unit_test_framework::test_suite;

namespace Sct{
  
  void testAddressingStringInterface() {

    BOOST_CHECK_THROW(UPID (""), Sct::InvalidArgumentError);
    BOOST_CHECK_THROW(UPID (":"), Sct::InvalidArgumentError);
    BOOST_CHECK_THROW(UPID (":3"), Sct::InvalidArgumentError);
    BOOST_CHECK_THROW(UPID ("UPID:"), Sct::InvalidArgumentError);
    BOOST_CHECK_THROW(UPID ("UPID:moo"), Sct::InvalidArgumentError);
    BOOST_CHECK_THROW(UPID ("moo:3"), Sct::InvalidArgumentError);
    BOOST_CHECK_THROW(UPID ("UPID:5.8"), Sct::InvalidArgumentError);

    BOOST_CHECK_THROW(UCID (""), Sct::InvalidArgumentError);
    BOOST_CHECK_THROW(UCID (":"), Sct::InvalidArgumentError);
    BOOST_CHECK_THROW(UCID (":3.9"), Sct::InvalidArgumentError);
    BOOST_CHECK_THROW(UCID ("UCID:"), Sct::InvalidArgumentError);
    BOOST_CHECK_THROW(UCID ("UCID:."), Sct::InvalidArgumentError);
    BOOST_CHECK_THROW(UCID ("UCID:3"), Sct::InvalidArgumentError);
    BOOST_CHECK_THROW(UCID ("UCID:3."), Sct::InvalidArgumentError);
    BOOST_CHECK_THROW(UCID ("UCID:.3"), Sct::InvalidArgumentError);
    BOOST_CHECK_THROW(UCID ("UCID:moo.moo"), Sct::InvalidArgumentError);
    BOOST_CHECK_THROW(UCID ("moo:3.8"), Sct::InvalidArgumentError);
    BOOST_CHECK_THROW(UCID ("UCID:5.8.6"), Sct::InvalidArgumentError);

    BOOST_CHECK_THROW(URID (""), Sct::InvalidArgumentError);
    BOOST_CHECK_THROW(URID (":"), Sct::InvalidArgumentError);
    BOOST_CHECK_THROW(URID (":3.9"), Sct::InvalidArgumentError);
    BOOST_CHECK_THROW(URID ("URID:"), Sct::InvalidArgumentError);
    BOOST_CHECK_THROW(URID ("URID:."), Sct::InvalidArgumentError);
    BOOST_CHECK_THROW(URID ("URID:3"), Sct::InvalidArgumentError);
    BOOST_CHECK_THROW(URID ("URID:3."), Sct::InvalidArgumentError);
    BOOST_CHECK_THROW(URID ("URID:.3"), Sct::InvalidArgumentError);
    BOOST_CHECK_THROW(URID ("URID:3.4.moo"), Sct::InvalidArgumentError);
    BOOST_CHECK_THROW(URID ("URID:3.moo.4"), Sct::InvalidArgumentError);
    BOOST_CHECK_THROW(URID ("URID:moo.5.6"), Sct::InvalidArgumentError);
    BOOST_CHECK_THROW(URID ("moo:.3.8"), Sct::InvalidArgumentError);
    BOOST_CHECK_THROW(URID ("moo:3..8"), Sct::InvalidArgumentError);
    BOOST_CHECK_THROW(URID ("moo:3.8."), Sct::InvalidArgumentError);
    BOOST_CHECK_THROW(URID ("URID:5.8.6.8"), Sct::InvalidArgumentError);

    const UPID ps_8("UPID:8");
    const UCID cs_8_12("UCID:8.12");
    const URID rs_8_12_42("URID:8.12.42");
    
    BOOST_CHECK( ps_8       == UPID(std::string(ps_8      )) );
    BOOST_CHECK( cs_8_12    == UCID(std::string(cs_8_12   )) );
    BOOST_CHECK( rs_8_12_42 == URID(std::string(rs_8_12_42)) );

    BOOST_CHECK( "UPID:8"       == std::string(UPID("UPID:8"      )) );
    BOOST_CHECK( "UCID:8.12"    == std::string(UCID("UCID:8.12"   )) );
    BOOST_CHECK( "URID:8.12.42" == std::string(URID("URID:8.12.42")) );
  };


  void testAddressingConstructors() {

    const UniquePartitionIdentifier p__8(8);

    const UPID p_9(9);
    const UPID p_8(8);
    const UPID ps_8("UPID:8");
    
    BOOST_CHECK(p__8 == p_8);
    BOOST_CHECK(p__8 == ps_8);
    BOOST_CHECK(p__8 != p_9);

    BOOST_CHECK(p_8 != p_9);
    BOOST_CHECK(p_8 == p__8);
    BOOST_CHECK(p_9 != p__8);
    BOOST_CHECK(p_9 != p_8);


    const UniqueCrateIdentifier c__8_12(p_8,12);

    const UCID c_8_12(8,12);
    const UCID c_9_12(9,12);
    const UCID c_8_13(8,13);
    const UCID c_9_13(9,13);
    const UCID cs_8_12("UCID:8.12");
    
    BOOST_CHECK(c__8_12 == c_8_12);
    BOOST_CHECK(c__8_12 == cs_8_12);
    BOOST_CHECK(c__8_12 != c_8_13);
    BOOST_CHECK(c__8_12 != c_9_12);
    BOOST_CHECK(c__8_12 != c_9_13);


    const UniqueRodIdentifier r__8_12_42(c__8_12,42);

    const URID r_8_12_42(8,12,42);
    const URID r_8_12_43(8,12,43);
    const URID r_8_13_42(8,13,42);
    const URID r_9_12_42(9,12,42);
    const URID rs_8_12_42("URID:8.12.42");
    
    BOOST_CHECK(r__8_12_42 == r_8_12_42);
    BOOST_CHECK(r__8_12_42 == rs_8_12_42);
    BOOST_CHECK(r__8_12_42 != r_8_12_43);
    BOOST_CHECK(r__8_12_42 != r_8_13_42);
    BOOST_CHECK(r__8_12_42 != r_9_12_42);

  };

  void testAddressingAssignment() {
    const UPID p__8(8);
    const UCID c__8_12(p__8,12);
    const URID r__8_12_42(c__8_12,42);

    UPID p;
    UCID c;
    URID r;
    
    p = p__8;
    c = c__8_12;
    r = r__8_12_42;

    BOOST_CHECK(p == p__8);
    BOOST_CHECK(c == c__8_12);
    BOOST_CHECK(r == r__8_12_42);
 };

  void testAddressingIOStream(){
    const UPID p__8(8);
    const UCID c__8_12(8,12);
    const URID r__8_12_42(8,12,42);
    const UTID t__8_12(8,12);
    {
      std::ostringstream oss;
      oss << p__8;
      BOOST_CHECK_EQUAL("UPID:8", oss.str());
    };
    {
      std::ostringstream oss;
      oss << c__8_12;
      BOOST_CHECK_EQUAL("UCID:8.12", oss.str());
    };
    {
      std::ostringstream oss;
      oss << r__8_12_42;
      BOOST_CHECK_EQUAL("URID:8.12.42", oss.str());
    };
    {
      std::ostringstream oss;
      oss << t__8_12;
      BOOST_CHECK_EQUAL("UTID:8.12", oss.str());
    };

  }

  Addressing_Test::Addressing_Test() {
    add(BOOST_TEST_CASE(&testAddressingConstructors));
    add(BOOST_TEST_CASE(&testAddressingStringInterface));
    add(BOOST_TEST_CASE(&testAddressingAssignment));
    add(BOOST_TEST_CASE(&testAddressingIOStream));
  }
}
