#include "UniqueID_Test.h"
#include "../src/LogicErrors.h"
#include "../src/UniqueID.h"
#include <boost/test/unit_test.hpp>

using boost::unit_test_framework::test_suite;

namespace Sct{

void testParse(){
    UniqueID n("123.56.234320443");
    BOOST_CHECK_EQUAL((unsigned)123, n.getRunNumber());
    BOOST_CHECK_EQUAL((unsigned)56, n.getScanNumber());
    BOOST_CHECK_EQUAL("234320443", n.getModule());
}

void testParseFail(){
    BOOST_CHECK_THROW(UniqueID n("123.56234320443"), Sct::InvalidArgumentError);
}


void testConstruct() {
    UniqueID n(123, 56, "234320443");
    BOOST_CHECK_EQUAL("123.56.234320443", (string)n.getUniqueID());
    BOOST_CHECK(UniqueID("123.56.234320443")==n.getUniqueID());
}

void testCast() {
    UniqueID n(123, 56, "234320443");
    BOOST_CHECK_EQUAL((string)n, (string)n.getUniqueID());
    BOOST_CHECK(UniqueID(n)==n.getUniqueID());
}

void testEquality(){
    UniqueID n(123, 56, "234320443");
    UniqueID b(n);
    BOOST_CHECK(b==n);
    BOOST_CHECK( !(b!=n));
    UniqueID c(1,1,"asdf");
    c=n;
    BOOST_CHECK(c==n);

}

void testIOStream(){
  UniqueID n(123, 56, "234320443");
  std::ostringstream oss;
  oss << n;
  BOOST_CHECK_EQUAL((string)n, oss.str());
  UniqueID c(1,1,"asdf");
  std::istringstream iss(oss.str());
  iss >> c;
  BOOST_CHECK(c==n);
}

UniqueID_Test::UniqueID_Test() {
  add(BOOST_TEST_CASE(&testParse));
  add(BOOST_TEST_CASE(&testParseFail));
  add(BOOST_TEST_CASE(&testConstruct));
  add(BOOST_TEST_CASE(&testCast));
  add(BOOST_TEST_CASE(&testEquality));
  add(BOOST_TEST_CASE(&testIOStream));
}
}
