
#ifndef SCT_STRINGSTREAMER_HH
#define SCT_STRINGSTREAMER_HH

// The purpose of the StringStreamer object is to make it easier to
// write mrs (message stream) messages containing mixtures of numbers
// and text without the need for explicit creation of temporary
// ostringstream objects to hold the MRS_TEXT while it is being built.

// For two examples of how to use StringStreamer object,
// see the foot of this file

// Note that this file is called StringStreamer rather than
// StringStreamerClass because the purpose of this file is to
// introduce the Sct::StringStreamer const identifier, not to
// define the StringStreamerClass which nobody will ever need
// to use directly.

// Author Chris Lester.

#include <sstream>
#include <string>

namespace Sct {

  class StringStreamerClass {
  private:
    std::string _s;
  public:
    /*
    template<class T>
    const StringStreamerClass operator,(const T & i) const {
      std::ostringstream oss;
      oss << _s;
      oss << i;
      return StringStreamerClass(oss.str());
    };
    */
    template<class T>
    const StringStreamerClass operator<<(const T & i) const {
      std::ostringstream oss;
      oss << _s;
      oss << i;
      return StringStreamerClass(oss.str());
    };
    operator std::string() const {
      return _s;
    };
    StringStreamerClass() {};
    StringStreamerClass(const std::string & s) : _s(s) {
    };
  };

  const StringStreamerClass StringStreamer;

};


// Example of use:
//
// If you were to comment out the mini-program below,
// compile it, and run it, it should produce the output:
//
// "s was set to [4-5 is the best!]"

/*

#include <iostream>

int main() {
  const std::string s = (Sct::StringStreamer, 4, "-", 5, " is the best!");
  std::cout << "s was set to ["<<s<<"]"<<std::endl;
  return 0;
};

// A more useful example (for which it was created!) would be:

int someNumber = XXX;
mrs_stream << MRS_TEXT((Sct::StringStreamer, "This message including ",someNumber, " would have been very awkward to send without Sct::StringStreamer")) << ENDM;

*/

#endif
