#include "SctApiConfigCache.h"
#include "ConfigurationUtility.h"
#include "SctApiException.h"

using boost::shared_ptr;
using SctApi::SctApiConfigCache;

SctApiConfigCache::SctApiConfigCache(){
}

SctApiConfigCache::~SctApiConfigCache(){
}

void SctApiConfigCache::set(UINT32 mid, shared_ptr<ModuleBanks> banks){
  boost::mutex::scoped_lock lock(cache_mutex);
  m_map[mid] = banks;
}

shared_ptr<SctApi::SctApiConfigCache::ModuleBanks> SctApiConfigCache::getFromMid(const UINT32 mid){
  boost::mutex::scoped_lock lock(cache_mutex);
  shared_ptr<ModuleBanks> banks;
  if (m_map.find(mid)!=m_map.end()) banks = m_map[mid];
  return banks;
}

shared_ptr<const SctApi::SctApiConfigCache::ModuleBanks> SctApiConfigCache::getFromMid(const UINT32 mid) const {
  boost::mutex::scoped_lock lock(cache_mutex);
  boost::shared_ptr<const ModuleBanks> banks;
  std::map<UINT32, boost::shared_ptr<ModuleBanks> >::const_iterator it=m_map.find(mid);
  if (it!=m_map.end()) banks = (*it).second;
  return banks;
}

boost::shared_ptr<ABCDModule> SctApiConfigCache::ModuleBanks::get(BankType b){
  boost::mutex::scoped_lock lock(bank_mutex);
  boost::shared_ptr<ABCDModule> config;
  std::map<BankType, boost::shared_ptr<ABCDModule> >::iterator it=m_map.find(b);
  if (it!=m_map.end()) config=(*it).second;
  return config;
}

boost::shared_ptr<const ABCDModule> SctApiConfigCache::ModuleBanks::get(BankType b) const{
  boost::mutex::scoped_lock lock(bank_mutex);
  boost::shared_ptr<const ABCDModule> config;
  std::map<BankType, boost::shared_ptr<ABCDModule> >::const_iterator it=m_map.find(b);
  if (it!=m_map.end()) config=(*it).second;
  return config;
}

void SctApiConfigCache::ModuleBanks::set(BankType bank, boost::shared_ptr<ABCDModule> config){
  boost::mutex::scoped_lock lock(bank_mutex);
  m_map[bank]=config;
}

void SctApiConfigCache::ModuleBanks::copy(BankType from, BankType to){
  boost::mutex::scoped_lock lock(bank_mutex);
  boost::shared_ptr<ABCDModule> copyme=get(from);
  if (!copyme.get()) {
    std::string message("Attempt to copy null bank");
    message += __FILE__;
    message += __LINE__;
    throw SctApiException(message);
  }
  std::auto_ptr<ABCDModule> copy=ConfigUtility::clone(*copyme);
  m_map[to]=copy;
}
