/**
   SctApiDDC.cxx

   Methods for DDC, using SCTDCSCommand
*/

#include "SctApiDDC.h"

//#include <vector>
using namespace std;
#include "sctddc/SCTDCSCommand.hxx"  // This needs using std before it!

#include "sctConf/configuration.h"

#include "utility.h"
#include "SctApiException.h"

//#include "SctApi.h" // for the SctApi DDC related commands that are not yet part of SctApiDDC
#include "log.h"

using namespace SctApi::Utility;

namespace SctApi {

  SctApiDDC::SctApiDDC(boost::shared_ptr<SctConfiguration::Configuration> conf,
                       boost::shared_ptr<Log> log) :
    m_config(conf),
    m_log(log),
    m_ddcPartition(0),
    m_ddcCmd(0) {
  };

  void SctApiDDC::initDDC() 
  {
    // Initialise DDC command
    m_ddcPartition = new IPCPartition("SCT");
    m_ddcCtrls[0] = new string("ddc_ct1");
    m_ddcCtrls[1] = 0;

    m_ddcCmd = new SCTDCSCommand(*m_ddcPartition, m_ddcCtrls);
  }

  SCTDCSCommand * SctApiDDC::ddcCmd() { if (!m_ddcCmd) { this->initDDC(); }; return m_ddcCmd; };

  /// safe private config accessor:
  SctConfiguration::Configuration * SctApiDDC::config() { return m_config.get(); };

  std::ostream & SctApiDDC::log() { 
    if (m_log.get()) { 
      return *m_log;
    } else {
      return std::cout;
    }; 
  };
  
  void SctApiDDC::requestHardResetAll() 
  {
    cout << "Unimplemented!\n";
  }

  void SctApiDDC::requestHardReset(UINT32 mid)
  {
    unsigned int MUR, module;

    unsigned int partition, crate, rod, channel;
    getpcrc(mid, partition, crate, rod, channel);

    config()->translateFromROD(partition, crate, rod, channel,
                               MUR, module);

    unsigned int ppartition, pcrate, pchannel;

    config()->translateToPowerSupply(MUR, module, 
                                     ppartition, pcrate, pchannel);

    std::cout << "Should do hard reset on module in " 
              << " partition: " << ppartition 
              << " crate: " << pcrate 
              << " channel: " << pchannel << std::endl;
  }

  void SctApiDDC::requestIVCurve(UINT32 mid, FLOAT32 start, FLOAT32 stop, FLOAT32 step, UINT16 delay, FLOAT32 currentLim)
  {
    log() << "requestIVCurve: " << mid << ", " << start << ", " << stop << ", " 
          << step << " " << delay << " " << currentLim << std::endl;

    unsigned int MUR, module;

    unsigned int partition, crate, rod, channel;
    getpcrc(mid, partition, crate, rod, channel);

    config()->translateFromROD(partition, crate, rod, channel,
                               MUR, module);

    unsigned int ppartition, pcrate, pchannel;

    config()->translateToPowerSupply(MUR, module, 
                                     ppartition, pcrate, pchannel);

    log() << "ddc->requestIVCurve: " << crate << ", " << channel 
          << ", " << start << ", " << stop << ", " << step 
          << " " << 0 << " " << currentLim << " " << delay << " 10" << std::endl;

#warning "End voltage and current should be changeable"
    float Vend = 0.0;
    float Iwarn = currentLim;

    bool result = ddcCmd()->requestIVCurve(crate, channel, start, stop, step, Vend, 
                                           Iwarn, currentLim, delay, 10);

    log() << "iv curve success got: " << result << std::endl;
  
  }

  void SctApiDDC::requestIVCurveAll(FLOAT32 start, FLOAT32 stop, FLOAT32 step, UINT16 delay, FLOAT32 currentLim)
  {
  }


  void SctApiDDC::setSelect(UINT32 mid, bool value)
  {
    unsigned int MUR, module;

    unsigned int partition, crate, rod, channel;
    getpcrc(mid, partition, crate, rod, channel);

    config()->translateFromROD(partition, crate, rod, channel,
                               MUR, module);
    
    unsigned int ppartition, pcrate, pchannel;

    config()->translateToPowerSupply(MUR, module, 
                                     ppartition, pcrate, pchannel);

    ddcSetChannelParameter(pcrate, pchannel, "LVchCLKS", value?1.0:0.0);
  }

  void SctApiDDC::setSelectAllInCrate(int crate, bool value, int timeout)
  {
    ddcCmd()->setChannelsSelectLine(crate, value, timeout);
  }

  void SctApiDDC::changeRunState(int state)
  {
    throw SctApiException("changeRunState not implemented!");
  }

  // void SctApiDDC::ddcGetChannelParameter(int crate, int channel, string dpeName,
  //                                     int& dpeValue, int timeout) 
  // {
  //   log() << "ddcGetChannelParameter: " << crate << ", " << channel << ", (" << dpeName << "), " << dpeName.size() << " " << timeout << std::endl;

  //   ddcCmd()->getChannelParameter(crate, channel, dpeName, dpeValue, timeout);

  //   log() << "channel parameter got: " << dpeValue << std::endl;
  // }

  void SctApiDDC::ddcChangeState (int crate, int channel, short state, int timeout) 
  {
    ddcCmd()->changeState(crate, channel, state, timeout);
  }

  void SctApiDDC::ddcChangeStates (int crate, const char* name, short state, int timeout)
  {
    ddcCmd()->changeState(crate, name, state, timeout);
  }

  void SctApiDDC::ddcChangeStateAll (int crate, short state, int timeout)
  {
    ddcCmd()->changeState(crate, "all", state, timeout);
  }

  void SctApiDDC::ddcLoadConfiguration (int crate, short state, int timeout)
  {
    ddcCmd()->loadConfiguration(crate, state, timeout);
  }

  void SctApiDDC::ddcHardReset (int crate, int channel, int timeout)
  {
    ddcCmd()->hardReset(crate, channel, timeout);
  }

  void SctApiDDC::ddcHardResetAll (int crate, int timeout)
  {
    ddcCmd()->hardReset(crate, "all", timeout);
  }

  int SctApiDDC::ddcGetChannelParameterInt (int crate, int channel, string name, int timeout)
  {
    log() << "ddcGetChannelParameterInt: " << crate << ", " << channel 
          << ", (" << name << "), " << name.size() << " " << timeout << std::endl;

    int dpeValue;
    ddcCmd()->getChannelParameter(crate, channel, name, dpeValue, timeout);
    log() << "channel parameter got: " << dpeValue << std::endl;

    return dpeValue;
  }

  float SctApiDDC::ddcGetChannelParameterFloat (int crate, int channel, string name, int timeout)
  {
    log() << "ddcGetChannelParameterFloat: " << crate << ", " << channel 
          << ", (" << name << "), " << name.size() << " " << timeout << std::endl;

    float dpeValue;
    ddcCmd()->getChannelParameter(crate, channel, name, dpeValue, timeout);
    log() << "channel parameter got: " << dpeValue << std::endl;

    return dpeValue;
  }

  string SctApiDDC::ddcGetChannelParameterString (int crate, int channel, string name, int timeout)
  {
    log() << "ddcGetChannelParameterString: " << crate << ", " << channel 
          << ", (" << name << "), " << name.size() << " " << timeout << std::endl;

    string dpeValue;
    ddcCmd()->getChannelParameter(crate, channel, name, dpeValue, timeout);
    log() << "channel parameter got: " << dpeValue << std::endl;

    return dpeValue;
  }

  int SctApiDDC::ddcGetCardParameterInt (int crate, int card, string name, int timeout)
  {
    log() << "ddcGetCardParameterInt: " << crate << ", " << card 
          << ", (" << name << "), " << name.size() << " " << timeout << std::endl;

    int dpeValue;
    ddcCmd()->getCardParameter(crate, card, name, dpeValue, timeout);
    log() << "card parameter got: " << dpeValue << std::endl;

    return dpeValue;
  }

  float SctApiDDC::ddcGetCardParameterFloat (int crate, int card, string name, int timeout)
  {
    log() << "ddcGetCardParameterFloat: " << crate << ", " << card 
          << ", (" << name << "), " << name.size() << " " << timeout << std::endl;

    float dpeValue;
    ddcCmd()->getCardParameter(crate, card, name, dpeValue, timeout);
    log() << "card parameter got: " << dpeValue << std::endl;

    return dpeValue;
  }

  string SctApiDDC::ddcGetCardParameterString (int crate, int card, string name, int timeout)
  {
    log() << "ddcGetCardParameterString: " << crate << ", " << card 
          << ", (" << name << "), " << name.size() << " " << timeout << std::endl;

    string dpeValue;
    ddcCmd()->getCardParameter(crate, card, name, dpeValue, timeout);
    log() << "card parameter got: " << dpeValue << std::endl;

    return dpeValue;
  }

  int SctApiDDC::ddcGetCrateParameterInt (int crate, string name, int timeout)
  {
    log() << "ddcGetCrateParameterInt: " << crate 
          << ", (" << name << "), " << name.size() << " " << timeout << std::endl;

    int dpeValue;
    ddcCmd()->getCrateCtrlParameter(crate, name, dpeValue, timeout);
    log() << " crate parameter got: " << dpeValue << std::endl;

    return dpeValue;
  }

  float SctApiDDC::ddcGetCrateParameterFloat (int crate, string name, int timeout)
  {
    log() << "ddcGetCrateParameterFloat: " << crate 
          << ", (" << name << "), " << name.size() << " " << timeout << std::endl;

    float dpeValue;
    ddcCmd()->getCrateCtrlParameter(crate, name, dpeValue, timeout);
    log() << " crate parameter got: " << dpeValue << std::endl;

    return dpeValue;
  }

  string SctApiDDC::ddcGetCrateParameterString (int crate, string name, int timeout)
  {
    log() << "ddcGetCrateParameterString: " << crate 
          << ", (" << name << "), " << name.size() << " " << timeout << std::endl;

    string dpeValue;
    ddcCmd()->getCrateCtrlParameter(crate, name, dpeValue, timeout);
    log() << " crate parameter got: " << dpeValue << std::endl;

    return dpeValue;
  }

  void SctApiDDC::ddcGetCrateParameters(int crate, std::vector<std::string> names, 
                                        std::map<std::string, float>& nameValues, int timeout)
  {
    log() << "ddcGetCrateParameters: " << crate << " " << timeout << std::endl;
    ddcCmd()->getCrateParameters(crate, names, nameValues, timeout);
  }

  void SctApiDDC::ddcSetChannelParameter (int crate, int channel, std::string name, float value, int timeout)
  {
    log() << "ddcSetChannelParameter: " << crate << " " << channel
          << ", (" << name << "), " << value << " " << timeout << std::endl;

    ddcCmd()->setChannelParameter(crate, channel, name, value, timeout);
  }

  void SctApiDDC::ddcSetCardParameter (int crate, int card, std::string name, float value, int timeout)
  {
    log() << "ddcSetCardParameter: " << crate << " " << card
          << ", (" << name << "), " << value << " " << timeout << std::endl;

    ddcCmd()->setCardParameter(crate, card, name, value, timeout);
  }

  void SctApiDDC::ddcSetCrateParameter (int crate, std::string name, float value, int timeout)
  {
    log() << "ddcSetCrateParameter: " << crate 
          << ", (" << name << "), " << value << " " << timeout << std::endl;

    ddcCmd()->setCrateCtrlParameter(crate, name, value, timeout);
  }

}
