
#ifndef SCT_SCTAPI_TRIGGER_H
#define SCT_SCTAPI_TRIGGER_H

/// Trigger interface



//fwd dec
namespace SctApi {
  class Trigger;
};

//includes
#include "CommonWithDsp/processor.h"
#include <utility> // for std::pair
#include <vector>

//defn
namespace SctApi {

  class Trigger {
  public:
    virtual ~Trigger() {};

    /// Type for representation as a series of pairs as defined for ROD DSP
    typedef std::pair<UINT16, UINT32>  RODTrigger;
    typedef std::vector<RODTrigger>    RODTriggers;

    /// Set to a single L1A
    virtual void singleL1A() = 0;
    /// Set to a double L1A
    virtual void doubleL1A(unsigned short delay) = 0;
    /// Set to a delayed L1A
    virtual void delayedL1A(unsigned short delay) = 0;
    /// Set to a Calibration pulse followed by an L1A BC0s later
    virtual void calL1A(unsigned short delay) = 0;
    virtual void pulseL1A(unsigned short delay) = 0;
    virtual void softL1A(unsigned short delay) = 0;
    virtual void softCalL1A(unsigned short delay, unsigned short delay2) = 0;
    virtual void softPulseL1A(unsigned short delay, unsigned short delay2) = 0;
    virtual void softBc(unsigned short delay) = 0;
    virtual void soft() = 0;
    virtual void bc() = 0;
    virtual void bcL1A(unsigned short delay) = 0;
    virtual void bcCalL1A(unsigned short delay, unsigned short delay2) = 0;
    virtual void bcPulseL1A(unsigned short delay, unsigned short delay2) = 0;

    virtual void setCommIncr(unsigned short command, unsigned short incr) = 0;
    virtual void getCommIncr(unsigned short &command, unsigned short &incr) const = 0;

    /// Get list of triggers in ROD format
    virtual const RODTriggers getRODTriggers() const = 0; 

    virtual void print() const = 0;
  };

};

#endif
