
// Many (but not all) parts of ApiServerStatusManager can be removed .. such as
// the "GeneralStatus" parts.

#include "ApiServerStatusManager.h"

#include "Sct/SctNames.h"
#include "Sct/MultiMessageDebugStream.h"
#include "is/isinfo.h"
#include "is/isinfotmpl.h"
#include "mrs/message.h"


Sct::ApiServerStatusManager::ApiServerStatusManager(const std::string & infoServiceName // something to distunguish us from all the other SctApiIPC{Crate,Top}Servers that might be running. Should take the form SERVER.NAME
                                                    ) :
  m_isDict(boost::shared_ptr<ISInfoDictionary>(new ISInfoDictionary(Sct::SctNames::getPartition()))),
  m_infoServiceName(infoServiceName),
  m_hasFinishedScanning(true) {
};

//std::string Sct::ApiServerStatusManager::nameOfGeneralStatusISObject() const {
//  return m_gsName;
//};

std::string Sct::ApiServerStatusManager::nameOfScanningStatusISObject() const {
  return m_infoServiceName;
};

//void Sct::ApiServerStatusManager::publishGeneralStatusForFirstTime() {
//  ISInfo::Status s = m_isDict->contains(this->nameOfGeneralStatusISObject());
//  if(s==ISInfo::Success) {
//    Sct::MultiMessageDebugStream m(true,false,true);
//    m << "*** Filename ["<< (this->nameOfGeneralStatusISObject()) <<"] already in IS, will run anyway but there could be another service running?\n";
//  } else {
//   ISInfoInt isPresent(ApiServerStatusManager::present);
//    s=m_isDict->insert(this->nameOfGeneralStatusISObject(), isPresent);
//    if (s!=ISInfo::Success){
//      Sct::MultiMessageDebugStream m(true,false,true);
//      m << "Problem publishing [" << (this->nameOfGeneralStatusISObject()) 
//        << "] in " << __FILE__ << " at line " <<  __LINE__;
//      m.flush();
//      Sct::SctNames::Mrs() << "IS_ERROR" << MRS_WARNING << MRS_QUALIF(m_uniqueIdentifier.c_str()) << MRS_TEXT("Error in ApiServerStatusManager.cc publishing presence to IS") << MRS_PARAM<int>("Status",static_cast<int>(s)) << ENDM;
//    }
//  }
//}

//void Sct::ApiServerStatusManager::setGeneralStatusTo(const GeneralStatusEnum i) {
//  ISInfoInt present(i);
//  m_isDict->update(this->nameOfGeneralStatusISObject(), present);
//};


//void Sct::ApiServerStatusManager::withdrawGeneralStatus() {
//  m_isDict->remove(this->nameOfGeneralStatusISObject());
//}

void Sct::ApiServerStatusManager::publishScanningState() {
  m_hasFinishedScanning = false;
  ISInfoInt scanning(1);
  m_isDict->insert(this->nameOfScanningStatusISObject(), scanning);
}

void Sct::ApiServerStatusManager::withdrawScanningState() {
  m_hasFinishedScanning = true;
  m_isDict->remove(this->nameOfScanningStatusISObject());
}

void Sct::ApiServerStatusManager::refreshCache() {
  const ISInfo::Status badlyTypedReturnValue 
    = m_isDict->contains(this->nameOfScanningStatusISObject());
  switch (badlyTypedReturnValue) {
  case ISInfo::NotFound:
    // so the IS server existed, but it doesn't contain the "scanning" remark, so probably scanning has stopped
    m_hasFinishedScanning = true;
    return;
  case ISInfo::Success:
    // so the IS server existed, and it does contain the "scanning" remark, so scanning has not yet finished (it is still going on).
    m_hasFinishedScanning = false;
    return;
  default:
    // Something else happened, which should worry us, because this suggests that the method is being called out of context.  Complain to the user, and do not change the state of the cache!
    {
      Sct::MultiMessageDebugStream m(true,false,true);
      m << "ApiServerStatusManager was asked to update its cache for " 
	<< (this->nameOfScanningStatusISObject())
	<< " but there was a problem: an unusual ISInfo::Status (value "
	<< badlyTypedReturnValue
	<< ") was returned while querying IS.  Possible causes might include (but be limited to) running the sw without starting the appropriate is_servers, a software bug, or high network load causing poor CORBA connections, eg timeouts.";
      m.flush();
    };
    return;
  };

};

bool Sct::ApiServerStatusManager::hasFinishedScanning() {
  if (!m_hasFinishedScanning) {
    // our cache var has indicated that scanning is still going on, but perhaps it is out of date ... let's refresh the cache:
    refreshCache();
  };
  return m_hasFinishedScanning;
}
