/**************** ScanServer ****************/

#include "ScanServer.h"

#include "Sct/SctNames.h"
#include "SctApiServer.h"
#include "TriggerServer.h"
#include "Sct/SequenceUtils.h"
#include "TriggerWrapper.h"
#include "Sct/OmniMarshalling.h"
#include <boost/date_time/posix_time/posix_time.hpp>
#include "SctApi/ScanDefImpl.h"
#include "SctApi/TriggerImpl.h"

ScanServer::ScanServer() 
  : IPCNamedObject<POA_Sct_SctApi::Scan>(Sct::SctNames::getPartition(),
					 getServerName().c_str()), 
    m_worker(new SctApi::ScanDefImpl())
{
  std::cout << "Create a new ScanServer\n";

//    dynamic_cast<::Scan*>(this)->print();
//   std::cout << "trigsPerBurst = " << worker().getNTrigs() << std::endl; 

  boost::shared_ptr< ::SctApi::Trigger> trig1 
    = boost::shared_ptr< ::SctApi::Trigger>(new ::SctApi::TriggerImpl);
  boost::shared_ptr< ::SctApi::Trigger> trig2 
    = boost::shared_ptr< ::SctApi::Trigger>(new ::SctApi::TriggerImpl);

  std::cout << "TrigServer : " ;
  worker().setTrigger1( trig1 );
  worker().setTrigger2( trig2 );
  std::cout << " getTrigger1() " << worker().getTrigger1();
  std::cout << " getTrigger2() " << worker().getTrigger2();
  std::cout << std::endl;

//    if(!publish()) {
//      cerr << "Failed to publish new scan" << std::endl;
//    }

//    std::cout << "Published scan\n";
  std::cout << " ... ScanServer created\n";
}

ScanServer::ScanServer(const ScanServer &copy) 
  : IPCNamedObject<POA_Sct_SctApi::Scan>(Sct::SctNames::getPartition(),
					 getServerName().c_str()),
    m_worker(SctApi::ScanDefImpl::clone(copy.m_worker))
{
  std::cout << "Create a copy of a ScanServer\n";

//   TriggerServer *temp = new TriggerServer(ipcServer.get());
//   worker().setTrigger1(IPCObjectVar<TriggerServer>::duplicate(temp)->m_worker);

//   TriggerServer *temp2 = new TriggerServer(ipcServer.get());
//   worker().setTrigger2(IPCObjectVar<TriggerServer>::duplicate(temp2)->m_worker);

#warning "Need getTrigger to create new TriggerServer if none found?"

  std::cout << " ... ScanServer copied\n";
}

ScanServer::~ScanServer() 
{
//    return IPCObjectVar<ScanServer>::release(this);
  
//    withdraw();
}

void ScanServer::shutdown() 
{
  SctApiServer::Scans &scans = SctApiServer::singleton->scans; // There might be an issue of scan ownership here over multicrate oeration

  for (SctApiServer::Scans::iterator item = scans.begin();
       item != scans.end();
       ++item) {
    if ((*item).get() == this) {
      scans.erase(item);
      return;
    };
  };
  std::cout << "*******Couldn't find destroyed scan in scans list**********!\n";
}


int ScanServer::nextNumber = 0;

std::string ScanServer::getServerName() {
  std::ostringstream serverName;
  serverName << "ScanServer" << nextNumber++;
  return serverName.str();
}

void ScanServer::print() 
{
  std::cout << "Try to print a scan\n";
  worker().print();
  std::cout << "Gave up or sucess (printing scan)\n";
}

void ScanServer::configure (CORBA::UShort type, CORBA::Double start, CORBA::Double stop, CORBA::Double step) 
{
  worker().configure(type, start, stop, step);
}

void ScanServer::configure2 (CORBA::UShort type, CORBA::Double start, CORBA::Double stop, CORBA::Double step) 
{
  worker().configure2(type, start, stop, step);
}

Sct_SctApi::Trigger_ptr ScanServer::getTrigger1 () 
{
  const boost::shared_ptr< ::SctApi::Trigger> t = worker().getTrigger1();
  TriggerServer * ts = new TriggerServer(t);
  // Now at this point one might worry that ts might never be deleted, but, the intention is that since TriggerServer inherits from PortableServer::RefCountServantBase then as soon as the last ref to it is removed, then it should be deleted.  Touch wood.  So we had better make the first ref: -- now known to be incorrect -- see update on next line.
  // Actually it was found that  PortableServer::RefCountServantBase didn't 
  // do what we thougt it did, so a new Sct::RefCountServantBase has been created
  // which is what TriggerServer *now* derives from.  This *does* do what we want, provided that users call TriggerServer::_sct_remove_ref() (C++ interface) or Trigger_ptr->removeRef() (CORBA interface) (as well as the converse ..add..ref) methods appropriately. 
  return ts->_this();
}

Sct_SctApi::Trigger_ptr ScanServer::getTrigger2 () 
{
  const boost::shared_ptr< ::SctApi::Trigger> t = worker().getTrigger2();
  TriggerServer * ts = new TriggerServer(t);
  // Now at this point one might worry that ts might never be deleted, but, the intention is that since TriggerServer inherits from PortableServer::RefCountServantBase then as soon as the last ref to it is removed, then it should be deleted.  Touch wood.  So we had better make the first ref: -- now known to be incorrect -- see update in method above.
  return ts->_this();
}

void ScanServer::setTrigger1 (Sct_SctApi::Trigger_ptr trig)
{
  //  TriggerServer *server = dynamic_cast<TriggerServer *>(trig);
  boost::shared_ptr< ::SctApi::Trigger> trigWrapper 
    = boost::shared_ptr< ::SctApi::Trigger>(new TriggerWrapper(trig, TriggerWrapper::doNotTakeOwnershipOfObjectReference));
  worker().setTrigger1(trigWrapper);
}

void ScanServer::setTrigger2 (Sct_SctApi::Trigger_ptr trig) 
{
  //  TriggerServer *server = dynamic_cast<TriggerServer *>(trig);
  boost::shared_ptr< ::SctApi::Trigger> trigWrapper 
    = boost::shared_ptr< ::SctApi::Trigger>(new TriggerWrapper(trig, TriggerWrapper::doNotTakeOwnershipOfObjectReference));
  worker().setTrigger2(trigWrapper);
}

void ScanServer::setScanVariable1 (CORBA::UShort var) 
{
  worker().setScanVariable1(var);
}

void ScanServer::setScanVariable2 (CORBA::UShort var) 
{
  worker().setScanVariable2(var);
}

CORBA::UShort ScanServer::getScanVariable1 () 
{
  return worker().getScanVariable1();
}

CORBA::UShort ScanServer::getScanVariable2 () 
{
  return worker().getScanVariable2();
}

void ScanServer::setNTrigs (CORBA::ULong nTrigs) 
{
  worker().setNTrigs(nTrigs);
}

CORBA::ULong ScanServer::getNTrigs () 
{
  return worker().getNTrigs();
}

/*
// Hopefully this function is now obsolete.
static void enumerateToLongList(CORBA::ULong val, void *list) {
SctApi::Scan::TrigPoints &myList = *(SctApi::Scan::TrigPoints *)list;

myList.push_back(val);
}
*/

Sct_SctApi::Scan::TrigPoints * ScanServer::getVariableTrigs() {
  Sct_SctApi::Scan::TrigPoints * result = Sct::SequenceUtils::newSafeSequence<Sct_SctApi::Scan::TrigPoints>(worker().getVariableTrigs().size());

  for(unsigned long i=0; i<worker().getVariableTrigs().size(); ++i) {
    (*result)[i] = worker().getVariableTrigs()[i];
  }

  return result;
};

void ScanServer::setVariableTrigs(const Sct_SctApi::Scan::TrigPoints& scans) {
  SctApi::Scan::TrigPoints newPoints(scans.length());
  for (unsigned int i=0; i<scans.length(); ++i) {
    newPoints[i] = scans[i];
  };
  worker().setVariableTrigs(newPoints);
};

void ScanServer::setVariableTrigRange(CORBA::UShort start, CORBA::UShort end, CORBA::ULong value) {
  worker().setVariableTrigRange(start, end, value);
};

Sct_SctApi::Scan::ScanPoints * ScanServer::getScanPoints1 () 
{
  Sct_SctApi::Scan::ScanPoints * result = Sct::SequenceUtils::newSafeSequence<Sct_SctApi::Scan::ScanPoints>(worker().getScanPoints1().size());

  for(unsigned long i=0; i<worker().getScanPoints1().size(); ++i) {
    (*result)[i] = worker().getScanPoints1()[i];
  }

  return result;
}

Sct_SctApi::Scan::ScanPoints * ScanServer::getScanPoints2 ()
{
  Sct_SctApi::Scan::ScanPoints * result = Sct::SequenceUtils::newSafeSequence<Sct_SctApi::Scan::ScanPoints>(worker().getScanPoints2().size());

  for(unsigned long i=0; i<worker().getScanPoints2().size(); ++i) {
    (*result)[i] = worker().getScanPoints2()[i];
  }

  return result;
}

/*
// Hopefully this function is now obsolete.
static void enumerateToRealList(CORBA::Double val, void *list) {
SctApi::Scan::ScanPoints &myList = *(SctApi::Scan::ScanPoints *)list;

myList.push_back(val);
}
*/

void ScanServer::setScanPoints1 (const Sct_SctApi::Scan::ScanPoints & scans) 
{
  SctApi::Scan::ScanPoints newPoints(scans.length());
  for (unsigned int i=0; i<scans.length(); ++i) {
    newPoints[i] = scans[i];
  };
  //scans.Enumerate(enumerateToRealList, &newPoints);
  worker().setScanPoints1(newPoints);
}
 
void ScanServer::setScanPoints2 (const Sct_SctApi::Scan::ScanPoints & scans) 
{
  SctApi::Scan::ScanPoints newPoints(scans.length());
  for (unsigned int i=0; i<scans.length(); ++i) {
    newPoints[i] = scans[i];
  };
  //scans.Enumerate(enumerateToRealList, &newPoints);
  worker().setScanPoints2(newPoints);
}

CORBA::Long ScanServer::getOption (Sct_SctApi::Scan::ScanOptions opt) 
{
  return worker().getOption((SctApi::Scan::ScanOptions)opt);
}

void ScanServer::setOption (Sct_SctApi::Scan::ScanOptions opt, CORBA::Long val) 
{
  worker().setOption((SctApi::Scan::ScanOptions)opt, val);
}

void ScanServer::setScanNumber (CORBA::UShort scan) {
  worker().setScanNumber(scan);
};

void ScanServer::setRunNumber (CORBA::UShort scan) {
  worker().setRunNumber(scan);
};


CORBA::ULong ScanServer::getScanNumber () 
{
  return worker().getScanNumber();
}
 
CORBA::ULong ScanServer::getRunNumber () 
{
  return worker().getRunNumber();
}


sctConf::ModuleList * ScanServer::getModuleList (CORBA::ULong group) 
{

  std::list<std::string> myList = worker().getModuleList(group);
  sctConf::ModuleList * result = Sct::SequenceUtils::newSafeSequence<sctConf::ModuleList>(myList.size());
  unsigned int i=0;
  for(std::list<std::string> :: const_iterator m = myList.begin();
      m!=myList.end();
      (++i,++m)) {
    (*result)[i] = CORBA::string_dup((*m).c_str());
  }

  return result;
}

void ScanServer::setModuleList (CORBA::ULong group, const sctConf::ModuleList& newList) {

  ::SctApi::ModuleList tmp;
  const unsigned int siz = newList.length();

  for (unsigned int i=0; i<siz; ++i) {
    tmp.push_back(   (newList[i])._NP_ref()   );
  };

  worker().setModuleList( group, tmp );

};




CORBA::ULong ScanServer::getNGroups () 
{
  return worker().getNGroups();
}
char * ScanServer::getStartTime()
{
  std::string startTime =  boost::posix_time::to_iso_string(worker().getStartTime()).substr(0, 15); 
  return Sct::copyStringToCorba(startTime);
}
char * ScanServer::getEndTime()
{
  std::string endTime = boost::posix_time::to_iso_string(worker().getEndTime()).substr(0, 15); 
  return Sct::copyStringToCorba(endTime);    
}

void ScanServer::setNGroups(CORBA::UShort val) {
  worker().setNGroups(val);
}
void ScanServer::setStartTime(const char* t) {
  const boost::posix_time::ptime pt =  boost::posix_time::from_iso_string(t);
  worker().setStartTime(pt);
}
void ScanServer::setEndTime(const char* t) {
  const boost::posix_time::ptime pt =  boost::posix_time::from_iso_string(t);
  worker().setEndTime(pt);
}


SctApi::Scan &ScanServer::worker() 
{
  return *m_worker;
}

