#ifndef SCTAPIIPC_SCANSERVER_H
#define SCTAPIIPC_SCANSERVER_H

// CORBA wrapper for SctApi::Scan class

// fwd dec
class ScanServer;

//includes
#include "Sct_SctApi/SctApi.hh"
#include <ipc/object.h>
#include <boost/shared_ptr.hpp>

// fwd dec others:
namespace SctApi {
  class Scan;
};

//declaration
class ScanServer : public IPCNamedObject<POA_Sct_SctApi::Scan>
{
  ScanServer(const ScanServer &);
  ScanServer &operator=(const ScanServer &);

  static int nextNumber;

 public:
  ScanServer();

  //ScanServer(const ScanServer &copy);

  virtual ~ScanServer();

  virtual void print ();
  virtual void configure (CORBA::UShort type, CORBA::Double start, CORBA::Double stop, CORBA::Double step);
  virtual void configure2 (CORBA::UShort type, CORBA::Double start, CORBA::Double stop, CORBA::Double step);
  virtual Sct_SctApi::Trigger_ptr getTrigger1 ();
  virtual Sct_SctApi::Trigger_ptr getTrigger2 ();
  virtual void setTrigger1 (Sct_SctApi::Trigger_ptr trig);
  virtual void setTrigger2 (Sct_SctApi::Trigger_ptr trig);
  virtual void setScanVariable1 (CORBA::UShort var);
  virtual void setScanVariable2 (CORBA::UShort var);
  virtual CORBA::UShort getScanVariable1 ();
  virtual CORBA::UShort getScanVariable2 ();
  virtual void setNTrigs (CORBA::ULong nTrigs);
  virtual CORBA::ULong getNTrigs ();
  virtual Sct_SctApi::Scan::TrigPoints * getVariableTrigs();
  virtual void setVariableTrigs(const Sct_SctApi::Scan::TrigPoints& scans);
  virtual void setVariableTrigRange(CORBA::UShort start, CORBA::UShort end, CORBA::ULong value);
  virtual Sct_SctApi::Scan::ScanPoints * getScanPoints1 ();
  virtual Sct_SctApi::Scan::ScanPoints * getScanPoints2 ();
  virtual void setScanPoints1 (const Sct_SctApi::Scan::ScanPoints & scans);
  virtual void setScanPoints2 (const Sct_SctApi::Scan::ScanPoints & scans);
  virtual CORBA::Long getOption (Sct_SctApi::Scan::ScanOptions opt);
  virtual void setOption (Sct_SctApi::Scan::ScanOptions opt, CORBA::Long val);
  virtual void setScanNumber (CORBA::UShort scan);
  virtual void setRunNumber (CORBA::UShort scan);
  virtual CORBA::ULong getScanNumber ();
  virtual CORBA::ULong getRunNumber ();
  virtual sctConf::ModuleList * getModuleList (CORBA::ULong group);
  virtual void setModuleList (CORBA::ULong group, const sctConf::ModuleList& newList);
  virtual CORBA::ULong getNGroups ();
  virtual void setNGroups(CORBA::UShort val);
  virtual char * getStartTime();
  virtual void setStartTime(const char* t);
  virtual char * getEndTime();
  virtual void setEndTime(const char* t);

  // Override default freeable
#warning LESTER is guessing that the old destroy method corresponds to the new shutdown, so ... there is ALSO a similar method in the ScanServer class which I have similarly altered.
  // LESTER WAS
  // virtual void destroy(ipcStatus *_status);
  // LESTER BECOMES
  virtual void shutdown();

  boost::shared_ptr<SctApi::Scan> m_worker;

 protected:
  static std::string getServerName();

 protected:
  SctApi::Scan &worker();

};

#endif
