#include <unistd.h>
#include <stdlib.h>
#include <iostream>

#include <mrs/message.h>

#include <algorithm>
#include <sstream>
#include <memory>

#include "Sct_SctApi/SctApi.hh"
#include "SctApiCrateServer.h"
#include "sctConfIPC/configipc.h"
#include "SctApi/autoConfig.h"

#include "Sct/MultiMessageDebugStream.h"
#include "Sct/SctNames.h"
#include "Sct/OmniMarshalling.h"
#include "Sct/DestroyingDeleter.h"
#include "sctConfIPC/marshalling.h"
#include "marshalling.h"

#include "SctApi/SctApi.h"

#include "TriggerWrapper.h"
#include    "ScanWrapper.h"

#include "Sct/SequenceUtils.h"
#include "Sct/StringStreamer.h"
#include "SctApi/SctApiException.h"
#include "IdiosyncrasyServer.h"

#include <boost/date_time/posix_time/posix_time.hpp>

using namespace Sct;
using namespace std;
//using namespace boost::posix_time;

#include "LocalUtils.h"
#include "ScanServer.h"
#include "TriggerServer.h"
#include "ApiServerStatusManager.h"

/*
  CORBA::ULong is long unsigned int
 */

SctApiCrateServer *SctApiCrateServer::singleton = 0;

SctApiCrateServer::SctApiCrateServer(SctApi::SctApi& api) : 
  IPCNamedObject<POA_Sct_SctApi::CrateIPC>(Sct::SctNames::getPartition(),
                                           api.idiosyncrasy().ipcNameOfSctApiCrateServer()),
  m_assm(boost::shared_ptr<Sct::ApiServerStatusManager>
         (new Sct::ApiServerStatusManager(api.idiosyncrasy().infoServiceNameOfScanStatusObject()))),
  m_idServer(new SctApi::IdiosyncrasyServer(api.idiosyncrasy())),
  scans(0), 
  defaultScan(0),
  defaultTrigger(0),
  mrsStream(Sct::SctNames::getPartition()), 
  // NOT NEEDED ddcSingleTimeout(3), ddcMultipleTimeout(15),
  api(api)
{
  
  defaultScan = new ScanServer;
  api.setMrsStream(&mrsStream);
  api.setIsDictionary();

  mrsStream << "SCTAPICRATE_CONSTRUCT" << MRS_INFORMATION << MRS_QUALIF("SctApiCrateServer") << MRS_TEXT("Constructor called") << ENDM;

  // REMOVE! m_assm->publishGeneralStatusForFirstTime();

  try {
    publish(); // publish should be late in the constructor so that we don't have to remember to withdraw() in the event that something following publish() were to throw!
  } catch (CORBA::Exception & e) {
    try {
      Sct::MultiMessageDebugStream m(true,true,true);
      m << "CORBA::Exception [" << e._name() << "] when publishing SctApiCrateServer in " << __FILE__<<" at line " << __LINE__;
      m.flush();
    } catch (...) {
      // fall through
    };
    throw;
  };
  singleton = this;
}

SctApiCrateServer::~SctApiCrateServer() 
{
  mrsStream << "SCTAPI_DESTRUCT" << MRS_INFORMATION << MRS_QUALIF("SctApiCrateServer") << MRS_TEXT("Destructor called") << ENDM;

  //REMOVE! m_assm->withdrawGeneralStatus();

  singleton = 0;

  withdraw();
  //deactivate();

  std::cout << "Forcefully deleting " << scans.size() << " scans in CrateServer\n";
  scans.clear();
  std::cout << "Forcefully deleted " << scans.size() << " scans in CrateServer\n";
  /*
    // This old method is (hopefuly) no longer needed now that elements of "scans" have custom delters!
    for(Scans::iterator i = scans.begin();
      i!= scans.end(); 
      i++) {
    delete *i;
  }
  */

#warning LESTER SUSPECTS THAT THE AUTHORS OF THIS ROUTINE SHOULD ALSO BE FREEING THINGS LIKE THE DEFAULT SCAN AND THE DEFAULT TRIGGER!
}

Sct_SctApi::Idiosyncrasy_ptr SctApiCrateServer::getIdiosyncrasy() {
  return m_idServer->_this();
};

void SctApiCrateServer::initialiseAll (CORBA::ULong runNumber)
{
  try {
    mrsStream << "SCTAPI_INIT" << MRS_INFORMATION << MRS_QUALIF("SctApiCrateServer") << MRS_TEXT(Sct::StringStreamer<< "initialiseAll called with run number "<< runNumber) << ENDM;

    worker().initialiseAll(runNumber);

    // REMOVE! m_assm->setGeneralStatusTo(Sct::ApiServerStatusManager::initialised);

  } catch(SctApi::SctApiException &s) {
    cout << "Exception thrown by initialiseAll: " << s.what() << endl;
    throw myRaise(s);
  }
}

void SctApiCrateServer::shutdownAll ()
{
  try {
    mrsStream << "SCTAPI_SHUTDOWN" << MRS_INFORMATION << MRS_QUALIF("SctApiCrateServer") << MRS_TEXT("shutdownAll called") << ENDM;

    worker().shutdownAll();

    // REMOVE! m_assm->setGeneralStatusTo(Sct::ApiServerStatusManager::present);

  } catch(SctApi::SctApiException &s) {
    cout << "Exception thrown by shutdownAll: " << s.what() << endl;
    throw myRaise(s);
  }
}

void SctApiCrateServer::status ()
{
  worker().status();
}

void SctApiCrateServer::awaitResponse (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod)
{
  try {
    worker().awaitResponse((unsigned int)rod, 5);
  } catch(SctApi::SctApiException &s) {
    cout << "Exception thrown by awaitResponse: " << s.what() << endl;
    throw myRaise(s);
  }
}

Sct_SctApi::DataBlock * SctApiCrateServer::getResponse (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod)
{
  unsigned long length;

  unsigned long *body = worker().getResponse((unsigned int)rod, length);

  return copyArrayToCorba<Sct_SctApi::DataBlock, unsigned long>(body, length);
}

void SctApiCrateServer::configureBOC (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod)
{
  try {
    worker().configureBOC((unsigned int)rod);
  } catch(SctApi::SctApiException &s) {
    cout << "Exception thrown by configureBOC: " << s.what() << endl;
    throw myRaise(s);
  }
}

Sct_SctApi::BOCMonitorSequence * SctApiCrateServer::getBOCMonitorArray (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod)
{
  try {
    const std::vector<double> workersBOCMonitorVec = worker().getBOCMonitorArray(rod);
    return copyVectorToCorba<Sct_SctApi::BOCMonitorSequence, double>(workersBOCMonitorVec);
  } catch(SctApi::SctApiException &s) {
    cout << "Exception thrown by getBOCMonitorArray: " << s.what() << endl;
    throw myRaise(s);
  }
}

void SctApiCrateServer::flashLED (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod, CORBA::UShort slaveNumber, CORBA::ULong period, CORBA::ULong flashes)
{
  try {
    worker().flashLED((unsigned int)rod, slaveNumber, period, flashes);
  } catch(SctApi::SctApiException &s) {
    cout << "Exception thrown by flashLED: " << s.what() << endl;
    throw myRaise(s);
  }
}

void SctApiCrateServer::echo (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod, const Sct_SctApi::DataBlock & data)
{
  try {
    unsigned long ourLength = data.length();

    unsigned long *array = new unsigned long[ourLength];
    unsigned long *body = copyCorbaToArray(data, array);

    worker().echo((unsigned int)rod, (unsigned int)ourLength, body);
  } catch(SctApi::SctApiException &s) {
    cout << "Exception thrown by echo: " << s.what() << endl;
    throw myRaise(s);
  }
}

void SctApiCrateServer::createDebugPrimList ()
{
  try {
    worker().createDebugPrimList();
  } catch(SctApi::SctApiException &s) {
    cout << "Exception thrown by createDebugPrimList: " << s.what() << endl;
    throw myRaise(s);
  }
}

void SctApiCrateServer::addDebugPrimList (CORBA::Long index, CORBA::Long id, CORBA::Long version, const Sct_SctApi::DataBlock & body)
{
  try {
    unsigned long ourLength = body.length();
    unsigned long *array = new unsigned long[ourLength];
    copyCorbaToArray(body, array);

    worker().addDebugPrimList(ourLength, index, id, version, array);
  } catch(SctApi::SctApiException &s) {
    cout << "Exception thrown by addDebugPrimList: " << s.what() << endl;
    throw myRaise(s);
  }
}

void SctApiCrateServer::sendDebugPrimList (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod)
{
  try {
    worker().sendDebugPrimList((unsigned int)rod);
  } catch(SctApi::SctApiException &s) {
    cout << "Exception thrown by sendDebugPrimList: " << s.what() << endl;
    throw myRaise(s);
  }
}

void SctApiCrateServer::sendDebugSlavePrimList (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod, CORBA::ULong slave, CORBA::Boolean await, CORBA::Boolean response)
{
  try {
    worker().sendDebugSlavePrimList((unsigned int)rod, slave, await, response);
  } catch(SctApi::SctApiException &s) {
    cout << "Exception thrown by sendDebugSlavePrimList: " << s.what() << endl;
    throw myRaise(s);
  }
}

void SctApiCrateServer::setRunNumber (CORBA::ULong newRun)
{
  worker().setRunNumber(newRun);
}

void SctApiCrateServer::setScanNumber (CORBA::ULong newScan)
{
  worker().setScanNumber(newScan);
}

CORBA::ULong SctApiCrateServer::getRunNumber ()
{
  return worker().getRunNumber();
}

CORBA::ULong SctApiCrateServer::getScanNumber ()
{
  return worker().getScanNumber();
}

void SctApiCrateServer::unsetDebugOption (const char * opt)
{
  worker().unsetDebugOption(opt);
}

void SctApiCrateServer::setDebugOption (const char * opt)
{
  worker().setDebugOption(opt);
}

Sct_SctApi::DebugOptionList * SctApiCrateServer::listEnabledDebugOptions ()
{

  std::list<std::string> myList = worker().listEnabledDebugOptions();
  Sct_SctApi::DebugOptionList * result = SequenceUtils::newSafeSequence<Sct_SctApi::DebugOptionList>(myList.size());
  unsigned int i=0;
  for(std::list<std::string> :: const_iterator m = myList.begin();
      m!=myList.end();
      (++m,++i)) {
    (*result)[i] = CORBA::string_dup((*m).c_str());
  }

  return result;
}

Sct_SctApi::DebugOptionList * SctApiCrateServer::listDebugOptions ()
{
  std::vector<std::string> myList = worker().listDebugOptions();

  Sct_SctApi::DebugOptionList * result = SequenceUtils::newSafeSequence<Sct_SctApi::DebugOptionList>(myList.size());
  unsigned int i=0;
  for(std::vector<std::string> :: const_iterator m = myList.begin();
      m!=myList.end();
      (++m,++i)) {
    (*result)[i] = CORBA::string_dup((*m).c_str());
  }

  return result;
}

void SctApiCrateServer::standardRegisterDumpRod (CORBA::ULong rod)
{
  worker().standardRegisterDump(rod);
}

void SctApiCrateServer::standardRegisterDumpAll ()
{
  worker().standardRegisterDumpAll();
}

sctConf::ModuleList * SctApiCrateServer::getModuleList () 
{
  // LESTER result = _sctConf_T_ModuleList_sequence::Create(4, NULL);
  // LESTER result->Clear(0);

  std::list<std::string> myList = worker().getModuleList();
  sctConf::ModuleList * result = SequenceUtils::newSafeSequence<sctConf::ModuleList>(myList.size());

  unsigned int i=0;
  for(std::list<std::string> :: const_iterator m = myList.begin();
      m!=myList.end();
      (++m, ++i)) {
    (*result)[i] = CORBA::string_dup((*m).c_str());
  }

  return result;
}

CORBA::Boolean SctApiCrateServer::isRODPresent (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod)
{
  return worker().isRODPresent(rod);
}

void SctApiCrateServer::loadConfiguration (const char * filename)
{
  try {
    worker().loadConfiguration(filename);
  } catch(SctApi::SctApiException &s) {
    cout << "Exception thrown by loadConfiguration: " << s.what() << endl;
    throw myRaise(s);
  }
}

void SctApiCrateServer::storeModuleConfigurations ()
{
  try {
    worker().storeModuleConfigurations();
  } catch(SctApi::SctApiException &s) {
    cout << "Exception thrown by storeModuleConfigurations: " << s.what() << endl;
    throw myRaise(s);
  }
}

void SctApiCrateServer::storeBOCConfigurations ()
{
  try {
    worker().storeBOCConfigurations();
  } catch(SctApi::SctApiException &s) {
    cout << "Exception thrown by storeModuleConfigurations: " << s.what() << endl;
    throw myRaise(s);
  }
}

void SctApiCrateServer::configureAllModules ()
{
  try {
    worker().sendAllABCDModules(myTranslateBank(Sct_SctApi::PHYSICS_CONFIG));
  } catch(SctApi::SctApiException &s) {
    cout << "Exception thrown by configureAllModules: " << s.what() << endl;
    throw myRaise(s);
  }
}

sctConf::ABCDModule SctApiCrateServer::retrieveModule (CORBA::ULong mid)
{
  ABCDModule *body = worker().retrieveModule(mid);

  return copyABCDModuleToCorba(*body);
}

CORBA::Short SctApiCrateServer::dspBlockDump (CORBA::ULong rod, 
                                             CORBA::Long dspStart, CORBA::Long numWords, CORBA::Long dspNumber)
{
  return worker().dspBlockDump((unsigned int)rod,
			       dspStart, numWords, dspNumber);
}

Sct_SctApi::DataBlock * SctApiCrateServer::dspBlockRead (CORBA::ULong rod, CORBA::Long dspStart, CORBA::Long numWords, CORBA::Long dspNumber)
{
  unsigned long length;
  unsigned long *body = worker().dspBlockRead((unsigned int)rod,
					      dspStart, numWords, dspNumber, length);

  return copyArrayToCorba<Sct_SctApi::DataBlock, unsigned long>(body, length);
}

CORBA::Short SctApiCrateServer::dspBlockWrite (CORBA::ULong rod,
                                              const Sct_SctApi::DataBlock & buffer, CORBA::ULong dspAddress, 
                                              CORBA::Long dspNumber)
{
  unsigned long length = buffer.length();
  unsigned long *array = new unsigned long[length];
  copyCorbaToArray(buffer, array);

  return worker().dspBlockWrite((unsigned int)rod, 
                                array, dspAddress, length, dspNumber);
}

CORBA::ULong SctApiCrateServer::dspSingleRead (CORBA::ULong rod, 
                                          CORBA::ULong dspAddr, CORBA::Long dspNumber)
{
  return worker().dspSingleRead((unsigned int)rod, 
				dspAddr, dspNumber);
}

void SctApiCrateServer::dspSingleWrite (CORBA::ULong rod, CORBA::ULong dspAddr, CORBA::ULong val, CORBA::Long dspNumber)
{
  worker().dspSingleWrite((unsigned int)rod, dspAddr, val, dspNumber);
}

Sct_SctApi::DataBlock * SctApiCrateServer::readSlaveDsp (CORBA::ULong rod, CORBA::Short s, CORBA::ULong add, CORBA::ULong words)
{
  unsigned long *body = worker().primReadSlaveDsp((unsigned int)rod, s, add, words);

  return copyArrayToCorba<Sct_SctApi::DataBlock, unsigned long>(body, words);
}

CORBA::ULong SctApiCrateServer::findModuleSerial (const char * sn)
{
  return worker().findModule(sn);
}

char * SctApiCrateServer::convertToString (CORBA::ULong mid)
{
  return copyStringToCorba(worker().convertToString(mid));
}

CORBA::ULong SctApiCrateServer::findModule (CORBA::UShort mur, CORBA::UShort number)
{
  return worker().findModule(mur, number);
}

CORBA::ULong SctApiCrateServer::findModuleRod (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod, CORBA::ULong channel) 
{
  UINT32 mid = 0;
  mid |= (partition & 0xff) << 24;
  mid |= (crate & 0xff) << 16;
  mid |= (rod & 0xff) << 8;
  mid |= (channel & 0xff) << 0;
  return mid;
}

CORBA::ULong SctApiCrateServer::findBarrelModule (
                                             CORBA::ULong barrel, CORBA::ULong row, CORBA::Short number)
{
  return worker().findBarrelModule(barrel, row, number);
}

void SctApiCrateServer::convertToBarrelModule (CORBA::ULong mid, CORBA::ULong & barrel, CORBA::ULong & row, CORBA::Short & number)
{

  UINT32 tmpBarrel = barrel;
  UINT32 tmpRow    = row   ;
  int    tmpNumber = number;

  worker().convertToBarrelModule(static_cast<unsigned int>(mid), tmpBarrel, tmpRow, tmpNumber);

  barrel = tmpBarrel;
  row    = tmpRow;
  number = tmpNumber;
}

CORBA::ULong SctApiCrateServer::findEndcapModule (CORBA::Long disk, CORBA::Long ring, CORBA::Long number)
{
  return worker().findEndcapModule(disk, ring, number);
}

void SctApiCrateServer::convertToEndcapModule (CORBA::ULong mid, CORBA::Long & disk, CORBA::ULong & ring, CORBA::ULong & number)
{

   INT32 tmpDisk   = disk;
  UINT32 tmpRing   = ring;
  UINT32 tmpNumber = number;

  worker().convertToEndcapModule((unsigned int)mid, tmpDisk, tmpRing, tmpNumber);

  disk   = tmpDisk;
  ring   = tmpRing;
  number = tmpNumber;
}

void SctApiCrateServer::getABCDModule (CORBA::ULong mid, Sct_SctApi::BankType bank)
{
  try {
    worker().getABCDModule(mid, myTranslateBank(bank));
  } catch(SctApi::SctApiException &s) {
    cout << "Exception thrown by getABCDModule: " << s.what() << endl;
    throw myRaise(s);
  }
}

void SctApiCrateServer::getABCDModules (Sct_SctApi::BankType bank)
{
  try {
    worker().getABCDModules(myTranslateBank(bank));
  } catch(SctApi::SctApiException &s) {
    cout << "Exception thrown by getABCDModules: " << s.what() << endl;
    throw myRaise(s);
  }
}

sctConf::ABCDModule SctApiCrateServer::getABCDModuleRaw (CORBA::ULong rod, 
                                                       CORBA::ULong slot, Sct_SctApi::BankType bank)
{
  try {
    boost::shared_ptr<ABCDModule> body = worker().getABCDModuleRaw(rod, slot, myTranslateBank(bank));
    return copyABCDModuleToCorba(*(body.get()));
  } catch(SctApi::SctApiException &s) {
    cout << "Exception thrown by getABCDModules: " << s.what() << endl;
    throw myRaise(s);
  }
}

void SctApiCrateServer::setABCDModule (CORBA::ULong mid, const Sct_SctApi::BankList& banks)
{
  try {
    worker().setABCDModule(mid, myTranslateBanks(banks));
  } catch(SctApi::SctApiException &s) {
    cout << "Exception thrown by setABCDModule: " << s.what() << endl;
    throw myRaise(s);
  }
}

void SctApiCrateServer::setABCDModules (const Sct_SctApi::BankList& banks)
{
  try {
    worker().setABCDModules(myTranslateBanks(banks));
  } catch(SctApi::SctApiException &s) {
    cout << "Exception thrown by setABCDModules: " << s.what() << endl;
    throw myRaise(s);
  }
}

void SctApiCrateServer::copyABCDModule (CORBA::ULong mid, Sct_SctApi::BankType source, const Sct_SctApi::BankList& to){
  try {
    worker().copyABCDModule (mid, myTranslateBank(source), myTranslateBanks(to));
  } catch(SctApi::SctApiException &s) {
    cout << "Exception thrown by copyABCDModule: " << s.what() << endl;
    throw;
  }
}

void SctApiCrateServer::copyABCDModules (Sct_SctApi::BankType source, const Sct_SctApi::BankList& to){
  try {
    worker().copyABCDModules(myTranslateBank(source), myTranslateBanks(to));
  } catch(SctApi::SctApiException &s) {
    cout << "Exception thrown by copyABCDModules: " << s.what() << endl;
    throw;
  }
}

void SctApiCrateServer::sendABCDModule (CORBA::ULong mid, Sct_SctApi::BankType bank, CORBA::ULong type)
{
  try {
    worker().sendABCDModule(mid, myTranslateBank(bank), SctApi::ConfigType(type));
  } catch(SctApi::SctApiException &s) {
    cout << "Exception thrown by sendABCDModule: " << s.what() << endl;
    throw myRaise(s);
  }
}

void SctApiCrateServer::sendABCDModules (Sct_SctApi::BankType bank)
{
  try {
    worker().sendAllABCDModules(myTranslateBank(bank));
  } catch(SctApi::SctApiException &s) {
    cout << "Exception thrown by sendABCDModules: " << s.what() << endl;
    throw myRaise(s);
  }
}

void SctApiCrateServer::modifyABCDMask (CORBA::ULong mid, const Sct_SctApi::DataBlock & mask)
{
  UINT32 array[48];
  worker().modifyABCDMask(mid, copyCorbaToArray(mask, array));
}

void SctApiCrateServer::modifyABCDTrims (CORBA::ULong mid, const Sct_SctApi::CharBlock & trims){
  UINT8 array[12*128];
  worker().modifyABCDTrims (mid, copyCorbaToArray(trims, array) );
} 

void SctApiCrateServer::modifyABCDRC (CORBA::ULong mid, CORBA::ULong chip, CORBA::UShort index, CORBA::Double p0, CORBA::Double p1, CORBA::Double p2)
{
  cout << "SctApiCrateServer::modifyABCDRC called" << endl;
  worker().modifyABCDRC(mid, chip, index, p0, p1, p2);
}

void SctApiCrateServer::modifyABCDVar (CORBA::ULong typ, CORBA::Double var)
{
  worker().modifyABCDVar(typ, var);
}

void SctApiCrateServer::modifyABCDModuleVar (CORBA::ULong mid, CORBA::ULong typ, CORBA::Double var)
{
  worker().modifyABCDVar(mid, typ, var);
}

void SctApiCrateServer::modifyABCDChipVar (CORBA::ULong mid, CORBA::ULong chip, CORBA::ULong typ, CORBA::Double var)
{
  worker().modifyABCDVar(mid, chip, typ, var);
}

void SctApiCrateServer::modifyABCDVarROD (CORBA::ULong typ, CORBA::Double var, const Sct_SctApi::BankList& banks)
{
  worker().modifyABCDVarROD(typ, var, myTranslateBanks(banks));
}

void SctApiCrateServer::modifyABCDModuleVarROD (CORBA::ULong mid, CORBA::ULong typ, CORBA::Double var, const Sct_SctApi::BankList& banks)
{
  worker().modifyABCDVarROD(mid, typ, var, myTranslateBanks(banks));
}

void SctApiCrateServer::modifyBOCParam (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod, 
					CORBA::ULong channel, CORBA::ULong type, CORBA::ULong val)
{
  worker().modifyBOCParam(rod, channel, type, val);
}

void SctApiCrateServer::lasersOff ()
{
  worker().lasersOff();
}

void SctApiCrateServer::timSetFrequency (CORBA::ULong partition, CORBA::ULong crate, 
                                    CORBA::Double trigFreq, CORBA::Double rstFreq)
{
  worker().timSetFrequency(trigFreq, rstFreq);
}

void SctApiCrateServer::freeTriggers (CORBA::ULong partition, CORBA::ULong crate)
{
  worker().freeTriggers();
}

void SctApiCrateServer::stopTriggers (CORBA::ULong partition, CORBA::ULong crate)
{
  worker().stopTriggers();
}

void SctApiCrateServer::timL1A (CORBA::ULong partition, CORBA::ULong crate)
{
  worker().timL1A();
}

void SctApiCrateServer::timCalL1A (CORBA::ULong partition, CORBA::ULong crate, CORBA::Short delay)
{
  worker().timCalL1A(delay);
}

CORBA::Short SctApiCrateServer::timReadRegister(CORBA::ULong partition,
					   CORBA::ULong crate,
					   CORBA::Short value) 
{
  return worker().timReadRegister(value);
}

void SctApiCrateServer::timWriteRegister (CORBA::ULong partition, CORBA::ULong crate, CORBA::Short reg, CORBA::Short value)
{
  worker().timWriteRegister(reg, value);
}

void SctApiCrateServer::sendTimTrigger (CORBA::ULong partition,
					CORBA::ULong crate,
					Sct_SctApi::Trigger_ptr trig) 
{
  const TriggerWrapper trigWrapper(trig, TriggerWrapper::doNotTakeOwnershipOfObjectReference);
  worker().sendTimTrigger(&trigWrapper);
}

void SctApiCrateServer::sendTrigger (CORBA::ULong partition,
				     CORBA::ULong crate,
				     CORBA::ULong rod,
				     Sct_SctApi::Trigger_ptr trig)
{
  const TriggerWrapper trigWrapper(trig, TriggerWrapper::doNotTakeOwnershipOfObjectReference);
  worker().sendTrigger(rod, &trigWrapper);
}

CORBA::Boolean SctApiCrateServer::hasFinishedScanning() {
  return worker().hasFinishedScanning();
};

void SctApiCrateServer::abortScan() {
  // not yet implemented!
};

void SctApiCrateServer::doScan(Sct_SctApi::Scan_ptr thisScan)
{
  mrsStream << "SCTAPI_SCAN" << MRS_INFORMATION << MRS_QUALIF("SctApiCrateServer") << MRS_TEXT("Starting scan") << ENDM;

  m_assm->publishScanningState();

  //ScanServer *serverScan = dynamic_cast<ScanServer*>(thisScan);
  //IPCObjectVar<ScanServer>::duplicate(serverScan);
  
  const boost::shared_ptr< ::SctApi::Scan> scanWrapper 
    = boost::shared_ptr< ::SctApi::Scan>(new ScanWrapper(thisScan/*, ScanWrapper::doNotTakeOwnershipOfObjectReference*/));

  try {
    worker().doScan(scanWrapper);
  } catch(SctApi::SctApiException &s) {
    cout << "Exception thrown by doScan: " << s.what() << endl;

//     serverScan->decRef();
    m_assm->withdrawScanningState();

    mrsStream << "SCAN_ABORT" << MRS_ERROR << MRS_QUALIF("SctApiCrateServer") << MRS_TEXT((string("Scan aborted: ") + s.what()).c_str()) << ENDM;
    throw myRaise(s);
  }

  // Don't decrease reference, so polling thread can keep it
//   realScan->decRef();
}

void SctApiCrateServer::doRawScan (Sct_SctApi::Scan_ptr thisScan,
			      CORBA::Long delay,
			      CORBA::Long width,
			      CORBA::Boolean configureModules,
			      CORBA::Boolean clockByTwo)
{
  mrsStream << "SCTAPI_RAWSCAN" << MRS_INFORMATION << MRS_QUALIF("SctApiCrateServer") << MRS_TEXT("Raw scan started") << ENDM;

  m_assm->publishScanningState();

  // ScanServer *realScan = dynamic_cast<ScanServer*>(thisScan);
  //IPCObjectVar<ScanServer>::duplicate(realScan);

  boost::shared_ptr< ::SctApi::Scan> scanWrapper
    = boost::shared_ptr< ::SctApi::Scan>(new ScanWrapper(thisScan/*, ScanWrapper::doNotTakeOwnershipOfObjectReference*/));

  try {
    worker().doRawScan(scanWrapper, delay, width, configureModules, clockByTwo);
  } catch(SctApi::SctApiException &s) {
    cout << "Exception thrown by doRawScan: " << s.what() << endl;

    m_assm->withdrawScanningState();

    mrsStream << "SCAN_RAW_ABORT" << MRS_ERROR << MRS_QUALIF("SctApiCrateServer") << MRS_TEXT((string("Raw scan aborted: ") + s.what()).c_str()) << ENDM;
    throw myRaise(s);
  }

}

void SctApiCrateServer::printBOCSetup (CORBA::ULong rod)
{
  worker().printBOCSetup(rod);
}

Sct_SctApi::BOCChannelList * SctApiCrateServer::currentBOCSetup (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod) 
{
  return copyBOCChannelConfigVectorToCorba(worker().currentBOCSetup(rod));
}

sctConf::BOCGlobalConfig SctApiCrateServer::currentBOCRegisters (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod) 
{
  return copyBOCGlobalConfigToCorba(worker().currentBOCRegisters(rod));
}

void SctApiCrateServer::printABCDModule (CORBA::ULong mid)
{
  worker().printABCDModule(mid);
}

void SctApiCrateServer::printABCDRodModule (CORBA::ULong mid, Sct_SctApi::BankType bank)
{
  worker().printABCDRodModule(mid, myTranslateBank(bank));
}

void SctApiCrateServer::decodeEvent (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod, CORBA::Short sl, CORBA::Short index, CORBA::Boolean extFlag, CORBA::Boolean errorType)
{
  worker().decodeEvent(rod, sl, index, extFlag, errorType);
}

void SctApiCrateServer::rawData (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod, CORBA::Long delay, CORBA::Long units, CORBA::Boolean setMask)
{
  worker().rawData(rod, delay, units, setMask);
}

void SctApiCrateServer::rawDataWithTrigger (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod, CORBA::Long delay, CORBA::Long units, CORBA::Boolean setMask, Sct_SctApi::Trigger_ptr trig)
{
  const TriggerWrapper trigWrapper(trig, TriggerWrapper::doNotTakeOwnershipOfObjectReference);
  worker().rawData(rod, delay, units, setMask, &trigWrapper);
}

void SctApiCrateServer::sendL1A (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod)
{
  worker().sendL1A(rod);
}

Sct_SctApi::CharBlock * SctApiCrateServer::probe (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod)
{
  return copyVectorToCorba<Sct_SctApi::CharBlock, char>(worker().probe(rod));
}

Sct_SctApi::CharBlock * SctApiCrateServer::probeHarness (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod, CORBA::ULong harness)
{
  return copyVectorToCorba<Sct_SctApi::CharBlock, char>(worker().probe(rod, harness));
}

Sct_SctApi::CharBlock * SctApiCrateServer::probeWithTrigger (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod, Sct_SctApi::Trigger_ptr trig)
{
  const TriggerWrapper trigWrapper(trig, TriggerWrapper::doNotTakeOwnershipOfObjectReference);
  vector<char> ret = worker().probeWithTrigger(rod, &trigWrapper);
  return copyVectorToCorba<Sct_SctApi::CharBlock, char>(ret);
}

Sct_SctApi::CharBlock * SctApiCrateServer::probeHarnessWithTrigger (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod, Sct_SctApi::Trigger_ptr trig, CORBA::ULong harness)
{
  const TriggerWrapper trigWrapper(trig, TriggerWrapper::doNotTakeOwnershipOfObjectReference);
  vector<char> ret = worker().probeWithTrigger(rod, &trigWrapper, harness);
  return copyVectorToCorba<Sct_SctApi::CharBlock, char>(ret);
}

Sct_SctApi::CharMatrix * SctApiCrateServer::probeScanHarnessWithTrigger (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod, Sct_SctApi::Scan_ptr scan, CORBA::ULong harness)
{
  const boost::shared_ptr< ::SctApi::Scan> scanWrapper 
    = boost::shared_ptr< ::SctApi::Scan>( new ScanWrapper(scan/*, ScanWrapper::doNotTakeOwnershipOfObjectReference*/));
  vector<vector<char> > ret = worker().probeScan(rod, scanWrapper, harness);
  return copyVectorVectorToCorba<Sct_SctApi::CharMatrix, Sct_SctApi::CharBlock, char>(ret);
}

CORBA::Boolean SctApiCrateServer::checkAllModulesProbe (const char * values)
{
  return worker().checkAllModulesProbe(values);
}

Sct_SctApi::AutoResultList * SctApiCrateServer::autoConfigureScan ()
{
  ::SctApi::AutoConf::AutoConfigurer configurer(worker());
  configurer.run();

  return copyAutoResultListToCorba(configurer.getResults()); // worker().autoConfigureScan();
}

CORBA::ULong SctApiCrateServer::getRodSlot (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod)
{
  return worker().getRodSlot(rod);
}

void SctApiCrateServer::testLinkOutSelect (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod, CORBA::ULong link)
{
  worker().testLinkOutSelect(rod, link);
}

CORBA::ULong SctApiCrateServer::testLinkOutRetrieve (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod)
{
  return worker().testLinkOutRetrieve(rod);
}

Sct_SctApi::Scan_ptr SctApiCrateServer::createScan ()
{
  std::cout << "Asked to create a scan\n";

  SharedScanServer defScan = ::Sct::DestroyingDeleter<ScanServer>::shared_ptr( new ScanServer ); // , *defaultScan);

  scans.push_back(defScan);

  return defScan->_this();
}

Sct_SctApi::Scan_ptr SctApiCrateServer::getDefaultScan ()
{
  std::cout << "Asked for a reference to the default scan\n";

  return defaultScan->_this();
}

Sct_SctApi::Trigger_ptr SctApiCrateServer::createTrigger ()
{
  std::cout << "Asked to create a trigger\n";

  TriggerServer * defTrig = new TriggerServer;
#warning DOES ANYONE EVER DELETE THIS? I THINK NOT! (ChrisLester)
  
  return defTrig->_this();
}

/*
Sct_SctApi::Trigger_ptr SctApiCrateServer::getDefaultTrigger ()
{
  std::cout << "Asked for the default trigger\n";
#warning I DONT THINK ANYONE ACTUALLY SETS THE DEFAULT TRIGGER! THIS IS AN INVITATION TO SEG-FAULT! (ChrisLester)
  std::cout << "I will now do my best to segmentation fault." << std::endl;
  assert(defaultTrigger);
  return defaultTrigger->_this();
}
*/

Sct_SctApi::ScanList * SctApiCrateServer::knownScans ()
{
  std::cout << "In known scans\n";

  Sct_SctApi::ScanList * myScans = SequenceUtils::newSafeSequence<Sct_SctApi::ScanList>(scans.size());

  std::cout << "Copying sequence of length " << scans.size() << std::endl;

  unsigned int i=0;
  for(Scans::iterator it=scans.begin();
      it != scans.end();
      (++i,++it)) {
    ScanServer & thisScan = **it;
    (*myScans)[i] = thisScan._this();
  }

  return myScans;
}

SctApi::SctApi &SctApiCrateServer::worker() 
{
  return api;
}

void SctApiCrateServer::shutdown () {
#warning WHAT SHOULD SHUTDOWN DO IN SCTAPICRATESERVER.  MAYBE IT SHOULD CALL SHUTDOWNALL ??
  this->stop();
};
