
#include "SctApiDDCServer.h"


#include "Sct/SctNames.h"
#include "SctApi/SctApiDDC.h"
#include "sctConfIPC/configipc.h"
#include "marshalling.h"
#include "sctConfIPC/marshalling.h"
#include "Sct/OmniMarshalling.h"

#include <string>

using std::string;

namespace SctApiDDC {

  SctApiDDCServer::SctApiDDCServer() :
    IPCNamedObject<POA_Sct_SctApi::SctApiDDCServer>
  (Sct::SctNames::getPartition(),
   Sct_SctApi::SctApiDDCServer::instanceName),
    m_ddcSingleTimeout(3),
    m_ddcMultipleTimeout(15) {

    // See that we depend on the configuration service!
    boost::shared_ptr<SctConfiguration::Configuration> 
      config = 
      boost::shared_ptr<SctConfiguration::Configuration>
      (new SctConfiguration::ConfigIPC);

    m_worker = boost::shared_ptr<SctApi::SctApiDDC>( new SctApi::SctApiDDC(config) );
  };

  SctApi::SctApiDDC * SctApiDDCServer::worker() { 
    return m_worker.get();
  };

  void SctApiDDCServer::requestHardReset (CORBA::ULong mid)
  {
    worker()->requestHardReset(mid);
  }

  void SctApiDDCServer::requestIVCurve (CORBA::ULong mid, CORBA::Float start, CORBA::Float stop, CORBA::Float step, CORBA::UShort delay, CORBA::Float currentLim) 
  {
    worker()->requestIVCurve(mid, start, stop, step, delay, currentLim);
  }

  void SctApiDDCServer::setSelect (CORBA::ULong mid, CORBA::Boolean value)
  {
    worker()->setSelect(mid, value);
  }

  void SctApiDDCServer::setSelectAllInCrate (CORBA::Long crate, CORBA::Boolean value)
  {
    worker()->setSelectAllInCrate(crate, (bool)value, m_ddcMultipleTimeout);
  }

  void SctApiDDCServer::changeRunState (CORBA::Long state)
  {
    worker()->changeRunState(state);
  }

  void SctApiDDCServer::ddcSetSingleTimeout (CORBA::Short timeout)
  {
    m_ddcSingleTimeout = timeout;
  }

  CORBA::Short SctApiDDCServer::ddcGetSingleTimeout () 
  {
    return m_ddcSingleTimeout;
  }

  void SctApiDDCServer::ddcSetMultipleTimeout (CORBA::Short timeout)
  {
    m_ddcMultipleTimeout = timeout;
  }

  CORBA::Short SctApiDDCServer::ddcGetMultipleTimeout ()
  {
    return m_ddcMultipleTimeout;
  }

  void SctApiDDCServer::ddcChangeState (CORBA::Long crate, CORBA::Long channel, CORBA::Short state)
  {
    worker()->ddcChangeState(crate, channel, state, m_ddcMultipleTimeout);
  }

  void SctApiDDCServer::ddcChangeStates (CORBA::Long crate, const char* name, CORBA::Short state)
  {
    worker()->ddcChangeStates(crate, name, state, m_ddcMultipleTimeout);
  }

  void SctApiDDCServer::ddcChangeStateAll (CORBA::Long crate, CORBA::Short state)
  {
    worker()->ddcChangeStateAll(crate, state, m_ddcMultipleTimeout);
  }

  void SctApiDDCServer::ddcLoadConfiguration (CORBA::Long crate, CORBA::Short state)
  {
    worker()->ddcLoadConfiguration(crate, state, m_ddcMultipleTimeout);
  }

  void SctApiDDCServer::ddcHardReset (CORBA::Long crate, CORBA::Long channel)
  {
    worker()->ddcHardReset(crate, channel, m_ddcSingleTimeout);
  }

  void SctApiDDCServer::ddcHardResetAll (CORBA::Long crate)
  {
    worker()->ddcHardResetAll(crate, m_ddcMultipleTimeout);
  }

  CORBA::ULong SctApiDDCServer::ddcGetChannelParameterInt (CORBA::Long crate, CORBA::Long channel, const char * name)
  {
    return worker()->ddcGetChannelParameterInt(crate, channel, string(name), m_ddcSingleTimeout);
  }

  CORBA::Float SctApiDDCServer::ddcGetChannelParameterFloat (CORBA::Long crate, CORBA::Long channel, const char * name)
  {
    return worker()->ddcGetChannelParameterFloat(crate, channel, string(name), m_ddcSingleTimeout);
  }

  char * SctApiDDCServer::ddcGetChannelParameterString (CORBA::Long crate, CORBA::Long channel, const char * name)
  {
    return Sct::copyStringToCorba(worker()->ddcGetChannelParameterString(crate, channel, string(name), m_ddcSingleTimeout));
  }

  CORBA::ULong SctApiDDCServer::ddcGetCardParameterInt (CORBA::Long crate, CORBA::Long card, const char * name)
  {
    return worker()->ddcGetCardParameterInt(crate, card, string(name), m_ddcSingleTimeout);
  }

  CORBA::Float SctApiDDCServer::ddcGetCardParameterFloat (CORBA::Long crate, CORBA::Long card, const char * name)
  {
    return worker()->ddcGetCardParameterFloat(crate, card, string(name), m_ddcSingleTimeout);
  }

  char * SctApiDDCServer::ddcGetCardParameterString (CORBA::Long crate, CORBA::Long card, const char * name)
  {
    return Sct::copyStringToCorba(worker()->ddcGetCardParameterString(crate, card, string(name), m_ddcSingleTimeout));
  }

  CORBA::ULong SctApiDDCServer::ddcGetCrateParameterInt (CORBA::Long crate, const char * name)
  {
    return worker()->ddcGetCrateParameterInt(crate, string(name), m_ddcSingleTimeout);
  }

  CORBA::Float SctApiDDCServer::ddcGetCrateParameterFloat (CORBA::Long crate, const char * name)
  {
    return worker()->ddcGetCrateParameterFloat(crate, string(name), m_ddcSingleTimeout);
  }

  char * SctApiDDCServer::ddcGetCrateParameterString (CORBA::Long crate, const char * name)
  {
    return Sct::copyStringToCorba(worker()->ddcGetCrateParameterString(crate, string(name), m_ddcSingleTimeout));
  }

  void SctApiDDCServer::ddcGetCrateParameters (CORBA::Long crate, const Sct_SctApi::NameList & names, Sct_SctApi::NameValueList_out nameValues) 
  {
    const std::vector<std::string> & myNames = copyCorbaToVector<Sct_SctApi::NameList, std::string>(names);

    std::map<std::string, float> myNameValues;

    worker()->ddcGetCrateParameters(crate, myNames, myNameValues, m_ddcMultipleTimeout);

    nameValues = copyMapToNameValueList(myNameValues);
  }

  void SctApiDDCServer::ddcSetCrateParameterFloat(CORBA::Long crate, const char* name, CORBA::Float value)
  {
    worker()->ddcSetCrateParameter(crate, name, value, m_ddcSingleTimeout);
  }

  void SctApiDDCServer::ddcSetCardParameterFloat(CORBA::Long crate, CORBA::Long card, const char* name, CORBA::Float value)
  {
    worker()->ddcSetCardParameter(crate, card, name, value, m_ddcSingleTimeout);
  }

  void SctApiDDCServer::ddcSetChannelParameterFloat(CORBA::Long crate, CORBA::Long channel, const char* name, CORBA::Float value)
  {
    worker()->ddcSetChannelParameter(crate, channel, name, value, m_ddcSingleTimeout);
  }

 
};
