#include <ipc/core.h>
#include <ipc/object.h>
#include <ipc/server.h>
#include <pmg/pmg_initSync.h>

#include <boost/bind.hpp>
#include <boost/lexical_cast.hpp>
#include <boost/shared_ptr.hpp>
#include <boost/thread.hpp>

#include "SctApiDDCServer.h"
#include "sctConfIPC/configipc.h"
#include "Sct/ApplicationStartupDebugTools.h"
#include "Sct/MultiMessageDebugStream.h"
#include "Sct/DestroyingDeleter.h"

int main(int argc, char** argv) 
{
  IPCCore::init(argc, argv); // Try reentrant for DDC IS callbacks
  Sct::ApplicationStartupDebugTools::announceStartOfMain(argc, argv, __FILE__, __LINE__);

  IPCPartition p("SCT");
  try {    

    boost::shared_ptr<SctApiDDC::SctApiDDCServer> h = 
      ::Sct::DestroyingDeleter<SctApiDDC::SctApiDDCServer>::shared_ptr
      (new SctApiDDC::SctApiDDCServer);

    // Try publishing it
    if (!h->publish()) {
      Sct::MultiMessageDebugStream m(true,true,true);
      m << "Failed to publish SctApiDDCServer in ["<<__FILE__ << "]";
      m.flush();
      return 0;
    } 

    std::cout << "Serving...\n";
    pmg_initSync();
    h->run();
    std::cout << "Run over" << std::endl;
    h->withdraw();
  } catch(SctConfiguration::ConfigurationException &c) {
    std::cout << "ConfigurationException: " << c.what() << std::endl;

    {
      Sct::MultiMessageDebugStream m(true,true,true);
      m << "Failed to start SctApiDDCServerMain at [" << __FILE__ << "].  Sorry.";
      m.flush();
    };

    exit(1);
  }
}
