#include <ipc/core.h>
#include <ipc/object.h>
#include <ipc/server.h>
#include <pmg/pmg_initSync.h>

#include <boost/bind.hpp>
#include <boost/lexical_cast.hpp>
#include <boost/shared_ptr.hpp>
#include <boost/thread.hpp>

#include "SctApi/SctApi.h"
#include "sctConfIPC/configipc.h"

#include "SctApiServer.h"
#include "Sct/ApplicationStartupDebugTools.h"
#include "Sct/MultiMessageDebugStream.h"

// Not needed AT THE MOMENT but keep for future use: void dcsPollThread(SctApi::SctApi *sctApi);

int main(int argc, char** argv) 
{
  IPCCore::init(argc, argv); // Try reentrant for DDC IS callbacks
  Sct::ApplicationStartupDebugTools::announceStartOfMain(argc, argv, __FILE__, __LINE__);

  IPCPartition p("SCT");
  try {    

    SctApiServer::CrateDescriptors crateDescriptors;
    // Find the Crate Controllers! Eventually this set
    // of crateDescriptors should be obtained from command line 
    // args or from a database.  For the moment, hard code in a
    // default valie:
    const Sct::UCID ucidForSomeCrate(0,0);
    {
      Sct::MultiMessageDebugStream m(true,false,true);
      m << "Warning, complain to lester that there is a temporary hard coding of a UCID set to "<<ucidForSomeCrate<<" near line " << __LINE__ << " of " << __FILE__ << " that he must fix."; // see above hard coding!
      m.flush();
    };

    crateDescriptors.push_back( ucidForSomeCrate );

    std::auto_ptr<SctApiServer> h(new SctApiServer
                                  (crateDescriptors, 
                                   Sct_SctApi::SctApiIPC::instanceName
                                   ));
    
    // Try publishing it
    if (!h->publish()) {
      std::cout << "Failed to publish SctApi" << std::endl;
      return 0;
    } 

    std::cout << "Serving...\n";
    // The following line commented by Alan 8/11/2004 folloing
    // discussion with bruce about problems during scanning
    // if this thread runs
    // boost::thread dcsThread(boost::bind(&dcsPollThread, api.get()));
    std::cout << "sent bind to dctPollThread" << std::endl;
    pmg_initSync();
    h->run();
    std::cout << "Run over" << std::endl;
    h->withdraw();
  } catch(SctConfiguration::ConfigurationException &c) {
    std::cout << "ConfigurationException: " << c.what() << std::endl;

    MRSStream mrsStream(p);
    mrsStream << "SCTAPI_FAIL" << MRS_FATAL << MRS_QUALIF("SctApiServer") << MRS_TEXT("Configuration server not running!") << ENDM;
    
    exit(1);
  }
}

/* Whole method needs a rethink for multicrate, so commented out for the moment since it is not used anayway

void dcsPollThread(SctApi::SctApi *sctApi) {
  // Let everything settle down first
  sleep(3);

  IPCPartition partition("SCT");

  ISInfoDictionary isDict(partition);

  while(1) {
    std::cout << "Polling BOC DCS\n";

    std::list<SctApi::RodLabel> rodList = sctApi->listRods();

    std::string isBaseName("SCTAPIServer.BOC_DCS");

    for(std::list<SctApi::RodLabel> ::const_iterator rodI = rodList.begin();
	rodI != rodList.end();
	rodI ++) {
      std::vector<double> thisMonitor 
	= sctApi->getBOCMonitorArray(rodI->partition, rodI->crate, rodI->rod);

      std::string isRodName(isBaseName
			    + "." + boost::lexical_cast<std::string>(rodI->partition) 
			    + "." + boost::lexical_cast<std::string>(rodI->crate) 
			    + "." + boost::lexical_cast<std::string>(rodI->rod));

      for(int i=0; i<12; i++) {
	std::string isName(isRodName + ".ADC" + boost::lexical_cast<std::string>(i));

	ISInfoDouble isValue(thisMonitor[i]);

	if(isDict.contains(isName.c_str())) {
	  isDict.update(isName.c_str(), isValue);
	} else {
	  isDict.insert(isName.c_str(), isValue);
	}
      }
    }

    sleep(60);
  }
}

*/
