
#ifndef SCTAPIIPC_TRIGGERWRAPPER_H
#define SCTAPIIPC_TRIGGERWRAPPER_H

/** @class TriggerWrapper TriggerWrapper.h
 *
 * The class TriggerWrapper is designed to look externally like a
 * ::SctApi::Trigger (i.e. implement the ::SctApi::Trigger interface) 
 * while actually just passing all these calls through to a CORBA 
 * trigger object, which is itself internally implemented by a 
 * "standard" SctApi trigger object!
 *
 * This class DOES NOT OWN the object reference which it is 
 * constructed with!  An alternative strategy could be developed ...
 *
 * @author Christopher Lester
 **/

//Forward declaration
class TriggerWrapper;

// Need complete types for public inheritence
#include "SctApi/Trigger.h" 
#include "Sct_SctApi/SctApi.hh" // for Trigger{_var,_ptr} interface 

class TriggerWrapper : public ::SctApi::Trigger {
 public:
  typedef enum MemoryPolicy_enum { 
    takeOwnershipOfObjectReference, 
    doNotTakeOwnershipOfObjectReference
  } MemoryPolicy;
 private:
  const MemoryPolicy m_memoryPolicy;
  typedef Sct_SctApi::Trigger_ptr WorkerType;
  WorkerType m_worker;
  const WorkerType worker() const { return m_worker; };
  WorkerType worker() { return m_worker; };
 public:
  typedef ::SctApi::Trigger::RODTriggers RODTriggers;
 public:
  TriggerWrapper(const WorkerType & w, const MemoryPolicy memoryPolicy) :
    m_memoryPolicy(memoryPolicy),
    m_worker(w) {
  };
  virtual ~TriggerWrapper() {
    if (m_memoryPolicy==takeOwnershipOfObjectReference) {
      CORBA::release(m_worker);
    };
  };
  void singleL1A() { 
    return worker()->singleL1A(); };
  void doubleL1A(unsigned short delay) { 
    return worker()->doubleL1A(delay); };
  void delayedL1A(unsigned short delay) { 
    return worker()->delayedL1A(delay); };
  void calL1A(unsigned short delay) { 
    return worker()->calL1A(delay); };
  void pulseL1A(unsigned short delay) { 
    return worker()->pulseL1A(delay); };
  void softL1A(unsigned short delay) { 
    return worker()->softL1A(delay); };
  void softCalL1A(unsigned short delay, unsigned short delay2) { 
    return worker()->softCalL1A(delay, delay2); };
  void softPulseL1A(unsigned short delay, unsigned short delay2) { 
    return worker()->softPulseL1A(delay, delay2); };
  void softBc(unsigned short delay) { 
    return worker()->softBc(delay); };
  void soft() { 
    return worker()->soft(); };
  void bc() { 
    return worker()->bc(); };
  void bcL1A(unsigned short delay) { 
    return worker()->bcL1A(delay); };
  void bcCalL1A(unsigned short delay, unsigned short delay2) { 
    return worker()->bcCalL1A(delay, delay2); };
  void bcPulseL1A(unsigned short delay, unsigned short delay2) { 
    return worker()->bcPulseL1A(delay, delay2); };
  void setCommIncr(unsigned short command, unsigned short incr) { 
    return worker()->setCommIncr(command, incr); };
  void getCommIncr(unsigned short &command, unsigned short &incr) const { 
    return worker()->getCommIncr(command, incr); };
  const RODTriggers getRODTriggers() const; 
  void print() const { 
    return worker()->print(); };  
};

#endif
