#ifndef SCTAPI_MARSHALLING_H
#define SCTAPI_MARSHALLING_H

#include <vector>
#include "Sct/SequenceUtils.h"
#include "SctApi/autoConfig.h"

inline Sct_SctApi::BOCChannelList * copyBOCChannelConfigVectorToCorba(const std::vector<SctConfiguration::BOCChannelConfig> &configList) {

  Sct_SctApi::BOCChannelList * result = ::Sct::SequenceUtils::newSafeSequence<Sct_SctApi::BOCChannelList>(configList.size());

  for(unsigned long i=0; i<configList.size(); i++) {
    const SctConfiguration::BOCChannelConfig &config = configList[i];

    sctConf::BOCChannelConfig res;

    res.current = config.current;
    res.delay = config.delay;
    res.markSpace = config.markSpace;
    res.threshold0 = config.threshold0;
    res.delay0 = config.delay0;
    res.threshold1 = config.threshold1;
    res.delay1 = config.delay1;

    (*result)[i]=res;
  }

  return result;
}


inline Sct_SctApi::AutoResultList * copyAutoResultListToCorba(const std::list< ::SctApi::AutoConf::AutoResult> &autoList) {
  Sct_SctApi::AutoResultList * result = ::Sct::SequenceUtils::newSafeSequence<Sct_SctApi::AutoResultList>(autoList.size());
  //result = _Sct_SctApi_T_AutoResultList_sequence::Create(4, NULL);
  //result->Clear(0);

  unsigned int i=0;

  for(std::list< ::SctApi::AutoConf::AutoResult>::const_iterator autoIter = autoList.begin();
      autoIter != autoList.end(); (++autoIter, ++i)) {
    const ::SctApi::AutoConf::AutoResult &autoResult = *autoIter;

    Sct_SctApi::AutoConfigResult res;

    res.partition = autoResult.partition;
    res.crate = autoResult.crate;
    res.rod = autoResult.rod;

    res.tx = autoResult.tx;
    res.rx = autoResult.rx;

    (*result)[i] = res;
  }

  return result;
}


template <typename ILUBlock, typename ArrayMember>
inline std::vector<ArrayMember> copyCorbaToVector(const ILUBlock & thisSeq) {
  std::vector<ArrayMember> array;

  for(unsigned int i=0; i<thisSeq.length(); ++i) {
    const ArrayMember am = thisSeq[i];
    array.push_back(am);
  }

  return array;
}

// explicit specialisation
template<>
inline std::vector<std::string> copyCorbaToVector<Sct_SctApi::NameList, std::string>(const Sct_SctApi::NameList & thisSeq) {
  std::vector<std::string> array;

  for(unsigned int i=0; i<thisSeq.length(); ++i) {
    const std::string am = thisSeq[i]._NP_ref();
    array.push_back(am);
  }

  return array;
}


inline Sct_SctApi::NameValueList * copyMapToNameValueList(std::map<std::string, float> myNameValues) {
  Sct_SctApi::NameValueList * result = ::Sct::SequenceUtils::newSafeSequence<Sct_SctApi::NameValueList>(myNameValues.size());

  //result = _Sct_SctApi_T_NameValueList_sequence::Create(4, NULL);
  //result->Clear(0);

  unsigned int i=0;
  for(std::map<std::string, float>::const_iterator iter=myNameValues.begin(); 
      iter != myNameValues.end(); 
      (++iter,++i)) {

    Sct_SctApi::NameValue member;

    member.name = CORBA::string_dup(iter->first.c_str());
    member.value = iter->second;

    (*result)[i] = member;
  }

  return result;
}

#endif
