#ifndef SCT_CONFIGSERVER_H
#define SCT_CONFIGSERVER_H
#include <ipc/object.h>
#include <ipc/server.h>

#include "sctConfIPC/configuration.hh"

#include "sctConf/configXMLImpl.h"

#include <is/isinfo.h>
#include <rc/RCStateInfo.h>
#include <memory>

class ConfigServer : public IPCNamedObject<POA_sctConf::Configuration, ipc::single_thread> {
  ConfigurationXMLImpl config;
  std::auto_ptr<ISInfoReceiver> infoReceiver;     ///<IS InfoReceiver for RC events
  
  /// Wrap configuration exception as corba exception.
  static sctConf::ConfigurationException wrapException(SctConfiguration::ConfigurationException& e);
 public:

  ConfigServer(const IPCPartition& p, const std::string& name);
  virtual ~ConfigServer();		// destructor
  
 // methods

  virtual sctConf::PartitionList* listPartitions ();
  virtual sctConf::CrateList* listCratesInPartition (CORBA::ULong partition);
  virtual CORBA::Boolean isDummyCrate (CORBA::ULong partition, CORBA::ULong crate);
  virtual sctConf::RodList* listRodsInCrate (CORBA::ULong partition, CORBA::ULong crate);
  virtual sctConf::MURList* listMURSInRod (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod);
  virtual sctConf::ModuleList* listModulesInMUR (CORBA::ULong partition, CORBA::ULong MUR);
  virtual sctConf::ModuleList* listRModulesInMUR (CORBA::ULong partition, CORBA::ULong MUR);
  virtual sctConf::ModuleList* listAllModules ();
  virtual sctConf::ModuleList* listUnusedModules ();
  virtual sctConf::ABCDModule getModuleConfig (const char* mod);
  virtual CORBA::UShort getModuleGroup (const char* mod);
  virtual void setModuleGroup (const char* mod, CORBA::Short group);
  virtual CORBA::ULong getModuleErrorMask (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod, CORBA::ULong channel, CORBA::UShort link);
  virtual void setModuleErrorMask (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod, CORBA::ULong channel, CORBA::UShort link, CORBA::ULong mask);
  virtual void setDefaultModuleErrorMask (CORBA::ULong mask);
  virtual sctConf::RodConfig * getRodConfig (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod);
  virtual sctConf::SlaveConfig* getDefaultSlaveConfig (CORBA::Octet type);
  virtual sctConf::TimConfig getTimConfig (CORBA::ULong partition, CORBA::ULong crate);
  virtual sctConf::CharBlock* getFibreMappings (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod);
  virtual void setFibreMapping(CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod, CORBA::ULong channel, CORBA::ULong tx, CORBA::ULong rx0, CORBA::ULong rx1);
  virtual sctConf::BOCChannelConfig getBOCConfig (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod, CORBA::ULong channel);
  virtual sctConf::BOCGlobalConfig getBOCGlobalConfig (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod);
  virtual void printModuleConfig(const sctConf::ABCDModule& conf);
  virtual sctConf::MURType getMURType (CORBA::ULong MUR);
  virtual CORBA::ULong getFreeMurId ();
  virtual void translateToROD(CORBA::ULong MUR, CORBA::ULong mod, CORBA::ULong& partition, CORBA::ULong& crate, CORBA::ULong& rod, CORBA::ULong& channel);
  virtual void translateToRROD(CORBA::ULong MUR, CORBA::ULong mod, CORBA::ULong& partition, CORBA::ULong& crate, CORBA::ULong& rod, CORBA::ULong& channel);
  virtual void translateToSN(CORBA::ULong MUR, CORBA::ULong mod, CORBA::String_out sn);
  virtual void translateToRMUR(CORBA::ULong RMUR, CORBA::ULong rmod, CORBA::ULong& MUR, CORBA::ULong& mod);
  virtual void translateToBarrel(CORBA::ULong MUR, CORBA::ULong mod, CORBA::ULong& barrel, CORBA::ULong& row, CORBA::Long& number);
  virtual void translateToEndcap(CORBA::ULong MUR, CORBA::ULong mod, CORBA::Long& disk, CORBA::ULong& ring, CORBA::ULong& number);
  virtual void translateToPowerSupply(CORBA::ULong MUR, CORBA::ULong mod, CORBA::ULong& partition, CORBA::ULong& crate, CORBA::ULong& channel);
  virtual void translateFromROD(CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod, CORBA::ULong channel, CORBA::ULong& MUR, CORBA::ULong& mod);
  virtual void translateFromRROD(CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod, CORBA::ULong channel, CORBA::ULong& MUR, CORBA::ULong& mod);
  virtual void translateFromSN (const char* sn, CORBA::ULong& MUR, CORBA::ULong& mod);
  virtual void translateFromRMUR(CORBA::ULong MUR, CORBA::ULong mod, CORBA::ULong& RMUR, CORBA::ULong& rmod);
  virtual void translateFromBarrel(CORBA::ULong barrel, CORBA::ULong row, CORBA::Long number, CORBA::ULong& MUR, CORBA::ULong& mod);
  virtual void translateFromEndcap(CORBA::Long disk, CORBA::ULong ring, CORBA::ULong number, CORBA::ULong& MUR, CORBA::ULong& mod);
  virtual void translateFromPowerSupply(CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong channel, CORBA::ULong& MUR, CORBA::ULong& mod);
  virtual void configureModuleFromStructure (const char* number, const sctConf::ABCDModule& mConf);
  virtual void configureModuleFromFile (const char* filename);
  virtual void configureROD (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod, const sctConf::RodConfig& conf);
  virtual void configureTIM (CORBA::ULong partition, CORBA::ULong crate, const sctConf::TimConfig& conf);
  virtual void configureBOC (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod, const sctConf::BOCGlobalConfig& conf);
  virtual void clearAll ();
  virtual void namePartition (CORBA::ULong partition, const char* name);
  virtual void mapRODMUR (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod, CORBA::ULong order, CORBA::ULong number);
  virtual void unmapRODMUR (CORBA::ULong MUR);
  virtual void mapBarrelMUR (CORBA::ULong MUR, CORBA::ULong barrel, CORBA::ULong row, CORBA::Long position);
  virtual void unmapBarrelMUR (CORBA::ULong MUR);
  virtual void mapEndcapMUR (CORBA::ULong MUR, CORBA::Long disk, CORBA::ULong quadrant, CORBA::ULong position);
  virtual void unmapEndcapMUR (CORBA::ULong MUR);
  virtual void mapPowerChannel (CORBA::ULong MUR, CORBA::ULong number, CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong channel);
  virtual void unmapPowerChannel (CORBA::ULong MUR, CORBA::ULong number);
  virtual void mapModuleMUR (CORBA::ULong MUR, CORBA::ULong order, CORBA::ULong RMUR, CORBA::ULong rorder, const char* sn);
  virtual void unmapModuleMUR (CORBA::ULong MUR, CORBA::ULong order);
  virtual void getMapMURROD (CORBA::ULong MUR, CORBA::ULong& partition, CORBA::ULong& crate, CORBA::ULong& rod, CORBA::ULong& order);
  virtual void getMapRODMUR (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod, CORBA::ULong order, CORBA::ULong& MUR);
  virtual sctConf::MURType getMapMURPhysical (CORBA::ULong MUR, CORBA::ULong& plane, CORBA::ULong& section, CORBA::ULong& position);
  virtual void swapMURNames (CORBA::ULong MUR1, CORBA::ULong MUR2);
  virtual void modifyDefaultPowerParam(const char* state, const char* name, const char* type, CORBA::Float value);
  virtual void modifyHVCardParam(CORBA::ULong crate, CORBA::ULong card, const char* name, const char* type, CORBA::Float value);
  virtual void modifyLVCardParam(CORBA::ULong crate, CORBA::ULong card, const char* name, const char* type, CORBA::Float value);
  virtual void modifyPowerParam(CORBA::ULong MUR, CORBA::ULong number, const char* state, const char* name, const char* type, CORBA::Float value);
  virtual void modifyPowerParamCC(CORBA::ULong crate, CORBA::ULong channel, const char* state, const char* name, const char* type, CORBA::Float value);
  virtual CORBA::Float getDefaultPowerParam(const char* state, const char* name, const char* type);
  virtual CORBA::Float getHVCardParam(CORBA::ULong crate, CORBA::ULong card, const char* name, const char* type);
  virtual CORBA::Float getLVCardParam(CORBA::ULong crate, CORBA::ULong card, const char* name, const char* type);
  virtual CORBA::Float getPowerParam(CORBA::ULong MUR, CORBA::ULong number, const char* state, const char* name, const char* type);
  virtual CORBA::Float getPowerParamCC(CORBA::ULong crate, CORBA::ULong channel, const char* state, const char* name, const char* type);
  virtual sctConf::NameList* listDefaultPowerParams ();
  virtual sctConf::NameList* listDCSChannelParams ();
  virtual sctConf::NameList* listDCSCardParams ();
  virtual sctConf::NameList* listDCSCrateParams ();
  virtual void configureBOCChannel (CORBA::ULong MUR, CORBA::ULong position, const sctConf::BOCChannelConfig& conf);
  virtual void loadConfiguration (const char* filename);
  virtual void saveConfiguration (const char* filename);
  virtual void saveModuleConfiguration (const char* sn, const char* filename);
  virtual char* getModuleConfigurationString (const char* modsn, const sctConf::ABCDModule& config);
  virtual void writePowerSupplyConfiguration (const char* directory);
  virtual void shutdown();
};

#endif
