#include "configserver.h"
#include "sctConfIPC/configipc.h"
#include "RunController.h"
#include <pmg/pmg_initSync.h>
#include "ipc/core.h"
#include "mrs/message.h"
#include "rc/Controller.h"
#include "Sct/SctNames.h"

using namespace std;

using namespace SctConfiguration;

int main(int argc, char** argv) 
{
  IPCCore::init(argc,argv);

  IPCPartition p("SCT");
  MRSStream mrsStream("SCT");
  
  ConfigServer *config_server;
  try {
    config_server = new ConfigServer(p, ::sctConf::Configuration::instanceName);
  } catch(ConfigurationException &c) { // non-corba type can be thrown at start-up
    mrsStream << "CONF_FAILED " << MRS_FATAL << MRS_QUALIF("sctConf") 
              << MRS_TEXT(string("Failed to create Configuration Service") + c.what()) << ENDM;
    return 1;
  }
  
  mrsStream << "CONF_START " << MRS_DIAGNOSTIC << MRS_QUALIF("sctConf") 
	    << MRS_TEXT("Config Server created") << ENDM;
  
  if (!config_server->publish()) {
    mrsStream << "CONF_FAILED " << MRS_FATAL << MRS_QUALIF("sctConf") 
              << MRS_TEXT("Failed to publish Configuration Service") << ENDM;

    cout << "Failed to publish Configuration" << endl;
    return 2;
  } 
  
  ConfigIPC wrapper(config_server->_this());

  SctConfigurationController::RunControl& user = 
    SctConfigurationController::RunControl::initialize(wrapper);
  
  RC::Controller run_controller(Sct::SctNames::getPartition(),user, "config_server");
  
  cout << "Configuration server published, awaiting commands\n";

  // now that we have fulfilled our contract to the world by having made a configuration server public, we inform the world that we are ready for others that depend on us to start:
  pmg_initSync();

  try {
    run_controller.run();
  } catch(ConfigurationException &c) {  // catch non-corba exceptions
    mrsStream << "CONF_FAILED " << MRS_FATAL << MRS_QUALIF("sctConf") 
              << MRS_TEXT((string("Failed to run Configuration Service") + c.what()).c_str()) << ENDM;
    cout << "Configuration run threw exception " << c.what() << endl;
    return 3;
  }

  cout << "Run over" << endl;

  config_server->withdraw();
  config_server->_destroy();
}
