#include <iostream> 
#include <list>
#include <memory>

#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>

using namespace std;

#include "configXMLImpl.h"
using namespace SctConfiguration;

int main(int argc, char **argv) {
  /* If you feel tempted to put in the following line:
          IPCCore::init(argc, argv);
      please don't 

      This is a standalone program which shouldn't be allowed access 
       to the rest of the system (eg IS services and MRS)

      IPC will complain if MRS messages are sent without it running
       this should be corrected by checking at that point.
  */
  std::cerr << "(Ignore warnings such as:\n";
  std::cerr << "        Your application must call the IPCCore::init method\n";
  std::cerr << "   which IPC will send)\n";

  std::cout << "========================================================================"<<endl;  
  std::auto_ptr<SctConfiguration::Configuration> conf(new ConfigurationXMLImpl);

  if(argc > 1) {
    conf->loadConfiguration(argv[1]);
  }

  SctConfiguration::Configuration &config=*conf;

  unsigned int totalPartitions = 0;
  unsigned int totalCrates = 0;
  unsigned int totalRods = 0;
  unsigned int totalModules = 0;

  try {
    list<unsigned int> partitions = config.listPartitions();

    for(list<unsigned int>::const_iterator p=partitions.begin();
        p!=partitions.end(); 
        p++) {
      list<unsigned int> crates = config.listCratesInPartition(*p);

      for(list<unsigned int>::const_iterator c=crates.begin(); 
          c!=crates.end(); 
          c++) {
        try {
          TimConfig tConf = config.getTimConfig(*p, *c);
        } catch(ConfigurationException &cx) {
          cout << "No TIM configuration for crate " << *p << " " << *c << endl;
        }

        list<unsigned int> rods = config.listRodsInCrate(*p, *c);

        for(list<unsigned int>::const_iterator r=rods.begin(); 
            r!=rods.end(); 
            r++) {
          RodConfig rodConf = config.getRodConfig(*p, *c, *r);

          {
            struct stat statBuf;
            int ret = stat(rodConf.slaves[0].idramFile.c_str(), &statBuf);

#warning "Probably need env subs"
//             if(ret)
//               perror("Error stating idramFile");
          }

          totalRods ++;
          list<unsigned int> MURs = config.listMURSInRod(*p, *c, *r);

          for(list<unsigned int>::const_iterator mur=MURs.begin(); 
              mur!=MURs.end(); 
              mur++) {
            list<string> modules = config.listModulesInMUR(*p, *mur);

	    std::string output_name("CheckedModules.txt");
            ofstream output(output_name.c_str()); 
	    
            for(list<string>::const_iterator m=modules.begin(); 
                m!=modules.end(); 
                m++) {
              try {
                // ROD
                unsigned int partition, crate, rod, channel;

                // MUR
                unsigned int MUR, module;

                // Serial number
                std::string sn;

                // Barrel format
                unsigned int barrel, row;
                int number;

                int group = config.getModuleGroup(*m);
//                 cout << "Got module group = " << group << endl;

                // Endcap format
                int disk;
                unsigned int ring, eNumber;

                // Power supply
                unsigned int ppartition, pcrate, pchannel;

                config.translateFromSN(*m, MUR, module);
                config.translateToROD(MUR, module, partition, crate, rod, channel);
                config.translateToSN(MUR, module, sn);

                if(sn != *m) {
                  std::cout << " Serial number conversion mismatch!\n";
                }

                MUR = 100000;
                module = 100;

                config.translateFromROD(partition, crate, rod, channel, MUR, module);

                try {
                  BOCChannelConfig bocConf = config.getBOCConfig(*p, *c, *r, channel);
                } catch(ConfigurationException &cex) {
                  cout << "No BOCConfig for " << *p << " " << *c << " " << *r << " " << channel << " (" << *m << ")!\n";
                  cout << "\t" << cex.what() << endl;
                }

                try {
                  MURType type = config.getMURType(MUR);

                  switch(type) {
                  case BARREL:
                    barrel = 100, row = 100;
                    number = 100;
                    config.translateToBarrel(MUR, module, barrel, row, number);
                    break;
                  case ENDCAP:
                    disk = 100, ring = 100;
                    eNumber = 100;
                    config.translateToEndcap(MUR, module, disk, ring, eNumber);
                    break;
                  case UNMAPPED:
                    cout << "No physical translation for MUR " << MUR << " " << module << " (" << *m << ")\n";
                    break;
                  case UNKNOWN:
                    cout << "MUR " << MUR << " of unkown type!\n";
                    break;
                  }
                } catch(ConfigurationException &c) {
                  cout << "No physical translation for MUR " << MUR << " " << module << " (" << *m << ")\n";
                }

                try {
                  unsigned int rpartition = 100;
                  unsigned int rcrate = 100;
                  unsigned int rrod = 100;
                  unsigned int rchannel = 100;
                  config.translateToRROD(MUR, module, rpartition, rcrate, rrod, rchannel);

		  output << "ROD:  " << partition << ", " << crate << ", " << rod << ", " << channel << endl;
		  output << "RROD: " << rpartition << ", " << rcrate << ", " << rrod << ", " << rchannel << endl;

//                   cout << "From Redundant Rod: ";
                  config.translateFromRROD(rpartition, rcrate, rrod, rchannel, MUR, module);
//                   cout << "MUR " << MUR << " " << "module " << module << endl;
                } catch(ConfigurationException &c) {
                  cout << "--> No redundant Rod mapping for " 
                       << MUR << " " << module << " (" << *m << ")\n";
                }

                try {
                  config.translateToPowerSupply(MUR, module, ppartition, pcrate, pchannel);
                } catch(ConfigurationException &c) {
                  cout << "No power supply mapping... for (" << MUR << " " << module << ") (" << *m << ")\n";
		  //  Add verbose option?
                  cout << "   due to: " << c.what() << endl;
                }
              } catch(ConfigurationException &c) {
                cout << "Unexpected exception doing tests on module translation\n -> " << c.what() << "\n";
              } 

              try {
//                 cout << "Module configuration for " << *m << ":\n";
                ABCDModule configuration = config.getModuleConfig(*m);
//                 cout << " Found module configuration\n";
//                 config.printModuleConfig(configuration);

//                 cout << " Get module string\n";
//                 cout << config.getModuleConfigurationString(*m, configuration).length() << endl;
//                 cout << config.getModuleConfigurationString(*m, configuration) << endl;
              } catch(ConfigurationException &c) {
                cout << "No module configuration found for " << *m << ":\n";
                cout << "\t" << c.what() << endl;
              }

              totalModules ++;
            }

            try {
              list<string> rmodules = config.listRModulesInMUR(*p, *mur);
            } catch(ConfigurationException &e) {
              cout << "Bad redundant modules for rod MUR " << *mur << endl;
            }
          }

          unsigned char *mappings = config.getFibreMappings(*p, *c, *r);

          delete [] mappings;
        }
        totalCrates++;
      }
      totalPartitions ++;
    }

    list<string> modules = config.listAllModules();

    list<string> uModules = config.listUnusedModules();

    if(uModules.size() > 0) {
      cout << "Configuration provided for " << uModules.size() << " modules that aren't linked into ROD hierarchy\n";
      std::string name="NotInHeirarchy.txt";
      std::cout << "  (see `" << name << "' for list)" << std::endl;
      std::ofstream extraModules(name.c_str());
      for(list<string>::const_iterator module=uModules.begin(); 
          module!=uModules.end(); 
          module++) {
        extraModules << "  " << *module << endl;
      }
    }
  } catch(ConfigurationException &c) {
    cout << "Bad configuration somewhere " << c.what() << "\n"; 
  }

  cout << "*** Configuration object counts ***\n";
  cout << "\ttotalPartitions: " << totalPartitions << endl;
  cout << "\ttotalCrates: " << totalCrates << endl;
  cout << "\ttotalRods: " << totalRods << endl;
  cout << "\ttotalModules: " << totalModules << endl;
}
