#include <memory>

#include "configipc.h"
#include "marshalling.h"
#include "Sct/OmniMarshalling.h"

using namespace std;
using namespace Sct;
using namespace SctConfiguration;

void myThrow();

ConfigIPC::ConfigIPC()
{
  IPCPartition p("SCT");
  handle = p.lookup<sctConf::Configuration>(sctConf::Configuration::instanceName);
  if (CORBA::is_nil(handle)) {
    std::cout << "Error" << std::endl;
    throw ConfigurationException("Can't initialise! (Can't find IPC config)");
  }
}

ConfigIPC::ConfigIPC(sctConf::Configuration_ptr handle) : handle(handle) {}

ConfigIPC::~ConfigIPC()
{
}

ConfigurationException ConfigIPC::convertException(sctConf::ConfigurationException& e){
  return ConfigurationException(string(e.detail));
}

std::list<unsigned int> ConfigIPC::listPartitions()
{
  try{
    sctConf::PartitionList* myList = handle->listPartitions();
    return copyCorbaToList<sctConf::PartitionList, unsigned int>(myList);
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  }
}

std::list<unsigned int> ConfigIPC::listCratesInPartition(unsigned int partition)
{
  try{
    sctConf::CrateList* myList = handle->listCratesInPartition(partition);
    return copyCorbaToList<sctConf::CrateList, unsigned int>(myList);
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  }
}

std::list<unsigned int> ConfigIPC::listRodsInCrate(unsigned int partition, unsigned int crate)
{
  try{
  sctConf::RodList* myList = handle->listRodsInCrate(partition, crate);
  return copyCorbaToList<sctConf::RodList, unsigned int>(myList);
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  }
}

std::list<unsigned int> ConfigIPC::listMURSInRod(unsigned int partition, unsigned int crate, unsigned int rod)
{
  try{
  sctConf::MURList* myList = handle->listMURSInRod(partition, crate, rod);
  return copyCorbaToList<sctConf::MURList, unsigned int>(myList);
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  }
}

std::list<std::string> ConfigIPC::listModulesInMUR(unsigned int partition, unsigned int MUR)
{
  try{
  sctConf::ModuleList* myList = handle->listModulesInMUR(partition, MUR);
  return copyCorbaToList<sctConf::ModuleList, string>(myList);
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  }
}

std::list<std::string> ConfigIPC::listRModulesInMUR(unsigned int partition, unsigned int MUR)
{
  try{
  sctConf::ModuleList* myList = handle->listRModulesInMUR(partition, MUR);
  return copyCorbaToList<sctConf::ModuleList, string>(myList);
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  }
}

std::list<std::string> ConfigIPC::listAllModules()
{
  try{
  sctConf::ModuleList* myList = handle->listAllModules();
  return copyCorbaToList<sctConf::ModuleList, string>(myList);
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  }
}

std::list<std::string> ConfigIPC::listUnusedModules()
{
  try{
  sctConf::ModuleList* myList = handle->listUnusedModules();
  return copyCorbaToList<sctConf::ModuleList, string>(myList);
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  }
}

bool ConfigIPC::isDummyCrate(unsigned int partition, unsigned int crate)
{
  try{
  bool ret = handle->isDummyCrate(partition, crate);
  return ret;
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  }
}


ABCDModule ConfigIPC::getModuleConfig(const std::string module)
{
  try{
  sctConf::ABCDModule iluModule = handle->getModuleConfig(module.c_str());
  return copyCorbaToABCDModule(iluModule);
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  }
}

short ConfigIPC::getModuleGroup(const std::string module)
{
  try{
  return handle->getModuleGroup(module.c_str()); 
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  }
}

void ConfigIPC::setModuleGroup(const std::string module, short group) 
{
  try{
  handle->setModuleGroup(module.c_str(), group); 
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  }
}

unsigned int ConfigIPC::getModuleErrorMask(unsigned int partition, unsigned int crate, unsigned int rod, unsigned int channel, unsigned int link) 
{
  try{
    unsigned int mask = handle->getModuleErrorMask(partition, crate, rod, channel, link);
    return mask;
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  }
}

void ConfigIPC::setModuleErrorMask(unsigned int partition, unsigned int crate, unsigned int rod, unsigned int channel, unsigned int link, unsigned int mask)
{
  try{
    handle->setModuleErrorMask(partition, crate, rod, channel, link, mask);
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  }
}

void ConfigIPC::setDefaultModuleErrorMask(unsigned int mask)
{
  try{
    handle->setDefaultModuleErrorMask(mask);
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  }
}

RodConfig ConfigIPC::getRodConfig(unsigned int partition, unsigned int crate, unsigned int rod)
{
  try{
  auto_ptr<sctConf::RodConfig> config (handle->getRodConfig(partition, crate, rod));
  return copyCorbaToRodConfig(*config);
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  }
}

SlaveConfig ConfigIPC::getDefaultSlaveConfig(char type)
{
  try{
  auto_ptr<sctConf::SlaveConfig> config (handle->getDefaultSlaveConfig(type));
  return copyCorbaToSlaveConfig(*config);
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  }
}

BOCChannelConfig ConfigIPC::getBOCConfig(unsigned int partition, unsigned int crate, unsigned int rod, 
                                         unsigned int channel)
{  
  try{
  sctConf::BOCChannelConfig config = handle->getBOCConfig(partition, crate, rod, channel);
  return copyCorbaToBOCChannelConfig(config);
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  }
}

BOCGlobalConfig ConfigIPC::getBOCGlobalConfig(unsigned int partition, unsigned int crate, unsigned int rod)
{
  try{
  sctConf::BOCGlobalConfig config = handle->getBOCGlobalConfig(partition, crate, rod);
  return copyCorbaToBOCGlobalConfig(config);
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  }
}

unsigned char *ConfigIPC::getFibreMappings(unsigned int partition, unsigned int crate, unsigned int rod)
{
  try{
  unsigned char *array = new unsigned char[48*3];
  auto_ptr<sctConf::CharBlock> block(handle->getFibreMappings(partition, crate, rod));
  copyCorbaToArray<sctConf::CharBlock, unsigned char>(*block, array);
  return array;
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  }
}

TimConfig ConfigIPC::getTimConfig(unsigned int partition, unsigned int crate)
{
  try{
  sctConf::TimConfig config (handle->getTimConfig(partition, crate));
  return copyCorbaToTimConfig(config);
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  }
}

void ConfigIPC::setFibreMapping(unsigned int partition, unsigned int crate, unsigned int rod, 
                                unsigned int channel, unsigned int tx, unsigned int rx0, unsigned int rx1)
{
  try{
  handle->setFibreMapping(partition, crate, rod, channel, tx, rx0, rx1);
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  }
}

void ConfigIPC::printModuleConfig(const ABCDModule &conf)
{
  try{
  sctConf::ABCDModule config (copyABCDModuleToCorba(conf));
  handle->printModuleConfig(config); 
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  } 
}

MURType ConfigIPC::getMURType(unsigned int MUR)
{
  try{
  MURType result;
  result = (MURType)handle->getMURType(MUR);
  return result;
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  }
}

unsigned int ConfigIPC::getFreeMurId() 
{
  try{
   unsigned int mur = handle->getFreeMurId();
   return mur;
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  }
}
void ConfigIPC::translateToROD(unsigned int MUR, unsigned int module,
                               unsigned int &partition, unsigned int &crate, 
                               unsigned int &rod, unsigned int &channel)
{
  try{
  unsigned long p, c, r, ch;
  handle->translateToROD(MUR, module, p, c, r, ch);
  partition = p; crate = c; rod = r; channel = ch;
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  }
  
}

void ConfigIPC::translateToRROD(unsigned int MUR, unsigned int module,
                                unsigned int &partition, unsigned int &crate, 
                                unsigned int &rod, unsigned int &channel)
{
  try{
  unsigned long p, c, r, ch;
  handle->translateToRROD(MUR, module, p, c, r, ch);
  partition = p; crate = c; rod = r; channel = ch; 
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  } 
}

void ConfigIPC::translateToSN(unsigned int MUR, unsigned int module,
                              std::string &sn)
{
  try{
  char *inputString = 0;
  handle->translateToSN(MUR, module, inputString);
  sn = inputString;
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  }
}

void ConfigIPC::translateToRMUR(unsigned int MUR, unsigned int module,
                                unsigned int &RMUR, unsigned int &rmodule)
{
  try{
  unsigned long rm, rmod;
  handle->translateToRMUR(MUR, module, rm, rmod);
  RMUR = rm; rmod = rmod; 
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  }
}

void ConfigIPC::translateToBarrel(unsigned int MUR, unsigned int module,
                                  unsigned int &barrel, unsigned int &row, int &number)
{
  try{
  unsigned long b, r;
  long n;
  handle->translateToBarrel(MUR, module, b, r, n);
  barrel = b; row = r; number = n; 
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  }
}

void ConfigIPC::translateToEndcap(unsigned int MUR, unsigned int module,
                                  int &disk, unsigned int &ring, unsigned int &number)
{
  try{
  long d;
  unsigned long r, n;
  handle->translateToEndcap(MUR, module, d, r, n);
  disk = d; ring = r; number = n; 
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  }
}

void ConfigIPC::translateToPowerSupply(unsigned int MUR, unsigned int module,
                                       unsigned int &partition, 
                                       unsigned int &crate, unsigned int &channel)
{
  try{
  unsigned long p, c, ch;
  handle->translateToPowerSupply(MUR, module, p, c, ch);
  partition = p; crate = c; channel = ch; 
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  }
}

void ConfigIPC::translateFromROD(unsigned int partition, unsigned int crate, 
                                 unsigned int rod, unsigned int channel,
                                 unsigned int &MUR, unsigned int &module)
{
  try{
  unsigned long m, mo;
  handle->translateFromROD(partition, crate, rod, channel, m, mo);
  MUR = m; module = mo; 
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  }
}

void ConfigIPC::translateFromRROD(unsigned int partition, unsigned int crate,
                                  unsigned int rod, unsigned int channel,
                                  unsigned int &MUR, unsigned int &module)
{
  try{
  unsigned long m, mo;
  handle->translateFromRROD(partition, crate, rod, channel, m, mo);
  MUR = m; module = mo;
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  }
}

void ConfigIPC::translateFromSN(const std::string sn,
                                unsigned int &MUR, unsigned int &module)
{
  try{
  unsigned long m, mo;
  handle->translateFromSN(sn.c_str(), m, mo);
  MUR = m; module = mo; 
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  }
}

void ConfigIPC::translateFromRMUR (unsigned int RMUR, unsigned int rmodule, unsigned int &MUR, unsigned int &module)
{
  try{
  unsigned long mur, mod;
  handle->translateFromRMUR(RMUR, rmodule, mur, mod);
  MUR = mur; module = mod;
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  }
  
}

void ConfigIPC::translateFromBarrel(unsigned int barrel, unsigned int row, int number,
                                    unsigned int &MUR, unsigned int &module)
{
  try{
  unsigned long m, mo;
  handle->translateFromBarrel(barrel, row, number, m, mo);
  MUR = m; module = mo;
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  }
}

void ConfigIPC::translateFromEndcap(int disk, unsigned int ring, unsigned int number,
                                    unsigned int &MUR, unsigned int &module)
{
  try{
  unsigned long m, mo;
  handle->translateFromEndcap(disk, ring, number, m, mo);
  MUR = m; module = mo;
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  }
  
}

void ConfigIPC::translateFromPowerSupply(unsigned int partition, 
                                         unsigned int crate, unsigned int channel, 
                                         unsigned int &MUR, unsigned int &module)
{
  try{
  unsigned long m, mo;
  handle->translateFromPowerSupply(partition, crate, channel, m, mo);
  MUR = m; module = mo; 
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  }
}

void ConfigIPC::configureModuleFromStructure(const std::string module, const ABCDModule conf)
{
  try{
  sctConf::ABCDModule config (copyABCDModuleToCorba(conf));
  handle->configureModuleFromStructure(module.c_str(), config);
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  }
  
}

void ConfigIPC::configureModuleFromFile(const std::string filename)
{
  try{
  handle->configureModuleFromFile(filename.c_str());
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  }
}

void ConfigIPC::configureROD(unsigned int partition, unsigned int crate, unsigned int rod, RodConfig conf)
{
  try{
  auto_ptr<sctConf::RodConfig> config (copyRodConfigToCorba(conf));
  handle->configureROD(partition, crate, rod, *config);
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  }
}

void ConfigIPC::configureTIM(unsigned int partition, unsigned int crate, TimConfig conf)
{
  try{
  sctConf::TimConfig config = copyTimConfigToCorba(conf);

  handle->configureTIM(partition, crate, config);  
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  }
}

void ConfigIPC::configureBOC(unsigned int partition, unsigned int crate, unsigned int rod, const BOCGlobalConfig &conf)
{
  try{
  sctConf::BOCGlobalConfig config = copyBOCGlobalConfigToCorba(conf);
  handle->configureBOC(partition, crate, rod, config);
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  }
}

void ConfigIPC::clearAll()
{
  try{
  handle->clearAll();
  
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  }
}

void ConfigIPC::namePartition(unsigned int partition, const std::string name)
{
  try{
  handle->namePartition(partition, name.c_str()); 
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  }
}

void ConfigIPC::configureBOCChannel(unsigned int MUR, unsigned int position, const BOCChannelConfig &conf)
{
  try{
  auto_ptr<sctConf::BOCChannelConfig> config (copyBOCChannelConfigToCorba(conf));
  handle->configureBOCChannel(MUR, position, *config);
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  }
}

void ConfigIPC::mapModuleMUR(unsigned int MUR, unsigned int order, unsigned int RMUR, unsigned int rorder, std::string number)
{
  try{
  handle->mapModuleMUR(MUR, order, RMUR, rorder, number.c_str());
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  }
}

void ConfigIPC::unmapModuleMUR(unsigned int MUR, unsigned int order)
{
  try{
  handle->unmapModuleMUR(MUR, order); 
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  }
}

void ConfigIPC::mapRODMUR(unsigned int partition, unsigned int crate, unsigned int rod, unsigned int order, unsigned int number)
{
  try{
  handle->mapRODMUR(partition, crate, rod, order, number); 
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  }
}

void ConfigIPC::unmapRODMUR(unsigned int MUR)
{
  try{
  handle->unmapRODMUR(MUR);
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  }
}

void ConfigIPC::mapBarrelMUR(unsigned int MUR, unsigned int barrel, unsigned int row, int position)
{
  try{
  handle->mapBarrelMUR(MUR, barrel, row, position);
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  }
}

void ConfigIPC::unmapBarrelMUR(unsigned int MUR)
{
  try{
  handle->unmapBarrelMUR(MUR);
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  }
}

void ConfigIPC::mapEndcapMUR(unsigned int MUR, int disk, unsigned int quadrant, unsigned int position)
{
  try{
  handle->mapEndcapMUR(MUR, disk, quadrant, position);
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  }
}

void ConfigIPC::unmapEndcapMUR(unsigned int MUR)
{
  try{
  handle->unmapEndcapMUR(MUR);
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  }
}

void ConfigIPC::getMapMURROD(unsigned int number, unsigned int &partition, unsigned int &crate, unsigned int &rod, unsigned int &order)
{
  try{
  unsigned long p, c, r, o;
  handle->getMapMURROD(number, p, c, r, o);
  partition = p; crate = c; rod = r; order = o;
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  }
}

void ConfigIPC::getMapRODMUR(unsigned int partition, unsigned int crate, unsigned int rod, int unsigned order, unsigned int &MUR)
{
  try{
  unsigned long m;
  handle->getMapRODMUR(partition, crate, rod, order, m);
  MUR = m;  
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  }
}

void ConfigIPC::getMapMURPhysical(unsigned int MUR, MURType &type, int &plane, int &section, int &position)
{
  try{
  unsigned long pl, se, ps;
  handle->getMapMURPhysical(type, pl, se, ps);
  plane = pl; section = se; position = ps;  
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  }
}

void ConfigIPC::swapMURNames(unsigned int MUR1, unsigned int MUR2)
{
  try{
  handle->swapMURNames(MUR1, MUR2);
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  }
}

void ConfigIPC::mapPowerChannel(unsigned int MUR, unsigned int number, unsigned int partition, unsigned int crate, unsigned int channel)
{
  try{
  handle->mapPowerChannel(MUR, number, partition, crate, channel);
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  }
}

void ConfigIPC::unmapPowerChannel(unsigned int MUR, unsigned int number)
{
  try{
  handle->unmapPowerChannel(MUR, number);
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  }
}

  void ConfigIPC::modifyDefaultPowerParam(std::string state, std::string name, std::string type, float value) {
    try{
      handle->modifyDefaultPowerParam(state.c_str(), name.c_str(), type.c_str(), value); 
    }catch(sctConf::ConfigurationException &e){
      throw convertException(e);
    }
  };
  void ConfigIPC::modifyHVCardParam(unsigned int crate, unsigned int card,
                                    std::string name, std::string type, float value) {
    try{
      handle->modifyHVCardParam(crate, card, name.c_str(), type.c_str(), value); 
    }catch(sctConf::ConfigurationException &e){
      throw convertException(e);
    }
  };
  void ConfigIPC::modifyLVCardParam(unsigned int crate, unsigned int card,
                                    std::string name, std::string type, float value) {
    try{
      handle->modifyLVCardParam(crate, card, name.c_str(), type.c_str(), value); 
    }catch(sctConf::ConfigurationException &e){
      throw convertException(e);
    }
  };
  void ConfigIPC::modifyPowerParam(unsigned int MUR, unsigned int number,
                                   std::string state, std::string name, std::string type, float value) {
    try{
      handle->modifyPowerParam(MUR, number, state.c_str(), name.c_str(), type.c_str(), value); 
    }catch(sctConf::ConfigurationException &e){
      throw convertException(e);
    }
  };
  void ConfigIPC::modifyPowerParamCC(unsigned int crate, unsigned int channel,
                                     std::string state, std::string name, std::string type, float value) {
    try{
      handle->modifyPowerParamCC(crate, channel, state.c_str(), name.c_str(), type.c_str(), value); 
    }catch(sctConf::ConfigurationException &e){
      throw convertException(e);
    }
  };

  float ConfigIPC::getDefaultPowerParam(std::string state, std::string name, std::string type) {
    try{
      const float result = handle->getDefaultPowerParam(state.c_str(), name.c_str(), type.c_str());
      return result;
    }catch(sctConf::ConfigurationException &e){
      throw convertException(e);
    }
  }
  float ConfigIPC::getHVCardParam(unsigned int crate, unsigned int card,
                                  std::string name, std::string type) {
    try{
      const float result = handle->getHVCardParam(crate, card, name.c_str(), type.c_str());
      return result;
    }catch(sctConf::ConfigurationException &e){
      throw convertException(e);
    }
  }
  float ConfigIPC::getLVCardParam(unsigned int crate, unsigned int card,
                                  std::string name, std::string type) {
    try{
      const float result = handle->getLVCardParam(crate, card, name.c_str(), type.c_str());
      return result;
    }catch(sctConf::ConfigurationException &e){
      throw convertException(e);
    }
  }
  float ConfigIPC::getPowerParam(unsigned int MUR, unsigned int number,
                                 std::string state, std::string name, std::string type) {
    try{
      const float result = handle->getPowerParam(MUR, number, state.c_str(), name.c_str(), type.c_str());
      return result;
    }catch(sctConf::ConfigurationException &e){
      throw convertException(e);
    }
  }
  float ConfigIPC::getPowerParamCC(unsigned int crate, unsigned int channel,
                                   std::string state, std::string name, std::string type) {
    try{
      const float result = handle->getPowerParamCC(crate, channel, state.c_str(), name.c_str(), type.c_str());
      return result;
    }catch(sctConf::ConfigurationException &e){
      throw convertException(e);
    }
  }

std::list<std::string> ConfigIPC::listDefaultPowerParams() 
{
  try{
  auto_ptr<sctConf::NameList> myList (handle->listDefaultPowerParams());
  return copyCorbaToList<sctConf::NameList, string>(myList.get());
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  }
}

std::list<std::string> ConfigIPC::listDCSChannelParams() 
{
  try{
    auto_ptr<sctConf::NameList> myList (handle->listDCSChannelParams());
    return copyCorbaToList<sctConf::NameList, string>(myList.get());
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  }
}

std::list<std::string> ConfigIPC::listDCSCardParams()
{
  try{
    auto_ptr<sctConf::NameList> myList (handle->listDCSCardParams());
    return copyCorbaToList<sctConf::NameList, string>(myList.get());
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  }
}

std::list<std::string> ConfigIPC::listDCSCrateParams()
{
  try{
  std::list<std::string> result;
  return result;
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  }
}

void ConfigIPC::loadConfiguration(const std::string &filename)
{
  try{
    handle->loadConfiguration(filename.c_str()); 
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  }
}

void ConfigIPC::saveConfiguration(const std::string filename)
{
  try{
    handle->saveConfiguration(filename.c_str());
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  } 
}

void ConfigIPC::saveModuleConfiguration(const std::string modsn, const std::string filename)
{
  try{
  handle->saveModuleConfiguration(modsn.c_str(),filename.c_str());
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  }
}

std::string ConfigIPC::getModuleConfigurationString(const std::string modsn, const ABCDModule config) 
{
  try{
  return  handle->getModuleConfigurationString(modsn.c_str(), copyABCDModuleToCorba(config));
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  }
}

void ConfigIPC::writePowerSupplyConfiguration(std::string filename)
{
  try{
  handle->writePowerSupplyConfiguration(filename.c_str());
  }catch(sctConf::ConfigurationException &e){
    throw convertException(e);
  }
}
