package DisplayGUI;
import java.io.*;
import guiUtilities.DaveUtils;

public class RxThresholdData {
    private static RxThresholdData instance = new RxThresholdData();
    ConfigurationInterface config=null;

    java.util.regex.Pattern modulePattern = java.util.regex.Pattern.compile(".*<module\\s+id=\"(\\d+)\"\\s+group=\"\\d+\">(\\d{14})</module>.*");
    java.util.regex.Pattern murPattern = java.util.regex.Pattern.compile(".*<MUR.*id=\"(\\d+)\".*>.*");
    java.util.regex.Pattern channelIDPattern = java.util.regex.Pattern.compile(".*<channel\\s+id=\"(\\d+)\">.*");
    java.util.regex.Pattern streamDataPattern = java.util.regex.Pattern.compile("(.*<stream)(0|1)(\\s+threshold=\")([0-9a-fA-FxX]+)(\".*)");
    java.util.regex.Pattern hrefPattern = java.util.regex.Pattern.compile(".*<xi:include\\s+href=\"(.*\\.xml)\"/>.*");
    java.util.regex.Pattern commentPattern = java.util.regex.Pattern.compile("(.*)<!--(.*)");
    java.util.regex.Pattern endOfCommentPattern = java.util.regex.Pattern.compile("(.*)-->(.*)");
    java.util.regex.Pattern filePattern = java.util.regex.Pattern.compile("(.*)\\.xml");

    java.util.regex.Matcher matcher;
    java.util.Vector xmlfileList;
    java.util.Map murMap; // maps MUR id MURInfo
    java.util.Map snMap;  // maps sn to MUR
    File xmlfile;
    String errorString=null;
    String thisID=null;
    String thisMUR=null;
    
    boolean DEBUG=false;

    public static RxThresholdData getInstance() {
        return instance;
    }

   private RxThresholdData() {
      clearParams();
      }
   private void clearParams() {
      xmlfileList = new java.util.Vector();
      murMap = new java.util.HashMap();
      snMap = new java.util.HashMap();
      xmlfile=null;
      }
   public void refresh() {
      clearParams();

      // get masterXMLFile from env variable HERE
      String dataFile = System.getProperty("SCTDAQ_ROD_CONFIGURATION_PATH", null);
      if(dataFile==null) {
         errorString = new String("Env variable SCTDAQ_ROD_CONFIGURATION_PATH not passed to sctGUI. Check rungui script?");
	 xmlfile=null;
	 return;
	 }
      xmlfile = new File(dataFile);
      if(!xmlfile.exists()) {
         errorString = new String("File "+xmlfile.getAbsolutePath()+" does not exist.");
	 return;
	 }
      getlmtFileList();

      }
      
   public String getStreamFileData(String sn,int stream) {
      if(!snMap.containsKey(sn)) {
         System.err.println("XML file does not seem to contain "+sn+", but its in the current configuration!");
	 return null;
	 }
      String murID = (String)snMap.get(sn);
      if(!murMap.containsKey(murID)) {
         System.err.println("murMap does not contain mur "+murID);
	 return null;
	 }
      MURInfo murInfo = (MURInfo)murMap.get(murID);
      return murInfo.getRxThrData(sn,stream,false);
      }

      
   public String getErrorString() {
      return errorString;
      }

   private void getlmtFileList() {
      boolean comment=false;
      boolean readLines=false;
      File pathname = xmlfile.getParentFile();
      if(DEBUG) System.out.println("Opening "+xmlfile.getAbsolutePath());
      String line;
      try {
      BufferedReader in = new BufferedReader(new FileReader(xmlfile));
      while((line=in.readLine())!=null) {
      
         matcher=commentPattern.matcher(line);
	 if(matcher.matches()) {
	    String preComment = line.substring(matcher.start(1),matcher.end(1));
	    String postComment = line.substring(matcher.start(2),matcher.end(2));
	    line=preComment;
	    comment=true;
	    java.util.regex.Matcher matcher2 = endOfCommentPattern.matcher(postComment);
	    if(matcher2.matches()) {
	       comment=false;
	       line += postComment.substring(matcher2.start(2),matcher2.end(2));
	       }
	    }
	 matcher=endOfCommentPattern.matcher(line);
	 if(matcher.matches()) {
	    comment=false;
	    line=line.substring(matcher.start(2),matcher.end(2));
	    }

         if(comment) {
	   if(DEBUG) System.out.println("Comment: "+line);
	   continue;
	   }
	 if(line.matches(".*<partition.*")) readLines=true;

	 if(!readLines) continue;

	 if(DEBUG) System.out.println("line>"+line);

      	 if(line.matches(".*</partition.*")) break;

	 checkLine(line,xmlfile); // in case stream info is in master xml file

         matcher = hrefPattern.matcher(line);
	 if(matcher.matches()) {
	    if(DEBUG) System.out.println("File references "+line.substring(matcher.start(1),matcher.end(1)));
	    xmlfileList.addElement(line.substring(matcher.start(1),matcher.end(1)));
	    }
	 }
      if(DEBUG) System.out.println("out of loop");
      in.close();
      for(int i=0;i<xmlfileList.size();i++) {
	 String href =(String)xmlfileList.elementAt(i);
	 File file = new File(href);
	 if(!file.exists()) file = new File(pathname,href);
	 if(!file.exists()) {
	    System.err.println("Referenced file "+file.getAbsolutePath()+" does not exist.");
	    errorString = new String(xmlfile.getName()+" references file "+file.getAbsolutePath()+" which does not exist.");
	    continue;
	    }
	 if(DEBUG) System.out.println("Opening "+file.getAbsolutePath());
         in = new BufferedReader(new FileReader(file));
	 while((line=in.readLine())!=null) {
	    matcher=commentPattern.matcher(line);
	    if(matcher.matches()) {
	       String preComment = line.substring(matcher.start(1),matcher.end(1));
	       String postComment = line.substring(matcher.start(2),matcher.end(2));
	       line=preComment;
	       comment=true;
	       java.util.regex.Matcher matcher2 = endOfCommentPattern.matcher(postComment);
	       if(matcher2.matches()) {
	          comment=false;
	          line += postComment.substring(matcher2.start(2),matcher2.end(2));
	          }
	       }
	    matcher=endOfCommentPattern.matcher(line);
	    if(matcher.matches()) {
	      comment=false;
	      line=line.substring(matcher.start(2),matcher.end(2));
	      }

           if(comment) {
	     if(DEBUG) System.out.println("Comment: "+line);
	     continue;
	     }

	   if(DEBUG) System.out.println("line>"+line);

	    checkLine(line,file);
	    }
	 }
      in.close();
      }catch(Exception e) {javax.swing.JOptionPane.showMessageDialog(null,"Exception parsing config file(s): "+e.toString());}
   }
   
   public String getMurID(String sn) {
      return (String)snMap.get(sn);
      }
   public String getModuleID(String sn) {
      if(!snMap.containsKey(sn)) return null;
      String murID = (String)snMap.get(sn);
      if(!murMap.containsKey(murID)) return null;
      MURInfo murInfo = (MURInfo)murMap.get(murID);
      return murInfo.getModuleID(sn);
      }
      
	 
   private void checkLine(String line, File file) {

      if(DEBUG) System.out.println("checking line "+line);

      if(line.matches(".*</channel>.*")) {
         thisID=null;
	 return;
	 }
      if(line.matches(".*</MUR>.*")) {
         thisMUR=null;
	 return;
	 }
      matcher = murPattern.matcher(line);
      if(matcher.matches()) {
         thisMUR=line.substring(matcher.start(1),matcher.end(1));
	 if(DEBUG) System.out.println("MUR "+thisMUR);
	 return;
	 }

      if(thisMUR==null) return;

      MURInfo murInfo = (murMap.containsKey(thisMUR)) ? (MURInfo)murMap.get(thisMUR) : new MURInfo(thisMUR,file);

      matcher = modulePattern.matcher(line);
      if(matcher.matches()) {
         // idMap mapsmodule id to serial number
	 String id = line.substring(matcher.start(1),matcher.end(1));
	 String sn = line.substring(matcher.start(2),matcher.end(2));
	 if(DEBUG) System.out.println("mapped id "+id+" to serial no "+sn);
         murInfo.addModule(id,sn);
	 snMap.put(sn,thisMUR);
	 }
      matcher = channelIDPattern.matcher(line);
      if(matcher.matches()) thisID = line.substring(matcher.start(1),matcher.end(1));

      matcher = streamDataPattern.matcher(line);
      if(matcher.matches() && thisID!=null && murInfo.containsModule(thisID)) {
          int stream = Integer.valueOf(line.substring(matcher.start(2),matcher.end(2))).intValue();
	  murInfo.addThr(thisID,stream,line.substring(matcher.start(4),matcher.end(4)),false);
	  }
      murMap.put(thisMUR,murInfo);
   }
   
   public void updateXMLfiles(java.util.Vector tableList, int threshold) {
       int murCount=0;
       int fileCount=0;
       int lineCount=0;
       java.util.Set fileSet = new java.util.HashSet(); // Collection of Files which require modification

       for(int i=1;i<tableList.size();i++) {
          java.util.Vector theLine = (java.util.Vector)tableList.elementAt(i);


	  String sn = (String)theLine.elementAt(0);
	  Integer stream = (Integer)theLine.elementAt(1);
	  String thisMUR= (String)theLine.elementAt(2);
	  String thisID = (String)theLine.elementAt(3);
	  String newThr = (String)theLine.elementAt(4);
	  Integer diff = (Integer)theLine.elementAt(6);
	  if(DEBUG) System.out.println(sn+"-"+stream.toString()+"-"+thisMUR+"-"+thisID+"-"+newThr+"-"+diff);
	  if(!murMap.containsKey(thisMUR)) {
	     System.err.println("No key for MUR "+thisMUR);
	     continue;
	     }
	  if(Math.abs(diff.intValue())>threshold) {
	     MURInfo murInfo = (MURInfo) murMap.get(thisMUR);
             murInfo.addThr(thisID,stream.intValue(),newThr,true);
	     fileSet.add(murInfo.getXMLFile());
	     }
	  }
	  
      // Now loop through MURs
         boolean fileError=false;
         for (java.util.Iterator i=fileSet.iterator(); i.hasNext(); ) {   // iterate over murMap to check with files need updating
	     File murFile = (File)i.next();
	     
	     fileCount++;
	     
	     // rename xxxx.xml to xxxx_original.xml
	     try {
	     String original_file_name = murFile.getAbsolutePath();
	     File original_file_backup=null;
	     matcher = filePattern.matcher(original_file_name);
	     if(matcher.matches()) {
	       String fname = original_file_name.substring(matcher.start(1),matcher.end(1));
	       fname += "_original.xml";
	       original_file_backup = new File(fname);
	       if(!murFile.renameTo(original_file_backup)) {
	          errorString = "FAILED to rename mur file from "+original_file_name+" to "+fname;
		  }
                else System.out.println("Copied "+original_file_name+" to "+fname);
                }
             else errorString = "file not recognised: "+original_file_name;
             if(errorString!=null) break;  // do not continue!!
	     if(original_file_backup==null) continue;
	     
	     // now read in xxx_original.xml and re-write xxxx.xml
	     BufferedReader in = new BufferedReader(new FileReader(original_file_backup));
	     BufferedWriter out = new BufferedWriter(new FileWriter(murFile));
             System.out.println("Opening "+murFile.getAbsolutePath()+" for update(s)...");
	     String line;
	     String serialno=null;
	     MURInfo murInfo=null;
	     while((line=in.readLine())!=null) {
	        // check of we are in an MUR declaration ------------
	        matcher = murPattern.matcher(line);
		if(matcher.matches()) {
		   thisMUR=line.substring(matcher.start(1),matcher.end(1));
                   if(!murMap.containsKey(thisMUR)) {
		      System.err.println("Found unrecognised MUR "+thisMUR+" in "+original_file_name);
		      errorString="Found unrecognised MUR "+thisMUR+" in "+original_file_name;
		      break;
		      }
		   murInfo = (MURInfo)murMap.get(thisMUR);
		   murCount++;
		   }
		if(line.matches(".*</MUR.*")) murInfo=null;
                //----------------------------------------------------
		//
		// check if we have an id to serial number declaration
	        matcher = channelIDPattern.matcher(line);
		if(matcher.matches()) {
		   if(murInfo==null) {
		      System.err.println("Found channel declaration outside of MUR declaration in "+original_file_name+" : "+line);
		      errorString="Found channel declaration outside of MUR declaration in "+original_file_name+" : "+line;
		      break;
		      }
		   String chanID = line.substring(matcher.start(1),matcher.end(1));
		   serialno = murInfo.getSerialno(chanID);
		   }
	        if(line.matches(".*</channel.*")) serialno=null;
		//-----------------------------------------------------

		matcher = streamDataPattern.matcher(line);
		if(matcher.matches()) {
		   if(serialno!=null) {
		     String firstPart = line.substring(matcher.start(1),matcher.end(1));
		     String streamS = line.substring(matcher.start(2),matcher.end(2));
		     String middlePart = line.substring(matcher.start(3),matcher.end(3));
		     String dataS = line.substring(matcher.start(4),matcher.end(4));
		     String lastPart = line.substring(matcher.start(5),matcher.end(5));
		     int stream = Integer.valueOf(streamS).intValue();
		     String newData = murInfo.getRxThrData(serialno,stream,true);
		     if(newData!=null) {
		        line=firstPart+streamS+middlePart+newData+lastPart;
//		        String thisDate = guiUtilities.DaveUtils.getTodaysDate();
//		        line += "  <!-- Changed "+dataS+" -> "+newData+" by sctGUI on "+thisDate+" -->";
		        System.out.println(murFile.getName()+" : "+serialno+" stream "+streamS+" "+dataS+" -> "+newData);
		        lineCount++;
		        }
		     }
		   }
		out.write(line);
		out.newLine();
  //              System.out.println("OUT: "+line);
		}
            in.close();
	    out.close();
	    System.out.println("Closed "+murFile.getAbsolutePath()+" after update(s).");
	    }catch(Exception eIO) {errorString = "Exception "+eIO.toString(); break;}

	     }
	 if(lineCount>0) javax.swing.JOptionPane.showMessageDialog(null,"Updated "+lineCount+" lines within "+murCount+" MURs in "+fileCount+" files. See console for details.");
         clearParams();

      }

}


