package DisplayGUI;
import GuiComponents.System.*;
import Sct_SctApi.*;

public class SctApiInfo {
    private static SctApiInfo instance = new SctApiInfo();
    Sct_SctApi.SctApiIPC api;
    ConfigurationInterface guiConfig=null;
    java.util.Map probeHash, countHash;
    String[] dTypes ={"0","1","2","J","E"};


    public static SctApiInfo getInstance() {
        return instance;
    }

   private SctApiInfo() {
      probeHash = new java.util.HashMap();
      countHash = new java.util.HashMap();
      }
      
   public void probe(int selectedView) {

   countHash = new java.util.HashMap();
   probeHash = new java.util.HashMap();

   api = SystemInterface.getInstance().getSctApi();
   guiConfig = ConfigurationInterface.getInstance();
   sctConf.Configuration h = SystemInterface.getInstance().getConfigurationService();
   if(h==null) {
      javax.swing.JOptionPane.showMessageDialog(null,"Configuration Service is not available.");
      return;
      }

   java.util.Map chanMap = new java.util.HashMap();
   java.util.regex.Pattern keyPattern = java.util.regex.Pattern.compile("(\\d+)\\*(\\d+)\\*(\\d+)");
   java.util.Vector chanList;
   // first loop trough all modules to determine which RODs we need to probe, and their channels
   for(java.util.Enumeration e = guiConfig.getSerialNumberMap(selectedView).elements(); e.hasMoreElements();) {
           ModuleCell thisCell = (ModuleCell) e.nextElement();
           String sn = thisCell.getSerialNo();
	   String crate = guiConfig.getSNInfo(sn,SNInfo.ROD_CRATE);
	   String rod = guiConfig.getSNInfo(sn,SNInfo.ROD_NUMBER);
	   String chan = guiConfig.getSNInfo(sn,SNInfo.ROD_CHANNEL);
	   String partition = guiConfig.getSNInfo(sn,SNInfo.PARTITION);
	   
	   String key = partition+"*"+crate+"*"+rod;
	   if(chanMap.containsKey(key)) chanList = (java.util.Vector)chanMap.get(key);
	   else chanList =  new java.util.Vector();
	   chanList.addElement(chan);
	   chanMap.put(key,chanList);
	   }
    // now chanMap is map of parition*crate*rod to vector of channels in that rod
    // iterate through chanMap
    for (java.util.Iterator i=chanMap.entrySet().iterator(); i.hasNext(); ) {
           java.util.Map.Entry e = (java.util.Map.Entry) i.next();
           String key = (String)e.getKey();
           chanList = (java.util.Vector)e.getValue();

	   java.util.regex.Matcher matcher = keyPattern.matcher(key);
	   if(!matcher.matches()) {
	      System.err.println("**ERROR no key match in SctApiInfo");
	      continue;
	      }

	   String partition = key.substring(matcher.start(1),matcher.end(1));
	   String crate = key.substring(matcher.start(2),matcher.end(2));
	   String rod = key.substring(matcher.start(3),matcher.end(3));

	   int icrate = Integer.valueOf(crate).intValue();
	   int irod = Integer.valueOf(rod).intValue();
	   int ipartition = Integer.valueOf(partition).intValue();

	   // probe rod
	   short[] results;
	   byte[] mappings;
	   try {
	     results = api.probe(ipartition, icrate, irod);
	     if(results.length!=96) {
	        System.err.println("Got "+results.length+" values from api.probe?");
	        continue;
	        }
	   // get fibre mappings for rod, in case any are set explicitly in xml file
	     mappings = h.getFibreMappings(ipartition,icrate,irod);
	 //    System.out.println("Got "+mappings.length+" mappings from getFibreMappings for rod "+irod);
	     }catch(Exception ee){System.err.println("**sctGUI Exception in SctapiInfo::probe - "+ee.toString()); continue;}
	   
	   for(int j=0;j<chanList.size();j++) {
	   	String chan = (String)chanList.elementAt(j);
	       	int ichan = Integer.valueOf(chan).intValue();

	      	LinkData linkData = new LinkData();
		
	//	System.out.println("Key "+key+" channel "+chan);
		
		for(int link=0;link<2;link++) {
		
		   int index = ichan*3 + link + 1;
		   byte mappingIndex = mappings[index];
		//   System.out.println("Key "+key+" Link "+link+" channel "+chan+" maps to "+mappingIndex);
		   if(mappingIndex<0) {  // link switched off?
	//	       System.out.println("*** mappingIndex "+mappingIndex+" for key "+key+" channel "+chan);
		       continue;
		       }
		
		   short value = (mappingIndex>=0) ? results[mappingIndex] : 48;


	      	   String thisChar = Character.toString((char)value);
	      	   if(!countHash.containsKey(thisChar)) countHash.put(thisChar, new Integer(1));
	      	   else {
	        	int thiscount = ((Integer)countHash.get(thisChar)).intValue();
			thiscount++;
			countHash.put(thisChar, new Integer(thiscount));
			}


	           Double data = new Double(5.0);
	           switch(value) {
	          	case 48:
		  		data = new Double(0.0);  // 0
		  		break;
                	case 49:
		  		data = new Double(1.0);  // 1
		  		break;
                	case 50:
		  		data = new Double(2.0);  // 2
		  		break;
      			case 69:
		  		data = new Double(4.0);  // E
		 		 break;
      			case 74:
		  		data = new Double(3.0);  // J
		  		break;
                	default:
		  	}
		    linkData.put(link,data);
		    } // link loop
		    String thisKey = partition+"*"+crate+"*"+rod+"*"+chan;
		    probeHash.put(thisKey,linkData);
		}  // chan loop
          } // rod loop
     }
	 
     public Double getProbeData(String sn, int link) {
         if(guiConfig==null) return null;
	 String crate = guiConfig.getSNInfo(sn,SNInfo.ROD_CRATE);
	 String rod = guiConfig.getSNInfo(sn,SNInfo.ROD_NUMBER);
	 String chan = guiConfig.getSNInfo(sn,SNInfo.ROD_CHANNEL);
	 String partition = guiConfig.getSNInfo(sn,SNInfo.PARTITION);
	 String key = partition+"*"+crate+"*"+rod+"*"+chan;
         if(!probeHash.containsKey(key)) return null;
	 LinkData lData = (LinkData)probeHash.get(key);
	 return lData.getData(link);
	 }
     public java.util.Vector getCountVector() {
         java.util.Vector table = new java.util.Vector();
	 java.util.Vector theLine = new java.util.Vector();
	 theLine.addElement("Result");
	 theLine.addElement("Count");
	 table.addElement(theLine);

	 for (int i=0;i<dTypes.length;i++) {
	    theLine = new java.util.Vector();
	    theLine.addElement(dTypes[i]);
	    if(!countHash.containsKey(dTypes[i])) theLine.addElement(new Integer(0));
	    else theLine.addElement((Integer)countHash.get(dTypes[i]));
	    table.addElement(theLine);
	    }
        return table;
        }
     public void checkProbe() {
        try  {
	    String txt = javax.swing.JOptionPane.showInputDialog(null,"Probe Pattern?","E");
	    txt.toUpperCase();
            if(SystemInterface.getInstance().getSctApi().checkAllModulesProbe(txt)) {
                javax.swing.JOptionPane.showMessageDialog(null, "Check modules succeeded");
            } else {
                javax.swing.JOptionPane.showMessageDialog(null, "Check modules failed, see MRS for details");
            }
        } catch(Sct_SctApi.SctApiException s) {
            javax.swing.JOptionPane.showMessageDialog(null, "Check modules aborted: " + s);
        }
    }


}

